@@ -6,18 +6,21 @@
 # Authors: CR Oldham, Shane Lee
 # Date: December 2015
 #
-# Description: This script sets up a build environment for salt on macOS.
+# Description: This script sets up a build environment for Salt on macOS.
 #
 # Requirements:
 #     - XCode Command Line Tools (xcode-select --install)
 #
 # Usage:
-#     This script is not passed any parameters
+#     This script can be passed 1 parameter
+#       $1 : <python version> : the version of Python to use for the
+#                               build environment. Default is 2
 #
 #     Example:
-#         The following will set up a build environment for salt on macOS
+#         The following will set up a Python 3 build environment for Salt
+#         on macOS
 #
-#         ./dev_env.sh
+#         ./dev_env.sh 3
 #
 ############################################################################
 
@@ -31,6 +34,15 @@ quit_on_error() {
     exit -1
 }
 
+############################################################################
+# Check passed parameters, set defaults
+############################################################################
+if [ "$1" == "" ]; then
+    PYVER=2
+else
+    PYVER=$1
+fi
+
 ############################################################################
 # Parameters Required for the script to function properly
 ############################################################################
@@ -45,6 +57,15 @@ SHADIR=$SCRIPTDIR/shasums
 PKG_CONFIG_PATH=/opt/salt/lib/pkgconfig
 CFLAGS="-I/opt/salt/include"
 LDFLAGS="-L/opt/salt/lib"
+if [ "$PYVER" == "2" ]; then
+    PYDIR=/opt/salt/lib/python2.7
+    PYTHON=/opt/salt/bin/python
+    PIP=/opt/salt/bin/pip
+else
+    PYDIR=/opt/salt/lib/python3.5
+    PYTHON=/opt/salt/bin/python3
+    PIP=/opt/salt/bin/pip3
+fi
 
 ############################################################################
 # Determine Which XCode is being used (XCode or XCode Command Line Tools)
@@ -121,8 +142,8 @@ BUILDDIR=$SCRIPTDIR/build
 ############################################################################
 echo -n -e "\033]0;Build_Env: pkg-config\007"
 
-PKGURL="http://pkgconfig.freedesktop.org/releases/pkg-config-0.29.tar.gz"
-PKGDIR="pkg-config-0.29"
+PKGURL="http://pkgconfig.freedesktop.org/releases/pkg-config-0.29.2.tar.gz"
+PKGDIR="pkg-config-0.29.2"
 
 download $PKGURL
 
@@ -140,8 +161,8 @@ sudo -H $MAKE install
 ############################################################################
 echo -n -e "\033]0;Build_Env: libsodium\007"
 
-PKGURL="https://download.libsodium.org/libsodium/releases/old/libsodium-1.0.12.tar.gz"
-PKGDIR="libsodium-1.0.12"
+PKGURL="https://download.libsodium.org/libsodium/releases/old/libsodium-1.0.13.tar.gz"
+PKGDIR="libsodium-1.0.13"
 
 download $PKGURL
 
@@ -159,8 +180,8 @@ sudo -H $MAKE install
 ############################################################################
 echo -n -e "\033]0;Build_Env: zeromq\007"
 
-PKGURL="http://download.zeromq.org/zeromq-4.1.3.tar.gz"
-PKGDIR="zeromq-4.1.3"
+PKGURL="http://download.zeromq.org/zeromq-4.1.4.tar.gz"
+PKGDIR="zeromq-4.1.4"
 
 download $PKGURL
 
@@ -178,13 +199,13 @@ sudo -H $MAKE install
 ############################################################################
 echo -n -e "\033]0;Build_Env: OpenSSL\007"
 
-PKGURL="http://openssl.org/source/openssl-1.0.2f.tar.gz"
-PKGDIR="openssl-1.0.2f"
+PKGURL="http://openssl.org/source/openssl-1.0.2p.tar.gz"
+PKGDIR="openssl-1.0.2p"
 
 download $PKGURL
 
 echo "################################################################################"
-echo "Building OpenSSL 1.0.2f"
+echo "Building OpenSSL"
 echo "################################################################################"
 cd $PKGDIR
 ./Configure darwin64-x86_64-cc --prefix=/opt/salt --openssldir=/opt/salt/openssl
@@ -197,13 +218,18 @@ sudo -H $MAKE install
 ############################################################################
 echo -n -e "\033]0;Build_Env: Python\007"
 
-PKGURL="https://www.python.org/ftp/python/2.7.12/Python-2.7.12.tar.xz"
-PKGDIR="Python-2.7.12"
+if [ "$PYVER" == "2" ]; then
+    PKGURL="https://www.python.org/ftp/python/2.7.13/Python-2.7.13.tar.xz"
+    PKGDIR="Python-2.7.13"
+else
+    PKGURL="https://www.python.org/ftp/python/3.5.3/Python-3.5.3.tar.xz"
+    PKGDIR="Python-3.5.3"
+fi
 
 download $PKGURL
 
 echo "################################################################################"
-echo "Building Python 2.7.12"
+echo "Building Python"
 echo "################################################################################"
 echo "Note there are some test failures"
 cd $PKGDIR
@@ -215,7 +241,7 @@ sudo -H $MAKE install
 ############################################################################
 # upgrade pip
 ############################################################################
-sudo -H /opt/salt/bin/pip install --upgrade pip
+sudo -H $PIP install --upgrade pip
 
 ############################################################################
 # Download and install salt python dependencies
@@ -227,23 +253,23 @@ cd $BUILDDIR
 echo "################################################################################"
 echo "Installing Salt Dependencies with pip (normal)"
 echo "################################################################################"
-sudo -H /opt/salt/bin/pip install \
-                          -r $SRCDIR/pkg/osx/req.txt \
-                          --no-cache-dir
+sudo -H $PIP install \
+     -r $SRCDIR/pkg/osx/req.txt \
+     --no-cache-dir
 
 echo "################################################################################"
 echo "Installing Salt Dependencies with pip (build_ext)"
 echo "################################################################################"
-sudo -H /opt/salt/bin/pip install \
-                          -r $SRCDIR/pkg/osx/req_ext.txt \
-                          --global-option=build_ext \
-                          --global-option="-I/opt/salt/include" \
-                          --no-cache-dir
+sudo -H $PIP install \
+     -r $SRCDIR/pkg/osx/req_ext.txt \
+     --global-option=build_ext \
+     --global-option="-I/opt/salt/include" \
+     --no-cache-dir
 
 echo "--------------------------------------------------------------------------------"
 echo "Create Symlink to certifi for openssl"
 echo "--------------------------------------------------------------------------------"
-sudo ln -s /opt/salt/lib/python2.7/site-packages/certifi/cacert.pem /opt/salt/openssl/cert.pem
+sudo ln -s $PYDIR/site-packages/certifi/cacert.pem /opt/salt/openssl/cert.pem
 
 echo -n -e "\033]0;Build_Env: Finished\007"
 
