@@ -4,6 +4,7 @@ Create ssh executor system
 '''
 from __future__ import absolute_import
 # Import python libs
+import logging
 import os
 import tarfile
 import tempfile
@@ -15,12 +16,16 @@ from contextlib import closing
 import salt.client.ssh.shell
 import salt.client.ssh
 import salt.utils
+import salt.utils.files
 import salt.utils.thin
+import salt.utils.url
 import salt.roster
 import salt.state
 import salt.loader
 import salt.minion
 
+log = logging.getLogger(__name__)
+
 
 class SSHState(salt.state.State):
     '''
@@ -30,13 +35,15 @@ class SSHState(salt.state.State):
         self.wrapper = wrapper
         super(SSHState, self).__init__(opts, pillar)
 
-    def load_modules(self, data=None):
+    def load_modules(self, data=None, proxy=None):
         '''
         Load up the modules for remote compilation via ssh
         '''
         self.functions = self.wrapper
-        locals_ = salt.loader.minion_mods(self.opts)
-        self.states = salt.loader.states(self.opts, locals_)
+        self.utils = salt.loader.utils(self.opts)
+        self.serializers = salt.loader.serializers(self.opts)
+        locals_ = salt.loader.minion_mods(self.opts, utils=self.utils)
+        self.states = salt.loader.states(self.opts, locals_, self.utils, self.serializers)
         self.rend = salt.loader.render(self.opts, self.functions)
 
     def check_refresh(self, data, ret):
@@ -63,6 +70,13 @@ class SSHHighState(salt.state.BaseHighState):
         salt.state.BaseHighState.__init__(self, opts)
         self.state = SSHState(opts, pillar, wrapper)
         self.matcher = salt.minion.Matcher(self.opts)
+        self.tops = salt.loader.tops(self.opts)
+
+        self._pydsl_all_decls = {}
+        self._pydsl_render_stack = []
+
+    def push_active(self):
+        salt.state.HighState.stack.append(self)
 
     def load_dynamic(self, matches):
         '''
@@ -70,6 +84,37 @@ class SSHHighState(salt.state.BaseHighState):
         '''
         return
 
+    def _ext_nodes(self):
+        '''
+        Evaluate master_tops locally
+        '''
+        if 'id' not in self.opts:
+            log.error('Received call for external nodes without an id')
+            return {}
+        if not salt.utils.verify.valid_id(self.opts, self.opts['id']):
+            return {}
+        # Evaluate all configured master_tops interfaces
+
+        grains = {}
+        ret = {}
+
+        if 'grains' in self.opts:
+            grains = self.opts['grains']
+        for fun in self.tops:
+            if fun not in self.opts.get('master_tops', {}):
+                continue
+            try:
+                ret.update(self.tops[fun](opts=self.opts, grains=grains))
+            except Exception as exc:
+                # If anything happens in the top generation, log it and move on
+                log.error(
+                    'Top function {0} failed with error {1} for minion '
+                    '{2}'.format(
+                        fun, exc, self.opts['id']
+                    )
+                )
+        return ret
+
 
 def lowstate_file_refs(chunks, extras=''):
     '''
@@ -125,33 +170,47 @@ def prep_trans_tar(file_client, chunks, file_refs, pillar=None, id_=None):
     data structure
     '''
     gendir = tempfile.mkdtemp()
-    trans_tar = salt.utils.mkstemp()
+    trans_tar = salt.utils.files.mkstemp()
     lowfn = os.path.join(gendir, 'lowstate.json')
     pillarfn = os.path.join(gendir, 'pillar.json')
     sync_refs = [
-            ['salt://_modules'],
-            ['salt://_states'],
-            ['salt://_grains'],
-            ['salt://_renderers'],
-            ['salt://_returners'],
-            ['salt://_output'],
-            ['salt://_utils'],
+            [salt.utils.url.create('_modules')],
+            [salt.utils.url.create('_states')],
+            [salt.utils.url.create('_grains')],
+            [salt.utils.url.create('_renderers')],
+            [salt.utils.url.create('_returners')],
+            [salt.utils.url.create('_output')],
+            [salt.utils.url.create('_utils')],
             ]
     with salt.utils.fopen(lowfn, 'w+') as fp_:
         fp_.write(json.dumps(chunks))
     if pillar:
         with salt.utils.fopen(pillarfn, 'w+') as fp_:
             fp_.write(json.dumps(pillar))
-    cachedir = os.path.join('salt-ssh', id_)
+
+    if id_ is None:
+        id_ = ''
+    try:
+        cachedir = os.path.join('salt-ssh', id_).rstrip(os.sep)
+    except AttributeError:
+        # Minion ID should always be a str, but don't let an int break this
+        cachedir = os.path.join('salt-ssh', str(id_)).rstrip(os.sep)
+
     for saltenv in file_refs:
+        # Location where files in this saltenv will be cached
+        cache_dest_root = os.path.join(cachedir, 'files', saltenv)
         file_refs[saltenv].extend(sync_refs)
         env_root = os.path.join(gendir, saltenv)
         if not os.path.isdir(env_root):
             os.makedirs(env_root)
         for ref in file_refs[saltenv]:
             for name in ref:
-                short = name[7:]
-                path = file_client.cache_file(name, saltenv, cachedir=cachedir)
+                short = salt.utils.url.parse(name)[0].lstrip('/')
+                cache_dest = os.path.join(cache_dest_root, short)
+                try:
+                    path = file_client.cache_file(name, saltenv, cachedir=cachedir)
+                except IOError:
+                    path = ''
                 if path:
                     tgt = os.path.join(env_root, short)
                     tgt_dir = os.path.dirname(tgt)
@@ -159,12 +218,13 @@ def prep_trans_tar(file_client, chunks, file_refs, pillar=None, id_=None):
                         os.makedirs(tgt_dir)
                     shutil.copy(path, tgt)
                     continue
-                files = file_client.cache_dir(name, saltenv, cachedir=cachedir)
+                try:
+                    files = file_client.cache_dir(name, saltenv, cachedir=cachedir)
+                except IOError:
+                    files = ''
                 if files:
                     for filename in files:
-                        fn = filename[filename.find(short) + len(short):]
-                        if fn.startswith('/'):
-                            fn = fn.strip('/')
+                        fn = filename[len(file_client.get_cachedir(cache_dest)):].strip('/')
                         tgt = os.path.join(
                                 env_root,
                                 short,
@@ -175,7 +235,8 @@ def prep_trans_tar(file_client, chunks, file_refs, pillar=None, id_=None):
                             os.makedirs(tgt_dir)
                         shutil.copy(filename, tgt)
                     continue
-    try:  # cwd may not exist if it was removed but salt was run from it
+    try:
+        # cwd may not exist if it was removed but salt was run from it
         cwd = os.getcwd()
     except OSError:
         cwd = None
