@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
+    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
 
 
     salt.log.mixins
@@ -17,6 +17,15 @@ import sys
 import logging
 
 
+class LoggingProfileMixIn(object):
+    '''
+    Simple mix-in class to add a trace method to python's logging.
+    '''
+
+    def profile(self, msg, *args, **kwargs):
+        self.log(getattr(logging, 'PROFILE', 15), msg, *args, **kwargs)
+
+
 class LoggingTraceMixIn(object):
     '''
     Simple mix-in class to add a trace method to python's logging.
@@ -45,7 +54,7 @@ class LoggingMixInMeta(type):
     the bases.
     '''
     def __new__(mcs, name, bases, attrs):
-        include_trace = include_garbage = True
+        include_profile = include_trace = include_garbage = True
         bases = list(bases)
         if name == 'SaltLoggingClass':
             for base in bases:
@@ -53,6 +62,8 @@ class LoggingMixInMeta(type):
                     include_trace = False
                 if hasattr(base, 'garbage'):
                     include_garbage = False
+        if include_profile:
+            bases.append(LoggingProfileMixIn)
         if include_trace:
             bases.append(LoggingTraceMixIn)
         if include_garbage:
@@ -73,7 +84,7 @@ class NewStyleClassMixIn(object):
 
 class ExcInfoOnLogLevelFormatMixIn(object):
     '''
-    Logging handler class mixin to properly handle including exc_info on a pre logging handler basis
+    Logging handler class mixin to properly handle including exc_info on a per logging handler basis
     '''
 
     def format(self, record):
@@ -82,7 +93,8 @@ class ExcInfoOnLogLevelFormatMixIn(object):
         '''
         formatted_record = super(ExcInfoOnLogLevelFormatMixIn, self).format(record)
         exc_info_on_loglevel = getattr(record, 'exc_info_on_loglevel', None)
-        if exc_info_on_loglevel is None:
+        exc_info_on_loglevel_formatted = getattr(record, 'exc_info_on_loglevel_formatted', None)
+        if exc_info_on_loglevel is None and exc_info_on_loglevel_formatted is None:
             return formatted_record
 
         # If we reached this far it means the log record was created with exc_info_on_loglevel
@@ -93,7 +105,7 @@ class ExcInfoOnLogLevelFormatMixIn(object):
             return formatted_record
 
         # If we reached this far it means we should include exc_info
-        if not record.exc_info_on_loglevel_instance:
+        if not record.exc_info_on_loglevel_instance and not exc_info_on_loglevel_formatted:
             # This should actually never occur
             return formatted_record
 
@@ -122,5 +134,8 @@ class ExcInfoOnLogLevelFormatMixIn(object):
             #     for a script. See issue 13232.
             formatted_record += record.exc_info_on_loglevel_formatted.decode(sys.getfilesystemencoding(),
                                                                              'replace')
-
+        # Reset the record.exc_info_on_loglevel_instance because it might need
+        # to "travel" through a multiprocessing process and it might contain
+        # data which is not pickle'able
+        record.exc_info_on_loglevel_instance = None
         return formatted_record
