@@ -544,6 +544,10 @@ def associate_vpc_with_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
             r = conn.associate_vpc_with_hosted_zone(**args)
             return _wait_for_sync(r['ChangeInfo']['Id'], conn)
         except ClientError as e:
+            if e.response.get('Error', {}).get('Code') == 'ConflictingDomainExists':
+                log.debug('VPC Association already exists.')
+                # return True since the current state is the desired one
+                return True
             if tries and e.response.get('Error', {}).get('Code') == 'Throttling':
                 log.debug('Throttled by AWS API.')
                 time.sleep(3)
@@ -636,6 +640,10 @@ def disassociate_vpc_from_hosted_zone(HostedZoneId=None, Name=None, VPCId=None,
             r = conn.disassociate_vpc_from_hosted_zone(**args)
             return _wait_for_sync(r['ChangeInfo']['Id'], conn)
         except ClientError as e:
+            if e.response.get('Error', {}).get('Code') == 'VPCAssociationNotFound':
+                log.debug('No VPC Association exists.')
+                # return True since the current state is the desired one
+                return True
             if tries and e.response.get('Error', {}).get('Code') == 'Throttling':
                 log.debug('Throttled by AWS API.')
                 time.sleep(3)
@@ -787,9 +795,6 @@ def change_resource_record_sets(HostedZoneId=None, Name=None,
                                 PrivateZone=None, ChangeBatch=None,
                                 region=None, key=None, keyid=None, profile=None):
     '''
-    Ugh!!!  Not gonna try to reproduce and validatethis mess in here - just pass what we get to AWS
-    and let it decide if it's valid or not...
-
     See the `AWS Route53 API docs`__ as well as the `Boto3 documentation`__ for all the details...
 
     .. __: https://docs.aws.amazon.com/Route53/latest/APIReference/API_ChangeResourceRecordSets.html
@@ -799,41 +804,42 @@ def change_resource_record_sets(HostedZoneId=None, Name=None,
     parameters and combinations thereof are quite varied, so perusal of the above linked docs is
     highly recommended for any non-trival configurations.
 
-    .. code-block:: json
-    ChangeBatch={
-        'Comment': 'string',
-        'Changes': [
-            {
-                'Action': 'CREATE'|'DELETE'|'UPSERT',
-                'ResourceRecordSet': {
-                    'Name': 'string',
-                    'Type': 'SOA'|'A'|'TXT'|'NS'|'CNAME'|'MX'|'NAPTR'|'PTR'|'SRV'|'SPF'|'AAAA',
-                    'SetIdentifier': 'string',
-                    'Weight': 123,
-                    'Region': 'us-east-1'|'us-east-2'|'us-west-1'|'us-west-2'|'ca-central-1'|'eu-west-1'|'eu-west-2'|'eu-central-1'|'ap-southeast-1'|'ap-southeast-2'|'ap-northeast-1'|'ap-northeast-2'|'sa-east-1'|'cn-north-1'|'ap-south-1',
-                    'GeoLocation': {
-                        'ContinentCode': 'string',
-                        'CountryCode': 'string',
-                        'SubdivisionCode': 'string'
-                    },
-                    'Failover': 'PRIMARY'|'SECONDARY',
-                    'TTL': 123,
-                    'ResourceRecords': [
-                        {
-                            'Value': 'string'
+    .. code-block:: text
+
+        {
+            "Comment": "string",
+            "Changes": [
+                {
+                    "Action": "CREATE"|"DELETE"|"UPSERT",
+                    "ResourceRecordSet": {
+                        "Name": "string",
+                        "Type": "SOA"|"A"|"TXT"|"NS"|"CNAME"|"MX"|"NAPTR"|"PTR"|"SRV"|"SPF"|"AAAA",
+                        "SetIdentifier": "string",
+                        "Weight": 123,
+                        "Region": "us-east-1"|"us-east-2"|"us-west-1"|"us-west-2"|"ca-central-1"|"eu-west-1"|"eu-west-2"|"eu-central-1"|"ap-southeast-1"|"ap-southeast-2"|"ap-northeast-1"|"ap-northeast-2"|"sa-east-1"|"cn-north-1"|"ap-south-1",
+                        "GeoLocation": {
+                            "ContinentCode": "string",
+                            "CountryCode": "string",
+                            "SubdivisionCode": "string"
                         },
-                    ],
-                    'AliasTarget': {
-                        'HostedZoneId': 'string',
-                        'DNSName': 'string',
-                        'EvaluateTargetHealth': True|False
-                    },
-                    'HealthCheckId': 'string',
-                    'TrafficPolicyInstanceId': 'string'
-                }
-            },
-        ]
-    }
+                        "Failover": "PRIMARY"|"SECONDARY",
+                        "TTL": 123,
+                        "ResourceRecords": [
+                            {
+                                "Value": "string"
+                            },
+                        ],
+                        "AliasTarget": {
+                            "HostedZoneId": "string",
+                            "DNSName": "string",
+                            "EvaluateTargetHealth": True|False
+                        },
+                        "HealthCheckId": "string",
+                        "TrafficPolicyInstanceId": "string"
+                    }
+                },
+            ]
+        }
 
     CLI Example:
 
