@@ -7,37 +7,35 @@ Connection module for Amazon EC2
 :configuration: This module accepts explicit EC2 credentials but can also
     utilize IAM roles assigned to the instance through Instance Profiles.
     Dynamic credentials are then automatically obtained from AWS API and no
-    further configuration is necessary. More Information available at:
+    further configuration is necessary. More Information available here__.
 
-    .. code-block:: text
+.. __: http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html
 
-        http://docs.aws.amazon.com/AWSEC2/latest/UserGuide/iam-roles-for-amazon-ec2.html
+If IAM roles are not used you need to specify them either in a pillar or
+in the minion's config file:
 
-    If IAM roles are not used you need to specify them either in a pillar or
-    in the minion's config file:
+.. code-block:: yaml
 
-    .. code-block:: yaml
+    ec2.keyid: GKTADJGHEIQSXMKKRBJ08H
+    ec2.key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
 
-        ec2.keyid: GKTADJGHEIQSXMKKRBJ08H
-        ec2.key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
+A region may also be specified in the configuration:
 
-    A region may also be specified in the configuration:
+.. code-block:: yaml
 
-    .. code-block:: yaml
+    ec2.region: us-east-1
 
-        ec2.region: us-east-1
+If a region is not specified, the default is us-east-1.
 
-    If a region is not specified, the default is us-east-1.
+It's also possible to specify key, keyid, and region via a profile, either
+as a passed in dict, or as a string to pull from pillars or minion config:
 
-    It's also possible to specify key, keyid, and region via a profile, either
-    as a passed in dict, or as a string to pull from pillars or minion config:
+.. code-block:: yaml
 
-    .. code-block:: yaml
-
-        myprofile:
-          keyid: GKTADJGHEIQSXMKKRBJ08H
-          key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
-          region: us-east-1
+    myprofile:
+      keyid: GKTADJGHEIQSXMKKRBJ08H
+      key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
+      region: us-east-1
 
 :depends: boto
 
@@ -49,13 +47,14 @@ Connection module for Amazon EC2
 from __future__ import absolute_import
 import logging
 import time
-from distutils.version import LooseVersion as _LooseVersion  # pylint: disable=import-error,no-name-in-module
+import json
 
 # Import Salt libs
 import salt.utils
 import salt.utils.compat
 import salt.ext.six as six
 from salt.exceptions import SaltInvocationError, CommandExecutionError
+from salt.utils.versions import LooseVersion as _LooseVersion
 
 # Import third party libs
 try:
@@ -63,6 +62,7 @@ try:
     import boto
     import boto.ec2
     # pylint: enable=unused-import
+    from boto.ec2.blockdevicemapping import BlockDeviceMapping, BlockDeviceType
     HAS_BOTO = True
 except ImportError:
     HAS_BOTO = False
@@ -272,8 +272,8 @@ def allocate_eip_address(domain=None, region=None, key=None, keyid=None, profile
 def release_eip_address(public_ip=None, allocation_id=None, region=None, key=None,
                         keyid=None, profile=None):
     '''
-    Free an Elastic IP address.  Pass either a public IP address to release a 'standard'
-    EC2 Elastic IP address, or an AllocationId to release a VPC Elastic IP address.
+    Free an Elastic IP address.  Pass either a public IP address to release an
+    EC2 Classic EIP, or an AllocationId to release a VPC EIP.
 
     public_ip
         (string) - The public IP address - for EC2 elastic IPs.
@@ -292,8 +292,8 @@ def release_eip_address(public_ip=None, allocation_id=None, region=None, key=Non
     .. versionadded:: 2016.3.0
     '''
     if not salt.utils.exactly_one((public_ip, allocation_id)):
-        raise SaltInvocationError('Exactly one (but not both) of \'public_ip\' '
-                                  'or \'allocation_id\' must be provided')
+        raise SaltInvocationError("Exactly one of 'public_ip' OR "
+                                  "'allocation_id' must be provided")
 
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
@@ -332,7 +332,7 @@ def associate_eip_address(instance_id=None, instance_name=None, public_ip=None,
         (bool)   – Allow a currently associated EIP to be re-associated with the new instance or interface.
 
     returns
-        (bool)   - True on success, False otherwise
+        (bool)   - True on success, False on failure.
 
     CLI Example:
 
@@ -377,9 +377,12 @@ def associate_eip_address(instance_id=None, instance_name=None, public_ip=None,
             return False
 
     try:
-        return conn.associate_address(instance_id=instance_id, public_ip=public_ip,
-              allocation_id=allocation_id, network_interface_id=network_interface_id,
-              private_ip_address=private_ip_address, allow_reassociation=allow_reassociation)
+        return conn.associate_address(instance_id=instance_id,
+                                      public_ip=public_ip,
+                                      allocation_id=allocation_id,
+                                      network_interface_id=network_interface_id,
+                                      private_ip_address=private_ip_address,
+                                      allow_reassociation=allow_reassociation)
     except boto.exception.BotoServerError as e:
         log.error(e)
         return False
@@ -390,15 +393,15 @@ def disassociate_eip_address(public_ip=None, association_id=None, region=None,
     '''
     Disassociate an Elastic IP address from a currently running instance. This
     requires exactly one of either 'association_id' or 'public_ip', depending
-    on whether you’re associating a VPC address or a plain EC2 address.
+    on whether you’re dealing with a VPC or EC2 Classic address.
 
     public_ip
-        (string) – Public IP address, for standard EC2 based allocations.
+        (string) – Public IP address, for EC2 Classic allocations.
     association_id
-        (string) – Association ID for a VPC-based EIP.
+        (string) – Association ID for a VPC-bound EIP.
 
     returns
-        (bool)   - True on success, False otherwise
+        (bool)   - True on success, False on failure.
 
     CLI Example:
 
@@ -417,6 +420,118 @@ def disassociate_eip_address(public_ip=None, association_id=None, region=None,
         return False
 
 
+def assign_private_ip_addresses(network_interface_name=None, network_interface_id=None,
+                                private_ip_addresses=None, secondary_private_ip_address_count=None,
+                                allow_reassignment=False, region=None, key=None,
+                                keyid=None, profile=None):
+    '''
+    Assigns one or more secondary private IP addresses to a network interface.
+
+    network_interface_id
+        (string) - ID of the network interface to associate the IP with (exclusive with 'network_interface_name')
+    network_interface_name
+        (string) - Name of the network interface to associate the IP with (exclusive with 'network_interface_id')
+    private_ip_addresses
+        (list) - Assigns the specified IP addresses as secondary IP addresses to the network interface (exclusive with 'secondary_private_ip_address_count')
+    secondary_private_ip_address_count
+        (int) - The number of secondary IP addresses to assign to the network interface. (exclusive with 'private_ip_addresses')
+    allow_reassociation
+        (bool)   – Allow a currently associated EIP to be re-associated with the new instance or interface.
+
+    returns
+        (bool)   - True on success, False on failure.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion boto_ec2.assign_private_ip_addresses network_interface_name=my_eni private_ip_addresses=private_ip
+        salt myminion boto_ec2.assign_private_ip_addresses network_interface_name=my_eni secondary_private_ip_address_count=2
+
+    .. versionadded:: 2017.7.0
+    '''
+    if not salt.utils.exactly_one((network_interface_name,
+                                   network_interface_id)):
+        raise SaltInvocationError("Exactly one of 'network_interface_name', "
+                                  "'network_interface_id' must be provided")
+
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    if network_interface_name:
+        try:
+            network_interface_id = get_network_interface_id(
+                network_interface_name, region=region, key=key, keyid=keyid,
+                profile=profile)
+        except boto.exception.BotoServerError as e:
+            log.error(e)
+            return False
+        if not network_interface_id:
+            log.error("Given network_interface_name '{0}' cannot be mapped to "
+                      "an network_interface_id".format(network_interface_name))
+            return False
+
+    try:
+        return conn.assign_private_ip_addresses(network_interface_id=network_interface_id,
+                                                private_ip_addresses=private_ip_addresses,
+                                                secondary_private_ip_address_count=secondary_private_ip_address_count,
+                                                allow_reassignment=allow_reassignment)
+    except boto.exception.BotoServerError as e:
+        log.error(e)
+        return False
+
+
+def unassign_private_ip_addresses(network_interface_name=None, network_interface_id=None,
+                                  private_ip_addresses=None, region=None,
+                                  key=None, keyid=None, profile=None):
+    '''
+    Unassigns one or more secondary private IP addresses from a network interface
+
+    network_interface_id
+        (string) - ID of the network interface to associate the IP with (exclusive with 'network_interface_name')
+    network_interface_name
+        (string) - Name of the network interface to associate the IP with (exclusive with 'network_interface_id')
+    private_ip_addresses
+        (list) - Assigns the specified IP addresses as secondary IP addresses to the network interface.
+
+    returns
+        (bool)   - True on success, False on failure.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt myminion boto_ec2.unassign_private_ip_addresses network_interface_name=my_eni private_ip_addresses=private_ip
+
+    .. versionadded:: 2017.7.0
+    '''
+    if not salt.utils.exactly_one((network_interface_name,
+                                   network_interface_id)):
+        raise SaltInvocationError("Exactly one of 'network_interface_name', "
+                                  "'network_interface_id' must be provided")
+
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    if network_interface_name:
+        try:
+            network_interface_id = get_network_interface_id(
+                network_interface_name, region=region, key=key, keyid=keyid,
+                profile=profile)
+        except boto.exception.BotoServerError as e:
+            log.error(e)
+            return False
+        if not network_interface_id:
+            log.error("Given network_interface_name '{0}' cannot be mapped to "
+                      "an network_interface_id".format(network_interface_name))
+            return False
+
+    try:
+        return conn.unassign_private_ip_addresses(network_interface_id=network_interface_id,
+                                                  private_ip_addresses=private_ip_addresses)
+    except boto.exception.BotoServerError as e:
+        log.error(e)
+        return False
+
+
 def get_zones(region=None, key=None, keyid=None, profile=None):
     '''
     Get a list of AZs for the configured region.
@@ -434,7 +549,7 @@ def get_zones(region=None, key=None, keyid=None, profile=None):
 
 def find_instances(instance_id=None, name=None, tags=None, region=None,
                    key=None, keyid=None, profile=None, return_objs=False,
-                   in_states=None):
+                   in_states=None, filters=None):
 
     '''
     Given instance properties, find and return matching instance ids
@@ -446,12 +561,11 @@ def find_instances(instance_id=None, name=None, tags=None, region=None,
         salt myminion boto_ec2.find_instances # Lists all instances
         salt myminion boto_ec2.find_instances name=myinstance
         salt myminion boto_ec2.find_instances tags='{"mytag": "value"}'
+        salt myminion boto_ec2.find_instances filters='{"vpc-id": "vpc-12345678"}'
 
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
 
-    if not any((instance_id, name, tags)):
-        return []
     try:
         filter_parameters = {'filters': {}}
 
@@ -465,7 +579,10 @@ def find_instances(instance_id=None, name=None, tags=None, region=None,
             for tag_name, tag_value in six.iteritems(tags):
                 filter_parameters['filters']['tag:{0}'.format(tag_name)] = tag_value
 
-        reservations = conn.get_all_instances(**filter_parameters)
+        if filters:
+            filter_parameters['filters'].update(filters)
+
+        reservations = conn.get_all_reservations(**filter_parameters)
         instances = [i for r in reservations for i in r.instances]
         log.debug('The filters criteria {0} matched the following '
                   'instances:{1}'.format(filter_parameters, instances))
@@ -487,7 +604,7 @@ def find_instances(instance_id=None, name=None, tags=None, region=None,
 
 def create_image(ami_name, instance_id=None, instance_name=None, tags=None, region=None,
                  key=None, keyid=None, profile=None, description=None, no_reboot=False,
-                 dry_run=False):
+                 dry_run=False, filters=None):
     '''
     Given instance properties that define exactly one instance, create AMI and return AMI-id.
 
@@ -502,7 +619,7 @@ def create_image(ami_name, instance_id=None, instance_name=None, tags=None, regi
 
     instances = find_instances(instance_id=instance_id, name=instance_name, tags=tags,
                                region=region, key=key, keyid=keyid, profile=profile,
-                               return_objs=True)
+                               return_objs=True, filters=filters)
 
     if not instances:
         log.error('Source instance not found')
@@ -530,7 +647,7 @@ def find_images(ami_name=None, executable_by=None, owners=None, image_ids=None,
 
     .. code-block:: bash
 
-        salt myminion boto_ec2.find_instances tags='{"mytag": "value"}'
+        salt myminion boto_ec2.find_images tags='{"mytag": "value"}'
 
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
@@ -570,7 +687,7 @@ def find_images(ami_name=None, executable_by=None, owners=None, image_ids=None,
 
 
 def terminate(instance_id=None, name=None, region=None,
-              key=None, keyid=None, profile=None):
+              key=None, keyid=None, profile=None, filters=None):
     '''
     Terminate the instance described by instance_id or name.
 
@@ -583,7 +700,8 @@ def terminate(instance_id=None, name=None, region=None,
     '''
     instances = find_instances(instance_id=instance_id, name=name,
                                region=region, key=key, keyid=keyid,
-                               profile=profile, return_objs=True)
+                               profile=profile, return_objs=True,
+                               filters=filters)
     if instances in (False, None, []):
         return instances
 
@@ -596,10 +714,10 @@ def terminate(instance_id=None, name=None, region=None,
 
 
 def get_id(name=None, tags=None, region=None, key=None,
-           keyid=None, profile=None, in_states=None):
+           keyid=None, profile=None, in_states=None, filters=None):
 
     '''
-    Given instace properties, return the instance id if it exist.
+    Given instance properties, return the instance id if it exists.
 
     CLI Example:
 
@@ -609,7 +727,8 @@ def get_id(name=None, tags=None, region=None, key=None,
 
     '''
     instance_ids = find_instances(name=name, tags=tags, region=region, key=key,
-                                  keyid=keyid, profile=profile, in_states=in_states)
+                                  keyid=keyid, profile=profile, in_states=in_states,
+                                  filters=filters)
     if instance_ids:
         log.info("Instance ids: {0}".format(" ".join(instance_ids)))
         if len(instance_ids) == 1:
@@ -623,7 +742,7 @@ def get_id(name=None, tags=None, region=None, key=None,
 
 
 def exists(instance_id=None, name=None, tags=None, region=None, key=None,
-           keyid=None, profile=None, in_states=None):
+           keyid=None, profile=None, in_states=None, filters=None):
     '''
     Given a instance id, check to see if the given instance id exists.
 
@@ -638,7 +757,7 @@ def exists(instance_id=None, name=None, tags=None, region=None, key=None,
     '''
     instances = find_instances(instance_id=instance_id, name=name, tags=tags,
                                region=region, key=key, keyid=keyid,
-                               profile=profile, in_states=in_states)
+                               profile=profile, in_states=in_states, filters=filters)
     if instances:
         log.info('Instance exists.')
         return True
@@ -647,6 +766,59 @@ def exists(instance_id=None, name=None, tags=None, region=None, key=None,
         return False
 
 
+def _to_blockdev_map(thing):
+    '''
+    Convert a string, or a json payload, or a dict in the right
+    format, into a boto.ec2.blockdevicemapping.BlockDeviceMapping as
+    needed by instance_present().  The following YAML is a direct
+    representation of what is expected by the underlying boto EC2 code.
+
+    YAML example:
+
+    .. code-block:: yaml
+        device-maps:
+            /dev/sdb:
+                ephemeral_name: ephemeral0
+            /dev/sdc:
+                ephemeral_name: ephemeral1
+            /dev/sdd:
+                ephemeral_name: ephemeral2
+            /dev/sde:
+                ephemeral_name: ephemeral3
+            /dev/sdf:
+                size: 20
+                volume_type: gp2
+
+    '''
+    if not thing:
+        return None
+    if isinstance(thing, BlockDeviceMapping):
+        return thing
+    if isinstance(thing, six.string_types):
+        thing = json.loads(thing)
+    if not isinstance(thing, dict):
+        log.error("Can't convert '{0}' of type {1} to a "
+                  "boto.ec2.blockdevicemapping.BlockDeviceMapping".format(thing, type(thing)))
+        return None
+
+    bdm = BlockDeviceMapping()
+    for d, t in six.iteritems(thing):
+        bdt = BlockDeviceType(ephemeral_name=t.get('ephemeral_name'),
+                              no_device=t.get('no_device', False),
+                              volume_id=t.get('volume_id'),
+                              snapshot_id=t.get('snapshot_id'),
+                              status=t.get('status'),
+                              attach_time=t.get('attach_time'),
+                              delete_on_termination=t.get('delete_on_termination', False),
+                              size=t.get('size'),
+                              volume_type=t.get('volume_type'),
+                              iops=t.get('iops'),
+                              encrypted=t.get('encrypted'))
+        bdm[d] = bdt
+
+    return bdm
+
+
 def run(image_id, name=None, tags=None, key_name=None, security_groups=None,
         user_data=None, instance_type='m1.small', placement=None,
         kernel_id=None, ramdisk_id=None, monitoring_enabled=None, vpc_id=None,
@@ -710,6 +882,26 @@ def run(image_id, name=None, tags=None, key_name=None, security_groups=None,
     block_device_map
         (boto.ec2.blockdevicemapping.BlockDeviceMapping) – A BlockDeviceMapping
         data structure describing the EBS volumes associated with the Image.
+        (string) - A string representation of a BlockDeviceMapping structure
+        (dict) - A dict describing a BlockDeviceMapping structure
+
+        YAML example:
+
+        .. code-block:: yaml
+
+            device-maps:
+                /dev/sdb:
+                    ephemeral_name: ephemeral0
+                /dev/sdc:
+                    ephemeral_name: ephemeral1
+                /dev/sdd:
+                    ephemeral_name: ephemeral2
+                /dev/sde:
+                    ephemeral_name: ephemeral3
+                /dev/sdf:
+                    size: 20
+                    volume_type: gp2
+
     disable_api_termination
         (bool) – If True, the instances will be locked and will not be able to
         be terminated via the API.
@@ -787,10 +979,11 @@ def run(image_id, name=None, tags=None, key_name=None, security_groups=None,
         raise SaltInvocationError('Only one of network_interface_id or '
                                   'network_interface_name may be provided.')
     if network_interface_name:
-        network_interface_id = get_network_interface_id(network_interface_name,
+        result = get_network_interface_id(network_interface_name,
                                                         region=region, key=key,
                                                         keyid=keyid,
                                                         profile=profile)
+        network_interface_id = result['result']
         if not network_interface_id:
             log.warning(
                 "Given network_interface_name '{0}' cannot be mapped to an "
@@ -816,7 +1009,8 @@ def run(image_id, name=None, tags=None, key_name=None, security_groups=None,
                                      user_data=user_data, instance_type=instance_type,
                                      placement=placement, kernel_id=kernel_id, ramdisk_id=ramdisk_id,
                                      monitoring_enabled=monitoring_enabled,
-                                     private_ip_address=private_ip_address, block_device_map=block_device_map,
+                                     private_ip_address=private_ip_address,
+                                     block_device_map=_to_blockdev_map(block_device_map),
                                      disable_api_termination=disable_api_termination,
                                      instance_initiated_shutdown_behavior=instance_initiated_shutdown_behavior,
                                      placement_group=placement_group, client_token=client_token,
@@ -975,7 +1169,7 @@ def get_keys(keynames=None, filters=None, region=None, key=None,
 
 
 def get_attribute(attribute, instance_name=None, instance_id=None, region=None, key=None,
-                  keyid=None, profile=None):
+                  keyid=None, profile=None, filters=None):
     '''
     Get an EC2 instance attribute.
 
@@ -1014,7 +1208,8 @@ def get_attribute(attribute, instance_name=None, instance_id=None, region=None,
         raise SaltInvocationError('Attribute must be one of: {0}.'.format(attribute_list))
     try:
         if instance_name:
-            instances = find_instances(name=instance_name, region=region, key=key, keyid=keyid, profile=profile)
+            instances = find_instances(name=instance_name, region=region, key=key, keyid=keyid, profile=profile,
+                                      filters=filters)
             if len(instances) > 1:
                 log.error('Found more than one EC2 instance matching the criteria.')
                 return False
@@ -1032,7 +1227,7 @@ def get_attribute(attribute, instance_name=None, instance_id=None, region=None,
 
 
 def set_attribute(attribute, attribute_value, instance_name=None, instance_id=None, region=None, key=None, keyid=None,
-                  profile=None):
+                  profile=None, filters=None):
     '''
     Set an EC2 instance attribute.
     Returns whether the operation succeeded or not.
@@ -1071,7 +1266,8 @@ def set_attribute(attribute, attribute_value, instance_name=None, instance_id=No
         raise SaltInvocationError('Attribute must be one of: {0}.'.format(attribute_list))
     try:
         if instance_name:
-            instances = find_instances(name=instance_name, region=region, key=key, keyid=keyid, profile=profile)
+            instances = find_instances(name=instance_name, region=region, key=key, keyid=keyid, profile=profile,
+                                      filters=filters)
             if len(instances) != 1:
                 raise CommandExecutionError('Found more than one EC2 instance matching the criteria.')
             instance_id = instances[0]
@@ -1455,3 +1651,326 @@ def modify_network_interface_attribute(
     except boto.exception.EC2ResponseError as e:
         r['error'] = __utils__['boto.get_error'](e)
     return r
+
+
+def get_all_volumes(volume_ids=None, filters=None, return_objs=False,
+                    region=None, key=None, keyid=None, profile=None):
+    '''
+    Get a list of all EBS volumes, optionally filtered by provided 'filters' param
+
+    .. versionadded:: 2016.11.0
+
+    volume_ids
+        (list) - Optional list of volume_ids.  If provided, only the volumes
+        associated with those in the list will be returned.
+    filters
+        (dict) - Additional constraints on which volumes to return.  Valid filters are:
+
+    - attachment.attach-time - The time stamp when the attachment initiated.
+    - attachment.delete-on-termination - Whether the volume is deleted on instance termination.
+    - attachment.device - The device name that is exposed to the instance (for example, /dev/sda1).
+    - attachment.instance-id - The ID of the instance the volume is attached to.
+    - attachment.status - The attachment state (attaching | attached | detaching | detached).
+    - availability-zone - The Availability Zone in which the volume was created.
+    - create-time - The time stamp when the volume was created.
+    - encrypted - The encryption status of the volume.
+    - size - The size of the volume, in GiB.
+    - snapshot-id - The snapshot from which the volume was created.
+    - status - The status of the volume (creating | available | in-use | deleting | deleted | error).
+    - tag:key=value - The key/value combination of a tag assigned to the resource.
+    - volume-id - The volume ID.
+    - volume-type - The Amazon EBS volume type. This can be ``gp2`` for General
+      Purpose SSD, ``io1`` for Provisioned IOPS SSD, ``st1`` for Throughput
+      Optimized HDD, ``sc1`` for Cold HDD, or ``standard`` for Magnetic volumes.
+
+    return_objs
+        (bool) - Changes the return type from list of volume IDs to list of
+        boto.ec2.volume.Volume objects
+
+    returns
+        (list) - A list of the requested values: Either the volume IDs or, if
+        return_objs is ``True``, boto.ec2.volume.Volume objects.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt-call boto_ec2.get_all_volumes filters='{"tag:Name": "myVolume01"}'
+
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+
+    try:
+        ret = conn.get_all_volumes(volume_ids=volume_ids, filters=filters)
+        return ret if return_objs else [r.id for r in ret]
+    except boto.exception.BotoServerError as e:
+        log.error(e)
+        return []
+
+
+def set_volumes_tags(tag_maps, authoritative=False, dry_run=False,
+                    region=None, key=None, keyid=None, profile=None):
+    '''
+    .. versionadded:: 2016.11.0
+
+    tag_maps (list)
+        List of dicts of filters and tags, where 'filters' is a dict suitable for passing to the
+        'filters' argument of get_all_volumes() above, and 'tags' is a dict of tags to be set on
+        volumes (via create_tags/delete_tags) as matched by the given filters.  The filter syntax
+        is extended to permit passing either a list of volume_ids or an instance_name (with
+        instance_name being the Name tag of the instance to which the desired volumes are mapped).
+        Each mapping in the list is applied separately, so multiple sets of volumes can be all
+        tagged differently with one call to this function.  If filtering by instance Name, You may
+        additionally limit the instances matched by passing in a list of desired instance states.
+        The default set of states is ('pending', 'rebooting', 'running', 'stopping', 'stopped').
+
+    YAML example fragment:
+
+    .. code-block:: yaml
+
+        - filters:
+            attachment.instance_id: i-abcdef12
+          tags:
+            Name: dev-int-abcdef12.aws-foo.com
+        - filters:
+            attachment.device: /dev/sdf
+          tags:
+            ManagedSnapshots: true
+            BillingGroup: bubba.hotep@aws-foo.com
+          in_states:
+          - stopped
+          - terminated
+        - filters:
+            instance_name: prd-foo-01.aws-foo.com
+          tags:
+            Name: prd-foo-01.aws-foo.com
+            BillingGroup: infra-team@aws-foo.com
+        - filters:
+            volume_ids: [ vol-12345689, vol-abcdef12 ]
+          tags:
+            BillingGroup: infra-team@aws-foo.com
+
+    authoritative (bool)
+        If true, any existing tags on the matched volumes, and not explicitly requested here, will
+        be removed.
+
+    dry_run (bool)
+        If true, don't change anything, just return a dictionary describing any changes which
+        would have been applied.
+
+    returns (dict)
+        A dict dsecribing status and any changes.
+
+    '''
+    ret = {'success': True, 'comment': '', 'changes': {}}
+    running_states = ('pending', 'rebooting', 'running', 'stopping', 'stopped')
+    for tm in tag_maps:
+        filters = tm.get('filters')
+        tags = tm.get('tags')
+        if not isinstance(filters, dict):
+            raise SaltInvocationError('Tag filters must be a dictionary: got {0}'.format(filters))
+        if not isinstance(tags, dict):
+            raise SaltInvocationError('Tags must be a dictionary: got {0}'.format(tags))
+        args = {'return_objs': True, 'region': region, 'key': key, 'keyid': keyid, 'profile': profile}
+        new_filters = {}
+        log.debug('got filters: {0}'.format(filters))
+        instance_id = None
+        in_states = tm.get('in_states', running_states)
+        try:
+            for k, v in six.iteritems(filters):
+                if k == 'volume_ids':
+                    args['volume_ids'] = v
+                elif k == 'instance_name':
+                    instance_id = get_id(name=v, in_states=in_states, region=region, key=key,
+                                         keyid=keyid, profile=profile)
+                    if not instance_id:
+                        msg = "Couldn't resolve instance Name {0} to an ID.".format(v)
+                        raise CommandExecutionError(msg)
+                    new_filters['attachment.instance_id'] = instance_id
+                else:
+                    new_filters[k] = v
+        except CommandExecutionError as e:
+            log.warning(e)
+            continue  # Hmme, abort or do what we can...?  Guess the latter for now.
+        args['filters'] = new_filters
+        volumes = get_all_volumes(**args)
+        log.debug('got volume list: {0}'.format(volumes))
+        changes = {'old': {}, 'new': {}}
+        for vol in volumes:
+            log.debug('current tags on vol.id {0}: {1}'.format(vol.id, dict(getattr(vol, 'tags', {}))))
+            curr = set(dict(getattr(vol, 'tags', {})).keys())
+            log.debug('requested tags on vol.id {0}: {1}'.format(vol.id, tags))
+            req = set(tags.keys())
+            add = list(req - curr)
+            update = [r for r in (req & curr) if vol.tags[r] != tags[r]]
+            remove = list(curr - req)
+            if add or update or (authoritative and remove):
+                changes['old'][vol.id] = dict(getattr(vol, 'tags', {}))
+                changes['new'][vol.id] = tags
+            else:
+                log.debug('No changes needed for vol.id {0}'.format(vol.id))
+            if len(add):
+                d = dict((k, tags[k]) for k in add)
+                log.debug('New tags for vol.id {0}: {1}'.format(vol.id, d))
+            if len(update):
+                d = dict((k, tags[k]) for k in update)
+                log.debug('Updated tags for vol.id {0}: {1}'.format(vol.id, d))
+            if not dry_run:
+                if not create_tags(vol.id, tags, region=region, key=key, keyid=keyid, profile=profile):
+                    ret['success'] = False
+                    ret['comment'] = "Failed to set tags on vol.id {0}: {1}".format(vol.id, tags)
+                    return ret
+                if authoritative:
+                    if len(remove):
+                        log.debug('Removed tags for vol.id {0}: {1}'.format(vol.id, remove))
+                        if not delete_tags(vol.id, remove, region=region, key=key, keyid=keyid, profile=profile):
+                            ret['success'] = False
+                            ret['comment'] = "Failed to remove tags on vol.id {0}: {1}".format(vol.id, remove)
+                            return ret
+        ret['changes'].update(changes) if changes['old'] or changes['new'] else None  # pylint: disable=W0106
+    return ret
+
+
+def create_tags(resource_ids, tags, region=None, key=None, keyid=None, profile=None):
+    '''
+    Create new metadata tags for the specified resource ids.
+
+    .. versionadded:: 2016.11.0
+
+    resource_ids
+        (string) or (list) – List of resource IDs.  A plain string will be converted to a list of one element.
+    tags
+        (dict) – Dictionary of name/value pairs. To create only a tag name, pass '' as the value.
+
+    returns
+        (bool) - True on success, False on failure.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt-call boto_ec2.create_tags vol-12345678 '{"Name": "myVolume01"}'
+
+    '''
+    if not isinstance(resource_ids, list):
+        resource_ids = [resource_ids]
+
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+    try:
+        conn.create_tags(resource_ids, tags)
+        return True
+    except boto.exception.BotoServerError as e:
+        log.error(e)
+        return False
+
+
+def delete_tags(resource_ids, tags, region=None, key=None, keyid=None, profile=None):
+    '''
+    Delete metadata tags for the specified resource ids.
+
+    .. versionadded:: 2016.11.0
+
+    resource_ids
+        (string) or (list) – List of resource IDs.  A plain string will be converted to a list of one element.
+    tags
+        (dict) or (list) – Either a dictionary containing name/value pairs or a list containing just tag names.
+                           If you pass in a dictionary, the values must match the actual tag values or the tag
+                           will not be deleted. If you pass in a value of None for the tag value, all tags with
+                           that name will be deleted.
+
+    returns
+        (bool) - True on success, False on failure.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt-call boto_ec2.delete_tags vol-12345678 '{"Name": "myVolume01"}'
+        salt-call boto_ec2.delete_tags vol-12345678 '["Name","MountPoint"]'
+
+    '''
+    if not isinstance(resource_ids, list):
+        resource_ids = [resource_ids]
+
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+    try:
+        conn.delete_tags(resource_ids, tags)
+        return True
+    except boto.exception.BotoServerError as e:
+        log.error(e)
+        return False
+
+
+def detach_volume(volume_id, instance_id=None, device=None, force=False,
+                  region=None, key=None, keyid=None, profile=None):
+    '''
+    Detach an EBS volume from an EC2 instance.
+
+    .. versionadded:: 2016.11.0
+
+    volume_id
+        (string) – The ID of the EBS volume to be detached.
+    instance_id
+        (string) – The ID of the EC2 instance from which it will be detached.
+    device
+        (string) – The device on the instance through which the volume is exposted (e.g. /dev/sdh)
+    force
+        (bool) – Forces detachment if the previous detachment attempt did not occur cleanly.
+                 This option can lead to data loss or a corrupted file system. Use this option
+                 only as a last resort to detach a volume from a failed instance. The instance
+                 will not have an opportunity to flush file system caches nor file system meta data.
+                 If you use this option, you must perform file system check and repair procedures.
+
+    returns
+        (bool) - True on success, False on failure.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt-call boto_ec2.detach_volume vol-12345678 i-87654321
+
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+    try:
+        return conn.detach_volume(volume_id, instance_id, device, force)
+    except boto.exception.BotoServerError as e:
+        log.error(e)
+        return False
+
+
+def delete_volume(volume_id, instance_id=None, device=None, force=False,
+                  region=None, key=None, keyid=None, profile=None):
+    '''
+    Detach an EBS volume from an EC2 instance.
+
+    .. versionadded:: 2016.11.0
+
+    volume_id
+        (string) – The ID of the EBS volume to be deleted.
+    force
+        (bool) – Forces deletion even if the device has not yet been detached from its instance.
+
+    returns
+        (bool) - True on success, False on failure.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt-call boto_ec2.delete_volume vol-12345678
+
+    '''
+    conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
+    try:
+        return conn.delete_volume(volume_id)
+    except boto.exception.BotoServerError as e:
+        if not force:
+            log.error(e)
+            return False
+    try:
+        conn.detach_volume(volume_id, force=force)
+        return conn.delete_volume(volume_id)
+    except boto.exception.BotoServerError as e:
+        log.error(e)
+        return False
