@@ -48,14 +48,11 @@ from __future__ import absolute_import
 
 # Import Python libs
 import logging
-import re
-from distutils.version import LooseVersion as _LooseVersion  # pylint: disable=import-error,no-name-in-module
-import salt.ext.six as six
-from salt.exceptions import SaltInvocationError
 
 log = logging.getLogger(__name__)
 
 # Import third party libs
+import salt.ext.six as six
 try:
     # pylint: disable=unused-import
     import boto
@@ -66,7 +63,10 @@ try:
 except ImportError:
     HAS_BOTO = False
 
+# Import Salt libs
 import salt.utils.odict as odict
+from salt.utils.versions import LooseVersion as _LooseVersion
+from salt.exceptions import SaltInvocationError, CommandExecutionError
 from salt.exceptions import SaltInvocationError
 
 
@@ -110,16 +110,11 @@ def exists(name=None, region=None, key=None, keyid=None, profile=None,
         return False
 
 
-def _check_vpc(vpc_id=None, vpc_name=None, region=None, key=None, keyid=None,
-               profile=None):
+def _vpc_name_to_id(vpc_id=None, vpc_name=None, region=None, key=None, keyid=None,
+                    profile=None):
     data = __salt__['boto_vpc.get_id'](name=vpc_name, region=region,
                                        key=key, keyid=keyid, profile=profile)
-    try:
-        return data.get('id')
-    except TypeError:
-        return None
-    except KeyError:
-        return None
+    return data.get('id')
 
 
 def _split_rules(rules):
@@ -159,7 +154,7 @@ def _get_group(conn=None, name=None, vpc_id=None, vpc_name=None, group_id=None,
                                   'are mutually exclusive.')
     if vpc_name:
         try:
-            vpc_id = _check_vpc(vpc_id=vpc_id, vpc_name=vpc_name, region=region,
+            vpc_id = _vpc_name_to_id(vpc_id=vpc_id, vpc_name=vpc_name, region=region,
                                 key=key, keyid=keyid, profile=profile)
         except boto.exception.BotoServerError as e:
             log.debug(e)
@@ -179,7 +174,7 @@ def _get_group(conn=None, name=None, vpc_id=None, vpc_name=None, group_id=None,
                     return group
             # If there are more security groups, and no vpc_id, we can't know which one to choose.
             if len(filtered_groups) > 1:
-                raise Exception('Security group belongs to more VPCs, specify the VPC ID!')
+                raise CommandExecutionError('Security group belongs to more VPCs, specify the VPC ID!')
             elif len(filtered_groups) == 1:
                 return filtered_groups[0]
             return None
@@ -315,13 +310,12 @@ def get_group_id(name, vpc_id=None, vpc_name=None, region=None, key=None,
         salt myminion boto_secgroup.get_group_id mysecgroup
     '''
     conn = _get_conn(region=region, key=key, keyid=keyid, profile=profile)
-
+    if name.startswith('sg-'):
+        log.debug('group {0} is a group id. get_group_id not called.'.format(name))
+        return name
     group = _get_group(conn=conn, name=name, vpc_id=vpc_id, vpc_name=vpc_name,
                        region=region, key=key, keyid=keyid, profile=profile)
-    if group:
-        return group.id
-    else:
-        return False
+    return getattr(group, 'id', None)
 
 
 def convert_to_group_ids(groups, vpc_id=None, vpc_name=None, region=None, key=None,
@@ -337,21 +331,15 @@ def convert_to_group_ids(groups, vpc_id=None, vpc_name=None, region=None, key=No
     log.debug('security group contents {0} pre-conversion'.format(groups))
     group_ids = []
     for group in groups:
-        if re.match('sg-.*', group):
-            log.debug('group {0} is a group id. get_group_id not called.'.format(group))
-            group_ids.append(group)
+        group_id = get_group_id(name=group, vpc_id=vpc_id,
+                                vpc_name=vpc_name, region=region,
+                                key=key, keyid=keyid, profile=profile)
+        if not group_id:
+            # Security groups are a big deal - need to fail if any can't be resolved...
+            raise CommandExecutionError('Could not resolve Security Group name '
+                                        '{0} to a Group ID'.format(group))
         else:
-            log.debug('calling boto_secgroup.get_group_id for'
-                      ' group name {0}'.format(group))
-            group_id = get_group_id(name=group, vpc_id=vpc_id,
-                                    vpc_name=vpc_name, region=region,
-                                    key=key, keyid=keyid, profile=profile)
-            if not group_id:
-                log.warning('group name {0} did not resolve to a group ID'.format(group))
-            else:
-                log.debug('group name {0} has group id {1}'.format(group, group_id))
-                group_ids.append(str(group_id))
-
+            group_ids.append(str(group_id))
     log.debug('security group contents {0} post-conversion'.format(group_ids))
     return group_ids
 
@@ -401,7 +389,7 @@ def create(name, description, vpc_id=None, vpc_name=None, region=None, key=None,
 
     if not vpc_id and vpc_name:
         try:
-            vpc_id = _check_vpc(vpc_id=vpc_id, vpc_name=vpc_name, region=region,
+            vpc_id = _vpc_name_to_id(vpc_id=vpc_id, vpc_name=vpc_name, region=region,
                                 key=key, keyid=keyid, profile=profile)
         except boto.exception.BotoServerError as e:
             log.debug(e)
@@ -488,6 +476,9 @@ def authorize(name=None, source_group_name=None,
                 log.error(msg)
                 return False
         except boto.exception.EC2ResponseError as e:
+            # if we are trying to add the same rule then we are already in the desired state, return true
+            if e.error_code == 'InvalidPermission.Duplicate':
+                return True
             msg = ('Failed to add rule to security group {0} with id {1}.'
                    .format(group.name, group.id))
             log.error(msg)
