@@ -114,7 +114,6 @@ def list_(consul_url=None, key=None, **kwargs):
     .. code-block:: bash
 
         salt '*' consul.list
-
         salt '*' consul.list key='web'
 
     '''
@@ -164,17 +163,15 @@ def get(consul_url=None, key=None, recurse=False, decode=False, raw=False):
     .. code-block:: bash
 
         salt '*' consul.get key='web/key1'
-
-        salt '*' consul.list key='web' recurse='True
-
-        salt '*' consul.list key='web' recurse='True' decode='True'
+        salt '*' consul.get key='web' recurse=True
+        salt '*' consul.get key='web' recurse=True decode=True
 
     By default values stored in Consul are base64 encoded, passing the
     decode option will show them as the decoded values.
 
     .. code-block:: bash
 
-        salt '*' consul.list key='web' recurse='True' decode='True' raw='True'
+        salt '*' consul.get key='web' recurse=True decode=True raw=True
 
     By default Consult will return other information about the key, the raw
     option will return only the raw value.
@@ -233,11 +230,9 @@ def put(consul_url=None, key=None, value=None, **kwargs):
 
         salt '*' consul.put key='web/key1' value="Hello there"
 
-        salt '*' consul.put key='web/key1' value="Hello there"
-                                acquire='d5d371f4-c380-5280-12fd-8810be175592'
+        salt '*' consul.put key='web/key1' value="Hello there" acquire='d5d371f4-c380-5280-12fd-8810be175592'
 
-        salt '*' consul.put key='web/key1' value="Hello there"
-                                release='d5d371f4-c380-5280-12fd-8810be175592'
+        salt '*' consul.put key='web/key1' value="Hello there" release='d5d371f4-c380-5280-12fd-8810be175592'
 
     '''
     ret = {}
@@ -347,7 +342,6 @@ def delete(consul_url=None, key=None, **kwargs):
     .. code-block:: bash
 
         salt '*' consul.delete key='web'
-
         salt '*' consul.delete key='web' recurse='True'
 
     '''
@@ -687,8 +681,7 @@ def agent_check_register(consul_url=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.agent_check_register name='Memory Utilization'
-                script='/usr/local/bin/check_mem.py' interval='15s'
+        salt '*' consul.agent_check_register name='Memory Utilization' script='/usr/local/bin/check_mem.py' interval='15s'
 
     '''
     ret = {}
@@ -802,8 +795,7 @@ def agent_check_pass(consul_url=None, checkid=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.agent_check_pass checkid='redis_check1'
-                note='Forcing check into passing state.'
+        salt '*' consul.agent_check_pass checkid='redis_check1' note='Forcing check into passing state.'
 
     '''
     ret = {}
@@ -851,8 +843,7 @@ def agent_check_warn(consul_url=None, checkid=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.agent_check_warn checkid='redis_check1'
-                note='Forcing check into warning state.'
+        salt '*' consul.agent_check_warn checkid='redis_check1' note='Forcing check into warning state.'
 
     '''
     ret = {}
@@ -900,8 +891,7 @@ def agent_check_fail(consul_url=None, checkid=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.agent_check_fail checkid='redis_check1'
-                note='Forcing check into critical state.'
+        salt '*' consul.agent_check_fail checkid='redis_check1' note='Forcing check into critical state.'
 
     '''
     ret = {}
@@ -963,9 +953,7 @@ def agent_service_register(consul_url=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.agent_service_register name='redis'
-            tags='["master", "v1"]' address="127.0.0.1" port="8080"
-            check_script="/usr/local/bin/check_redis.py" interval="10s"
+        salt '*' consul.agent_service_register name='redis' tags='["master", "v1"]' address="127.0.0.1" port="8080" check_script="/usr/local/bin/check_redis.py" interval="10s"
 
     '''
     ret = {}
@@ -1106,8 +1094,7 @@ def agent_service_maintenance(consul_url=None, serviceid=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.agent_service_deregister serviceid='redis'
-                enable='True' reason='Down for upgrade'
+        salt '*' consul.agent_service_deregister serviceid='redis' enable='True' reason='Down for upgrade'
 
     '''
     ret = {}
@@ -1177,8 +1164,7 @@ def session_create(consul_url=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.session_create node='node1' name='my-session'
-                behavior='delete' ttl='3600s'
+        salt '*' consul.session_create node='node1' name='my-session' behavior='delete' ttl='3600s'
 
     '''
     ret = {}
@@ -1401,9 +1387,7 @@ def catalog_register(consul_url=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.catalog_register node='node1' address='192.168.1.1'
-            service='redis' service_address='127.0.0.1' service_port='8080'
-            service_id='redis_server1'
+        salt '*' consul.catalog_register node='node1' address='192.168.1.1' service='redis' service_address='127.0.0.1' service_port='8080' service_id='redis_server1'
 
     '''
     ret = {}
@@ -1504,8 +1488,7 @@ def catalog_deregister(consul_url=None, **kwargs):
 
     .. code-block:: bash
 
-        salt '*' consul.catalog_register node='node1'
-            serviceid='redis_server1' checkid='redis_check1'
+        salt '*' consul.catalog_register node='node1' serviceid='redis_server1' checkid='redis_check1'
 
     '''
     ret = {}
@@ -1978,6 +1961,8 @@ def acl_create(consul_url=None, **kwargs):
     Create a new ACL token.
 
     :param consul_url: The Consul server URL.
+    :param id: Unique identifier for the ACL to create
+               leave it blank to let consul server generate one
     :param name: Meaningful indicator of the ACL's purpose.
     :param type: Type is either client or management. A management
                  token is comparable to a root user and has the
@@ -2008,6 +1993,9 @@ def acl_create(consul_url=None, **kwargs):
     else:
         raise SaltInvocationError('Required argument "name" is missing.')
 
+    if 'id' in kwargs:
+        data['ID'] = kwargs['id']
+
     if 'type' in kwargs:
         data['Type'] = kwargs['type']
 
@@ -2126,7 +2114,7 @@ def acl_delete(consul_url=None, **kwargs):
         ret['res'] = False
         return ret
 
-    function = 'acl/delete/{0}'.format(kwargs['id'])
+    function = 'acl/destroy/{0}'.format(kwargs['id'])
     res = _query(consul_url=consul_url,
                  data=data,
                  method='PUT',
