@@ -64,7 +64,7 @@ def _get_config(**kwargs):
     if '__salt__' in globals():
         config_key = '{0}.config'.format(__virtualname__)
         config.update(__salt__['config.get'](config_key, {}))
-    for k in set(config.keys()) & set(kwargs.keys()):
+    for k in set(config) & set(kwargs):
         config[k] = kwargs[k]
     return config
 
@@ -193,58 +193,70 @@ def set_channel_access(channel=14, access_update_mode='non_volatile',
     Set channel access
 
     :param channel: number [1:7]
-    :param access_update_mode: one of
+
+    :param access_update_mode:
         - 'dont_change'  = don't set or change Channel Access
         - 'non_volatile' = set non-volatile Channel Access
         - 'volatile'     = set volatile (active) setting of Channel Access
 
-    :param alerting: PEF Alerting Enable/Disable
+    :param alerting:
+        PEF Alerting Enable/Disable
+
         - True  = enable PEF Alerting
         - False = disable PEF Alerting on this channel
-                (Alert Immediate command can still be used to generate alerts)
+          (Alert Immediate command can still be used to generate alerts)
+
+    :param per_msg_auth:
+        Per-message Authentication
 
-    :param per_msg_auth: Per-message Authentication
         - True  = enable
         - False = disable Per-message Authentication. [Authentication required to
-                activate any session on this channel, but authentication not
-                used on subsequent packets for the session.]
+          activate any session on this channel, but authentication not
+          used on subsequent packets for the session.]
+
+    :param user_level_auth:
+        User Level Authentication Enable/Disable
 
-    :param user_level_auth: User Level Authentication Enable/Disable.
         - True  = enable User Level Authentication. All User Level commands are
-            to be authenticated per the Authentication Type that was
-            negotiated when the session was activated.
+          to be authenticated per the Authentication Type that was
+          negotiated when the session was activated.
         - False = disable User Level Authentication. Allow User Level commands to
-            be executed without being authenticated.
-            If the option to disable User Level Command authentication is
-            accepted, the BMC will accept packets with Authentication Type
-            set to None if they contain user level commands.
-            For outgoing packets, the BMC returns responses with the same
-            Authentication Type that was used for the request.
+          be executed without being authenticated.
+          If the option to disable User Level Command authentication is
+          accepted, the BMC will accept packets with Authentication Type
+          set to None if they contain user level commands.
+          For outgoing packets, the BMC returns responses with the same
+          Authentication Type that was used for the request.
 
-    :param access_mode: Access Mode for IPMI messaging
-        (PEF Alerting is enabled/disabled separately from IPMI messaging)
-        * disabled = disabled for IPMI messaging
-        * pre_boot = pre-boot only channel only available when system is in a
-                powered down state or in BIOS prior to start of boot.
-        * always   = channel always available regardless of system mode.
-                BIOS typically dedicates the serial connection to the BMC.
-        * shared   = same as always available, but BIOS typically leaves the
-                serial port available for software use.
+    :param access_mode:
+        Access Mode for IPMI messaging (PEF Alerting is enabled/disabled
+        separately from IPMI messaging)
 
-    :param privilege_update_mode: Channel Privilege Level Limit.
-        This value sets the maximum privilege level
-        that can be accepted on the specified channel.
-        * dont_change  = don't set or change channel Privilege Level Limit
-        * non_volatile = non-volatile Privilege Level Limit according
-        * volatile     = volatile setting of Privilege Level Limit
+        - disabled = disabled for IPMI messaging
+        - pre_boot = pre-boot only channel only available when system is
+          in a powered down state or in BIOS prior to start of boot.
+        - always   = channel always available regardless of system mode.
+          BIOS typically dedicates the serial connection to the BMC.
+        - shared   = same as always available, but BIOS typically leaves the
+          serial port available for software use.
 
-    :param privilege_level: Channel Privilege Level Limit
-        * reserved      = unused
-        * callback
-        * user
-        * operator
-        * administrator
-        * proprietary   = used by OEM
+    :param privilege_update_mode:
+        Channel Privilege Level Limit. This value sets the maximum privilege
+        level that can be accepted on the specified channel.
+
+        - dont_change  = don't set or change channel Privilege Level Limit
+        - non_volatile = non-volatile Privilege Level Limit according
+        - volatile     = volatile setting of Privilege Level Limit
+
+    :param privilege_level:
+        Channel Privilege Level Limit
+
+        - reserved      = unused
+        - callback
+        - user
+        - operator
+        - administrator
+        - proprietary   = used by OEM
 
     :param kwargs:
         - api_host=127.0.0.1
@@ -269,9 +281,11 @@ def get_channel_access(channel=14, read_mode='non_volatile', **kwargs):
     :param kwargs:api_host='127.0.0.1' api_user='admin' api_pass='example' api_port=623
 
     :param channel: number [1:7]
+
     :param read_mode:
         - non_volatile  = get non-volatile Channel Access
         - volatile      = get present volatile (active) setting of Channel Access
+
     :param kwargs:
         - api_host=127.0.0.1
         - api_user=admin
@@ -279,8 +293,12 @@ def get_channel_access(channel=14, read_mode='non_volatile', **kwargs):
         - api_port=623
         - api_kg=None
 
-    :return: A Python dict with the following keys/values:
+    Return Data
+
+        A Python dict with the following keys/values:
+
         .. code-block:: python
+
             {
                 alerting:
                 per_msg_auth:
@@ -322,14 +340,17 @@ def get_channel_info(channel=14, **kwargs):
         - api_port=623
         - api_kg=None
 
-    :return:
-    session_support:
-        no_session: channel is session-less
-        single: channel is single-session
-        multi: channel is multi-session
-        auto: channel is session-based (channel could alternate between
-            single- and multi-session operation, as can occur with a
-            serial/modem channel that supports connection mode auto-detect)
+    Return Data
+        channel session supports
+
+        .. code-block:: none
+
+                - no_session: channel is session-less
+                - single: channel is single-session
+                - multi: channel is multi-session
+                - auto: channel is session-based (channel could alternate between
+                    single- and multi-session operation, as can occur with a
+                    serial/modem channel that supports connection mode auto-detect)
 
     CLI Examples:
 
@@ -350,38 +371,39 @@ def set_user_access(uid, channel=14, callback=True, link_auth=True, ipmi_msg=Tru
 
     :param channel: number [1:7]
 
-    :parm callback: User Restricted to Callback
-    False = User Privilege Limit is determined by the User Privilege Limit
-        parameter, below, for both callback and non-callback connections.
-    True  = User Privilege Limit is determined by the User Privilege Limit
-        parameter for callback connections, but is restricted to Callback
-        level for non-callback connections. Thus, a user can only initiate
-        a Callback when they 'call in' to the BMC, but once the callback
-        connection has been made, the user could potentially establish a
-        session as an Operator.
+    :param callback:
+        User Restricted to Callback
 
-    :param link_auth: User Link authentication
-    enable/disable (used to enable whether this
-    user's name and password information will be used for link
-    authentication, e.g. PPP CHAP) for the given channel. Link
-    authentication itself is a global setting for the channel and is
-    enabled/disabled via the serial/modem configuration parameters.
+        - False = User Privilege Limit is determined by the User Privilege Limit
+          parameter, below, for both callback and non-callback connections.
+        - True  = User Privilege Limit is determined by the User Privilege Limit
+          parameter for callback connections, but is restricted to Callback
+          level for non-callback connections. Thus, a user can only initiate
+          a Callback when they 'call in' to the BMC, but once the callback
+          connection has been made, the user could potentially establish a
+          session as an Operator.
 
-    :param ipmi_msg: User IPMI Messaginge:
-    (used to enable/disable whether
-    this user's name and password information will be used for IPMI
-    Messaging. In this case, 'IPMI Messaging' refers to the ability to
-    execute generic IPMI commands that are not associated with a
-    particular payload type. For example, if IPMI Messaging is disabled for
-    a user, but that user is enabled for activatallow_authing the SOL
-    payload type, then IPMI commands associated with SOL and session
-    management, such as Get SOL Configuration Parameters and Close Session
-    are available, but generic IPMI commands such as Get SEL Time are
-    unavailable.)
+    :param link_auth: User Link authentication enable/disable (used to enable
+        whether this user's name and password information will be used for link
+        authentication, e.g. PPP CHAP) for the given channel. Link
+        authentication itself is a global setting for the channel and is
+        enabled/disabled via the serial/modem configuration parameters.
+
+    :param ipmi_msg: User IPMI Messaging: (used to enable/disable whether
+        this user's name and password information will be used for IPMI
+        Messaging. In this case, 'IPMI Messaging' refers to the ability to
+        execute generic IPMI commands that are not associated with a
+        particular payload type. For example, if IPMI Messaging is disabled for
+        a user, but that user is enabled for activating the SOL
+        payload type, then IPMI commands associated with SOL and session
+        management, such as Get SOL Configuration Parameters and Close Session
+        are available, but generic IPMI commands such as Get SEL Time are
+        unavailable.)
 
     :param privilege_level:
-    User Privilege Limit. (Determines the maximum privilege level that the
-    user is allowed to switch to on the specified channel.)
+        User Privilege Limit. (Determines the maximum privilege level that the
+        user is allowed to switch to on the specified channel.)
+
         - callback
         - user
         - operator
@@ -419,7 +441,10 @@ def get_user_access(uid, channel=14, **kwargs):
         - api_port=623
         - api_kg=None
 
-    :return:
+    Return Data
+
+    .. code-block:: none
+
         channel_info:
             - max_user_count = maximum number of user IDs on this channel
             - enabled_users = count of User ID slots presently in use
@@ -431,8 +456,6 @@ def get_user_access(uid, channel=14, **kwargs):
             - privilege_level: [reserved, callback, user, operator
                                administrator, proprietary, no_access]
 
-
-
     CLI Examples:
 
     .. code-block:: bash
@@ -597,10 +620,13 @@ def get_sensor_data(**kwargs):
 
         salt-call ipmi.get_sensor_data api_host=127.0.0.1 api_user=admin api_pass=pass
     '''
+    import ast
     with _IpmiCommand(**kwargs) as s:
         data = {}
         for reading in s.get_sensor_data():
-            data[reading['name']] = reading
+            if reading:
+                r = ast.literal_eval(repr(reading))
+                data[r.pop('name')] = r
     return data
 
 
@@ -673,21 +699,24 @@ def set_bootdev(bootdev='default', persist=False, uefiboot=False, **kwargs):
     Set boot device to use on next reboot
 
     :param bootdev:
-                    *network -- Request network boot
-                    *hd -- Boot from hard drive
-                    *safe -- Boot from hard drive, requesting 'safe mode'
-                    *optical -- boot from CD/DVD/BD drive
-                    *setup -- Boot into setup utility
-                    *default -- remove any IPMI directed boot device
-                                request
+        - network: Request network boot
+        - hd: Boot from hard drive
+        - safe: Boot from hard drive, requesting 'safe mode'
+        - optical: boot from CD/DVD/BD drive
+        - setup: Boot into setup utility
+        - default: remove any IPMI directed boot device
+          request
+
     :param persist: If true, ask that system firmware use this device
                     beyond next boot.  Be aware many systems do not honor
                     this
+
     :param uefiboot: If true, request UEFI boot explicitly.  Strictly
-                    speaking, the spec sugests that if not set, the system
+                    speaking, the spec suggests that if not set, the system
                     should BIOS boot and offers no "don't care" option.
                     In practice, this flag not being set does not preclude
                     UEFI boot on any system I've encountered.
+
     :param kwargs:
         - api_host=127.0.0.1
         - api_user=admin
@@ -770,7 +799,10 @@ def get_user(uid, channel=14, **kwargs):
         - api_port=623
         - api_kg=None
 
-    :return:
+    Return Data
+
+    .. code-block:: none
+
         name: (str)
         uid: (int)
         channel: (int)
@@ -780,6 +812,7 @@ def get_user(uid, channel=14, **kwargs):
             - ipmi_msg (bool)
             - privilege_level: (str)[callback, user, operatorm administrator,
                                     proprietary, no_access]
+
     CLI Examples:
 
     .. code-block:: bash
@@ -797,6 +830,7 @@ def get_users(channel=14, **kwargs):
     get list of users and access information
 
     :param channel: number [1:7]
+
     :param kwargs:
         - api_host=127.0.0.1
         - api_user=admin
@@ -805,15 +839,15 @@ def get_users(channel=14, **kwargs):
         - api_kg=None
 
     :return:
-        name: (str)
-        uid: (int)
-        channel: (int)
-        access:
+        - name: (str)
+        - uid: (int)
+        - channel: (int)
+        - access:
             - callback (bool)
             - link_auth (bool)
             - ipmi_msg (bool)
             - privilege_level: (str)[callback, user, operatorm administrator,
-                                    proprietary, no_access]
+              proprietary, no_access]
 
     CLI Examples:
 
