@@ -5,26 +5,25 @@ Support for iptables
 Configuration Options
 ---------------------
 
-The following options can be set in the :ref:`minion config
-<configuration-salt-minion>`, :ref:`minion grains<configuration-minion-grains>`
-, :ref:`minion pillar<configuration-minion-pillar>`, or
-`master config<configuration-salt-master>`.
+The following options can be set in the minion config, grains, pillar, or
+master config. The configuration is read using :py:func:`config.get
+<salt.modules.config.get>`.
 
 - ``iptables.save_filters``: List of REGEX strings to FILTER OUT matching lines
 
-    This is useful for filtering out chains, rules, etc that you do not
-    wish to persist, such as ephemeral Docker rules.
+  This is useful for filtering out chains, rules, etc that you do not wish to
+  persist, such as ephemeral Docker rules.
 
-    The default is to not filter out anything.
+  The default is to not filter out anything.
 
-    .. code-block:: yaml
+  .. code-block:: yaml
 
-        iptables.save_filters:
-           - "-j CATTLE_PREROUTING"
-           - "-j DOCKER"
-           - "-A POSTROUTING"
-           - "-A CATTLE_POSTROUTING"
-           - "-A FORWARD"
+      iptables.save_filters:
+        - "-j CATTLE_PREROUTING"
+        - "-j DOCKER"
+        - "-A POSTROUTING"
+        - "-A CATTLE_POSTROUTING"
+        - "-A FORWARD"
 '''
 from __future__ import absolute_import
 
