@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-Module for editing power settings on Mac OS X
+Module for editing power settings on macOS
 
  .. versionadded:: 2016.3.0
 '''
@@ -18,11 +18,11 @@ __virtualname__ = 'power'
 
 def __virtual__():
     '''
-    Only for Mac OS X
+    Only for macOS
     '''
     if not salt.utils.is_darwin():
         return (False, 'The mac_power module could not be loaded: '
-                       'module only works on Mac OS X systems.')
+                       'module only works on macOS systems.')
 
     return __virtualname__
 
@@ -75,7 +75,8 @@ def get_sleep():
     Computer, Display, and Hard Disk are displayed.
 
     :return: A dictionary containing the sleep status for Computer, Display, and
-    Hard Disk
+        Hard Disk
+
     :rtype: dict
 
     CLI Example:
@@ -284,8 +285,8 @@ def set_wake_on_modem(enabled):
     detected.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -331,8 +332,8 @@ def set_wake_on_network(enabled):
     is detected.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -378,8 +379,8 @@ def set_restart_power_failure(enabled):
     failure.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -423,12 +424,12 @@ def set_restart_freeze(enabled):
     '''
     Specifies whether the server restarts automatically after a system freeze.
     This setting doesn't seem to be editable. The command completes successfully
-    but the setting isn't actually updated. This is probably an OS X bug. The
+    but the setting isn't actually updated. This is probably a macOS. The
     functions remains in case they ever fix the bug.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
@@ -446,6 +447,7 @@ def set_restart_freeze(enabled):
     return salt.utils.mac_utils.confirm_updated(
         state,
         get_restart_freeze,
+        True
     )
 
 
@@ -455,7 +457,8 @@ def get_sleep_on_power_button():
     supported
 
     :return: A string value representing the "allow power button to sleep
-    computer" settings
+        computer" settings
+
     :rtype: string
 
     CLI Example:
@@ -467,7 +470,7 @@ def get_sleep_on_power_button():
     ret = salt.utils.mac_utils.execute_return_result(
         'systemsetup -getallowpowerbuttontosleepcomputer')
     return salt.utils.mac_utils.validate_enabled(
-        salt.utils.mac_utils.parse_return(ret)) == 'on'
+            salt.utils.mac_utils.parse_return(ret)) == 'on'
 
 
 def set_sleep_on_power_button(enabled):
@@ -475,8 +478,8 @@ def set_sleep_on_power_button(enabled):
     Set whether or not the power button can sleep the computer.
 
     :param bool enabled: True to enable, False to disable. "On" and "Off" are
-    also acceptable values. Additionally you can pass 1 and 0 to represent True
-    and False respectively
+        also acceptable values. Additionally you can pass 1 and 0 to represent
+        True and False respectively
 
     :return: True if successful, False if not
     :rtype: bool
