@@ -9,12 +9,12 @@ from __future__ import absolute_import
 import os
 import re
 import plistlib
-from distutils.version import LooseVersion
 
 # Import salt libs
 import salt.utils
 import salt.utils.decorators as decorators
 from salt.exceptions import CommandExecutionError
+from salt.utils.versions import LooseVersion as _LooseVersion
 
 # Import 3rd party libs
 import salt.ext.six as six
@@ -43,7 +43,7 @@ def __virtual__():
         return (False, 'Failed to load the mac_service module:\n'
                        'Required binary not found: "plutil"')
 
-    if LooseVersion(__grains__['osrelease']) < LooseVersion('10.11'):
+    if _LooseVersion(__grains__['osrelease']) < _LooseVersion('10.11'):
         return (False, 'Failed to load the mac_service module:\n'
                        'Requires macOS 10.11 or newer')
 
@@ -421,7 +421,7 @@ def status(name, sig=None, runas=None):
     for line in output.splitlines():
         if 'PID' in line:
             continue
-        if re.search(name, line):
+        if re.search(name, line.split()[-1]):
             if line.split()[0].isdigit():
                 if pids:
                     pids += '\n'
@@ -496,7 +496,7 @@ def enabled(name, runas=None):
         return False
 
 
-def disabled(name, runas=None):
+def disabled(name, runas=None, domain='system'):
     '''
     Check if the specified service is not enabled. This is the opposite of
     ``service.enabled``
@@ -505,6 +505,8 @@ def disabled(name, runas=None):
 
     :param str runas: User to run launchctl commands
 
+    :param str domain: domain to check for disabled services. Default is system.
+
     :return: True if the specified service is NOT enabled, otherwise False
     :rtype: bool
 
@@ -514,8 +516,22 @@ def disabled(name, runas=None):
 
         salt '*' service.disabled org.cups.cupsd
     '''
-    # A service is disabled if it is not enabled
-    return not enabled(name, runas=runas)
+    ret = False
+    disabled = launchctl('print-disabled',
+                         domain,
+                         return_stdout=True,
+                         output_loglevel='trace',
+                         runas=runas)
+    for service in disabled.split("\n"):
+        if name in service:
+            srv_name = service.split("=>")[0].split("\"")[1]
+            status = service.split("=>")[1]
+            if name != srv_name:
+                pass
+            else:
+                return True if 'true' in status.lower() else False
+
+    return False
 
 
 def get_all(runas=None):
