@@ -306,7 +306,11 @@ def glob(tgt, minion_id=None):
         return False
 
 
-def filter_by(lookup, expr_form='compound', minion_id=None):
+def filter_by(lookup,
+              tgt_type='compound',
+              minion_id=None,
+              expr_form=None,
+              default='default'):
     '''
     Return the first match in a dictionary of target patterns
 
@@ -320,20 +324,76 @@ def filter_by(lookup, expr_form='compound', minion_id=None):
 
     Pillar Example:
 
-    .. code-block:: yaml
+    .. code-block:: jinja
 
+        # Filter the data for the current minion into a variable:
         {% set roles = salt['match.filter_by']({
             'web*': ['app', 'caching'],
             'db*': ['db'],
-        }) %}
+        }, default='web*') %}
+
+        # Make the filtered data available to Pillar:
+        roles: {{ roles | yaml() }}
     '''
+    # remember to remove the expr_form argument from this function when
+    # performing the cleanup on this deprecation.
+    if expr_form is not None:
+        salt.utils.warn_until(
+            'Fluorine',
+            'the target type should be passed using the \'tgt_type\' '
+            'argument instead of \'expr_form\'. Support for using '
+            '\'expr_form\' will be removed in Salt Fluorine.'
+        )
+        tgt_type = expr_form
+
     expr_funcs = dict(inspect.getmembers(sys.modules[__name__],
         predicate=inspect.isfunction))
 
     for key in lookup:
-        if minion_id and expr_funcs[expr_form](key, minion_id):
-            return lookup[key]
-        elif expr_funcs[expr_form](key):
+        params = (key, minion_id) if minion_id else (key, )
+        if expr_funcs[tgt_type](*params):
             return lookup[key]
 
-    return None
+    return lookup.get(default, None)
+
+
+def search_by(lookup, tgt_type='compound', minion_id=None):
+    '''
+    Search a dictionary of target strings for matching targets
+
+    This is the inverse of :py:func:`match.filter_by
+    <salt.modules.match.filter_by>` and allows matching values instead of
+    matching keys. A minion can be matched by multiple entries.
+
+    .. versionadded:: 2017.7.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' match.search_by '{web: [node1, node2], db: [node2, node]}'
+
+    Pillar Example:
+
+    .. code-block:: jinja
+
+        {% set roles = salt.match.search_by({
+            'web': ['G@os_family:Debian not nodeX'],
+            'db': ['L@node2,node3 and G@datacenter:west'],
+            'caching': ['node3', 'node4'],
+        }) %}
+
+        # Make the filtered data available to Pillar:
+        roles: {{ roles | yaml() }}
+    '''
+    expr_funcs = dict(inspect.getmembers(sys.modules[__name__],
+        predicate=inspect.isfunction))
+
+    matches = []
+    for key, target_list in lookup.items():
+        for target in target_list:
+            params = (target, minion_id) if minion_id else (target, )
+            if expr_funcs[tgt_type](*params):
+                matches.append(key)
+
+    return matches or None
