@@ -26,16 +26,9 @@ from __future__ import absolute_import
 import logging
 log = logging.getLogger(__file__)
 
-
-try:
-    # will try to import NAPALM
-    # https://github.com/napalm-automation/napalm
-    # pylint: disable=W0611
-    from napalm_base import get_network_driver
-    # pylint: enable=W0611
-    HAS_NAPALM = True
-except ImportError:
-    HAS_NAPALM = False
+# import NAPALM utils
+import salt.utils.napalm
+from salt.utils.napalm import proxy_napalm_wrap
 
 # ----------------------------------------------------------------------------------------------------------------------
 # module properties
@@ -51,17 +44,10 @@ __proxyenabled__ = ['napalm']
 
 
 def __virtual__():
-
     '''
-    NAPALM library must be installed for this module to work.
-    Also, the key proxymodule must be set in the __opts___ dictionary.
+    NAPALM library must be installed for this module to work and run in a (proxy) minion.
     '''
-
-    if HAS_NAPALM and 'proxy' in __opts__:
-        return __virtualname__
-    else:
-        return (False, 'The module NTP cannot be loaded: \
-                napalm or proxy could not be loaded.')
+    return salt.utils.napalm.virtual(__opts__, __virtualname__, __file__)
 
 # ----------------------------------------------------------------------------------------------------------------------
 # helper functions -- will not be exported
@@ -72,7 +58,8 @@ def __virtual__():
 # ----------------------------------------------------------------------------------------------------------------------
 
 
-def peers():
+@proxy_napalm_wrap
+def peers(**kwargs):  # pylint: disable=unused-argument
 
     '''
     Returns a list the NTP peers configured on the network device.
@@ -98,7 +85,8 @@ def peers():
 
     '''
 
-    ntp_peers = __proxy__['napalm.call'](
+    ntp_peers = salt.utils.napalm.call(
+        napalm_device,  # pylint: disable=undefined-variable
         'get_ntp_peers',
         **{
         }
@@ -114,7 +102,8 @@ def peers():
     return ntp_peers
 
 
-def servers():
+@proxy_napalm_wrap
+def servers(**kwargs):  # pylint: disable=unused-argument
 
     '''
     Returns a list of the configured NTP servers on the device.
@@ -126,7 +115,8 @@ def servers():
         salt '*' ntp.servers
     '''
 
-    ntp_servers = __proxy__['napalm.call'](
+    ntp_servers = salt.utils.napalm.call(
+        napalm_device,  # pylint: disable=undefined-variable
         'get_ntp_servers',
         **{
         }
@@ -142,7 +132,8 @@ def servers():
     return ntp_servers
 
 
-def stats(peer=None):
+@proxy_napalm_wrap
+def stats(peer=None, **kwargs):  # pylint: disable=unused-argument
 
     '''
     Returns a dictionary containing synchronization details of the NTP peers.
@@ -189,7 +180,8 @@ def stats(peer=None):
         ]
     '''
 
-    proxy_output = __proxy__['napalm.call'](
+    proxy_output = salt.utils.napalm.call(
+        napalm_device,  # pylint: disable=undefined-variable
         'get_ntp_stats',
         **{
         }
@@ -210,15 +202,18 @@ def stats(peer=None):
     return proxy_output
 
 
+@proxy_napalm_wrap
 def set_peers(*peers, **options):
 
     '''
     Configures a list of NTP peers on the device.
 
     :param peers: list of IP Addresses/Domain Names
-    :param test (bool): discard loaded config. By default `test` is False (will not dicard the changes)
-    :commit commit (bool): commit loaded config. By default `commit` is True (will commit the changes). Useful when
-    the user does not want to commit after each change, but after a couple.
+    :param test (bool): discard loaded config. By default `test` is False (will
+        not dicard the changes)
+    :commit commit (bool): commit loaded config. By default `commit` is True
+        (will commit the changes). Useful when the user does not want to commit
+        after each change, but after a couple.
 
     By default this function will commit the config changes (if any). To load without committing, use the `commit`
     option. For dry run use the `test` argument.
@@ -238,18 +233,20 @@ def set_peers(*peers, **options):
     return __salt__['net.load_template']('set_ntp_peers',
                                          peers=peers,
                                          test=test,
-                                         commit=commit)
+                                         commit=commit,
+                                         inherit_napalm_device=napalm_device)  # pylint: disable=undefined-variable
 
 
+@proxy_napalm_wrap
 def set_servers(*servers, **options):
-
     '''
     Configures a list of NTP servers on the device.
 
     :param servers: list of IP Addresses/Domain Names
     :param test (bool): discard loaded config. By default `test` is False (will not dicard the changes)
-    :commit commit (bool): commit loaded config. By default `commit` is True (will commit the changes). Useful when
-    the user does not want to commit after each change, but after a couple.
+    :commit commit (bool): commit loaded config. By default `commit` is True
+        (will commit the changes). Useful when the user does not want to commit
+        after each change, but after a couple.
 
     By default this function will commit the config changes (if any). To load without committing, use the `commit`
     option. For dry run use the `test` argument.
@@ -269,21 +266,26 @@ def set_servers(*servers, **options):
     return __salt__['net.load_template']('set_ntp_servers',
                                          servers=servers,
                                          test=test,
-                                         commit=commit)
+                                         commit=commit,
+                                         inherit_napalm_device=napalm_device)  # pylint: disable=undefined-variable
 
 
+@proxy_napalm_wrap
 def delete_peers(*peers, **options):
 
     '''
     Removes NTP peers configured on the device.
 
     :param peers: list of IP Addresses/Domain Names to be removed as NTP peers
-    :param test (bool): discard loaded config. By default `test` is False (will not dicard the changes)
-    :commit commit (bool): commit loaded config. By default `commit` is True (will commit the changes). Useful when
-    the user does not want to commit after each change, but after a couple.
+    :param test (bool): discard loaded config. By default `test` is False (will
+        not dicard the changes)
+    :param commit (bool): commit loaded config. By default `commit` is True
+        (will commit the changes). Useful when the user does not want to commit
+        after each change, but after a couple.
 
-    By default this function will commit the config changes (if any). To load without committing, use the `commit`
-    option. For dry run use the `test` argument.
+    By default this function will commit the config changes (if any). To load
+    without committing, use the ``commit`` option. For a dry run, use the
+    ``test`` argument.
 
     CLI Example:
 
@@ -300,21 +302,27 @@ def delete_peers(*peers, **options):
     return __salt__['net.load_template']('delete_ntp_peers',
                                          peers=peers,
                                          test=test,
-                                         commit=commit)
+                                         commit=commit,
+                                         inherit_napalm_device=napalm_device)  # pylint: disable=undefined-variable
 
 
+@proxy_napalm_wrap
 def delete_servers(*servers, **options):
 
     '''
     Removes NTP servers configured on the device.
 
-    :param servers: list of IP Addresses/Domain Names to be removed as NTP servers
-    :param test (bool): discard loaded config. By default `test` is False (will not dicard the changes)
-    :commit commit (bool): commit loaded config. By default `commit` is True (will commit the changes). Useful when
-    the user does not want to commit after each change, but after a couple.
+    :param servers: list of IP Addresses/Domain Names to be removed as NTP
+        servers
+    :param test (bool): discard loaded config. By default `test` is False (will
+        not dicard the changes)
+    :param commit (bool): commit loaded config. By default `commit` is True
+        (will commit the changes). Useful when the user does not want to commit
+        after each change, but after a couple.
 
-    By default this function will commit the config changes (if any). To load without committing, use the `commit`
-    option. For dry run use the `test` argument.
+    By default this function will commit the config changes (if any). To load
+    without committing, use the ``commit`` option. For dry run use the ``test``
+    argument.
 
     CLI Example:
 
@@ -331,4 +339,5 @@ def delete_servers(*servers, **options):
     return __salt__['net.load_template']('delete_ntp_servers',
                                          servers=servers,
                                          test=test,
-                                         commit=commit)
+                                         commit=commit,
+                                         inherit_napalm_device=napalm_device)  # pylint: disable=undefined-variable
