@@ -168,20 +168,7 @@ def _get_pip_bin(bin_env):
             # If the python binary was passed, return it
             if 'python' in os.path.basename(bin_env):
                 return [os.path.normpath(bin_env), '-m', 'pip']
-            # Try to find the python binary based on the location of pip in a
-            # virtual environment, should be relative
-            if 'pip' in os.path.basename(bin_env):
-                # Look in the same directory as the pip binary, and also its
-                # parent directories.
-                pip_dirname = os.path.dirname(bin_env)
-                pip_parent_dir = os.path.dirname(pip_dirname)
-                for bin_path in _search_paths(pip_dirname, pip_parent_dir):
-                    if os.path.isfile(bin_path):
-                        logger.debug('pip: Found python binary: %s', bin_path)
-                        return [os.path.normpath(bin_path), '-m', 'pip']
-
-            # Couldn't find python, use the passed pip binary
-            # This has the limitation of being unable to update pip itself
+            # We have been passed a pip binary, use the pip binary.
             return [os.path.normpath(bin_env)]
 
         raise CommandExecutionError(
@@ -452,13 +439,17 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         Path to requirements
 
     bin_env
-        Path to pip bin or path to virtualenv. If doing a system install,
-        and want to use a specific pip bin (pip-2.7, pip-2.6, etc..) just
-        specify the pip bin you want.
+        Path to pip (or to a virtualenv). This can be used to specify the path
+        to the pip to use when more than one Python release is installed (e.g.
+        ``/usr/bin/pip-2.7`` or ``/usr/bin/pip-2.6``. If a directory path is
+        specified, it is assumed to be a virtualenv.
 
         .. note::
-            If installing into a virtualenv, just use the path to the
-            virtualenv (e.g. ``/home/code/path/to/virtualenv/``)
+
+            For Windows, if the pip module is being used to upgrade the pip
+            package, bin_env should be the path to the virtualenv or to the
+            python binary that should be used.  The pip command is unable to
+            upgrade itself in Windows.
 
     use_wheel
         Prefer wheel archives (requires pip>=1.4)
@@ -561,7 +552,7 @@ def install(pkgs=None,  # pylint: disable=R0912,R0913,R0914
         The user under which to run pip
 
     cwd
-        Current working directory to run pip from
+        Directory from which to run pip
 
     pre_releases
         Include pre-releases in the available versions
@@ -931,36 +922,38 @@ def uninstall(pkgs=None,
               saltenv='base',
               use_vt=False):
     '''
-    Uninstall packages with pip
-
-    Uninstall packages individually or from a pip requirements file. Uninstall
-    packages globally or from a virtualenv.
+    Uninstall packages individually or from a pip requirements file
 
     pkgs
         comma separated list of packages to install
+
     requirements
-        path to requirements.
+        Path to requirements file
+
     bin_env
-        path to pip bin or path to virtualenv. If doing an uninstall from
-        the system python and want to use a specific pip bin (pip-2.7,
-        pip-2.6, etc..) just specify the pip bin you want.
-        If uninstalling from a virtualenv, just use the path to the virtualenv
-        (/home/code/path/to/virtualenv/)
+        Path to pip (or to a virtualenv). This can be used to specify the path
+        to the pip to use when more than one Python release is installed (e.g.
+        ``/usr/bin/pip-2.7`` or ``/usr/bin/pip-2.6``. If a directory path is
+        specified, it is assumed to be a virtualenv.
+
     log
         Log file where a complete (maximum verbosity) record will be kept
+
     proxy
-        Specify a proxy in the form
-        user:passwd@proxy.server:port. Note that the
-        user:password@ is optional and required only if you
-        are behind an authenticated proxy.  If you provide
-        user@proxy.server:port then you will be prompted for a
-        password.
+        Specify a proxy in the format ``user:passwd@proxy.server:port``. Note
+        that the ``user:password@`` is optional and required only if you are
+        behind an authenticated proxy.  If you provide
+        ``user@proxy.server:port`` then you will be prompted for a password.
+
     timeout
         Set the socket timeout (default 15 seconds)
+
     user
         The user under which to run pip
+
     cwd
-        Current working directory to run pip from
+        Directory from which to run pip
+
     use_vt
         Use VT terminal emulation (see output while installing)
 
@@ -972,7 +965,6 @@ def uninstall(pkgs=None,
         salt '*' pip.uninstall requirements=/path/to/requirements.txt
         salt '*' pip.uninstall <package name> bin_env=/path/to/virtualenv
         salt '*' pip.uninstall <package name> bin_env=/path/to/pip_bin
-
     '''
     cmd = _get_pip_bin(bin_env)
     cmd.extend(['uninstall', '-y'])
@@ -1054,32 +1046,27 @@ def freeze(bin_env=None,
     virtualenv
 
     bin_env
-        path to pip bin or path to virtualenv. If doing an uninstall from
-        the system python and want to use a specific pip bin (pip-2.7,
-        pip-2.6, etc..) just specify the pip bin you want.
-        If uninstalling from a virtualenv, just use the path to the virtualenv
-        (/home/code/path/to/virtualenv/)
+        Path to pip (or to a virtualenv). This can be used to specify the path
+        to the pip to use when more than one Python release is installed (e.g.
+        ``/usr/bin/pip-2.7`` or ``/usr/bin/pip-2.6``. If a directory path is
+        specified, it is assumed to be a virtualenv.
+
     user
         The user under which to run pip
+
     cwd
-        Current working directory to run pip from
+        Directory from which to run pip
 
     .. note::
-
         If the version of pip available is older than 8.0.3, the list will not
-        include the packages pip, wheel, setuptools, or distribute even if they
-        are installed.
+        include the packages ``pip``, ``wheel``, ``setuptools``, or
+        ``distribute`` even if they are installed.
 
     CLI Example:
 
     .. code-block:: bash
 
-        salt '*' pip.freeze /home/code/path/to/virtualenv/
-
-    .. versionchanged:: 2016.11.2
-
-        The packages pip, wheel, setuptools, and distribute are included if the
-        installed pip is new enough.
+        salt '*' pip.freeze bin_env=/home/code/path/to/virtualenv
     '''
     cmd = _get_pip_bin(bin_env)
     cmd.append('freeze')
@@ -1124,21 +1111,16 @@ def list_(prefix=None,
     .. note::
 
         If the version of pip available is older than 8.0.3, the packages
-        wheel, setuptools, and distribute will not be reported by this function
-        even if they are installed. Unlike
-        :py:func:`pip.freeze <salt.modules.pip.freeze>`, this function always
-        reports the version of pip which is installed.
+        ``wheel``, ``setuptools``, and ``distribute`` will not be reported by
+        this function even if they are installed. Unlike :py:func:`pip.freeze
+        <salt.modules.pip.freeze>`, this function always reports the version of
+        pip which is installed.
 
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' pip.list salt
-
-    .. versionchanged:: 2016.11.2
-
-        The packages wheel, setuptools, and distribute are included if the
-        installed pip is new enough.
     '''
     packages = {}
 
@@ -1392,9 +1374,10 @@ def list_all_versions(pkg,
         The package to check
 
     bin_env
-        Path to pip bin or path to virtualenv. If doing a system install,
-        and want to use a specific pip bin (pip-2.7, pip-2.6, etc..) just
-        specify the pip bin you want.
+        Path to pip (or to a virtualenv). This can be used to specify the path
+        to the pip to use when more than one Python release is installed (e.g.
+        ``/usr/bin/pip-2.7`` or ``/usr/bin/pip-2.6``. If a directory path is
+        specified, it is assumed to be a virtualenv.
 
     include_alpha
         Include alpha versions in the list
@@ -1409,7 +1392,7 @@ def list_all_versions(pkg,
         The user under which to run pip
 
     cwd
-        Current working directory to run pip from
+        Directory from which to run pip
 
     CLI Example:
 
