@@ -136,6 +136,7 @@ def search(pkg_name):
 def latest_version(*names, **kwargs):
     '''
     .. versionchanged: 2016.3.0
+
     Return the latest version of the named package available for upgrade or
     installation.
 
@@ -244,6 +245,7 @@ def refresh_db():
 def list_pkgs(versions_as_list=False, **kwargs):
     '''
     .. versionchanged: 2016.3.0
+
     List the packages currently installed as a dict::
 
         {'<package_name>': '<version>'}
@@ -594,6 +596,7 @@ def file_list(package):
 def file_dict(*packages):
     '''
     .. versionchanged: 2016.3.0
+
     List the files that belong to a package.
 
     CLI Examples:
@@ -621,7 +624,7 @@ def file_dict(*packages):
                 continue  # unexpected string
 
     ret = {'errors': errors, 'files': files}
-    for field in ret.keys():
+    for field in ret:
         if not ret[field] or ret[field] == '':
             del ret[field]
     return ret
