@@ -10,6 +10,8 @@ from __future__ import absolute_import
 import json
 import re
 import logging
+import os
+import os.path
 import random
 import string
 
@@ -23,24 +25,86 @@ from salt.exceptions import CommandExecutionError
 
 log = logging.getLogger(__name__)
 
+RABBITMQCTL = None
+RABBITMQ_PLUGINS = None
+
 
 def __virtual__():
     '''
     Verify RabbitMQ is installed.
     '''
-    return salt.utils.which('rabbitmqctl') is not None
+    global RABBITMQCTL
+    global RABBITMQ_PLUGINS
+
+    if salt.utils.is_windows():
+        from salt.ext.six.moves import winreg
+        key = None
+        try:
+            key = winreg.OpenKeyEx(
+                winreg.HKEY_LOCAL_MACHINE,
+                'SOFTWARE\\VMware, Inc.\\RabbitMQ Server',
+                0,
+                winreg.KEY_READ | winreg.KEY_WOW64_32KEY
+            )
+            (dir_path, value_type) = winreg.QueryValueEx(
+                key,
+                'Install_Dir'
+            )
+            if value_type != winreg.REG_SZ:
+                raise TypeError('Invalid RabbitMQ Server directory type: {0}'.format(value_type))
+            if not os.path.isdir(dir_path):
+                raise IOError('RabbitMQ directory not found: {0}'.format(dir_path))
+            subdir_match = ''
+            for name in os.listdir(dir_path):
+                if name.startswith('rabbitmq_server-'):
+                    subdir_path = os.path.join(dir_path, name)
+                    # Get the matching entry that is last in ASCII order.
+                    if os.path.isdir(subdir_path) and subdir_path > subdir_match:
+                        subdir_match = subdir_path
+            if not subdir_match:
+                raise IOError('"rabbitmq_server-*" subdirectory not found in: {0}'.format(dir_path))
+            RABBITMQCTL = os.path.join(subdir_match, 'sbin', 'rabbitmqctl.bat')
+            RABBITMQ_PLUGINS = os.path.join(subdir_match, 'sbin', 'rabbitmq-plugins.bat')
+        except Exception:
+            pass
+        finally:
+            if key is not None:
+                winreg.CloseKey(key)
+    else:
+        RABBITMQCTL = salt.utils.which('rabbitmqctl')
+        RABBITMQ_PLUGINS = salt.utils.which('rabbitmq-plugins')
+
+    if not RABBITMQCTL:
+        return (False, 'Module rabbitmq: module only works when RabbitMQ is installed')
+    return True
+
+
+def _check_response(response):
+    if isinstance(response, dict):
+        if response['retcode'] != 0 or response['stderr']:
+            raise CommandExecutionError(
+                'RabbitMQ command failed: {0}'.format(response['stderr'])
+            )
+    else:
+        if 'Error' in response:
+            raise CommandExecutionError(
+                'RabbitMQ command failed: {0}'.format(response)
+            )
 
 
 def _format_response(response, msg):
-    error = 'RabbitMQ command failed: {0}'.format(response)
     if isinstance(response, dict):
-        if response['retcode'] != 0:
-            raise CommandExecutionError(error)
+        if response['retcode'] != 0 or response['stderr']:
+            raise CommandExecutionError(
+                'RabbitMQ command failed: {0}'.format(response['stderr'])
+            )
         else:
-            msg = response['stdout']
+            response = response['stdout']
     else:
         if 'Error' in response:
-            raise CommandExecutionError(error)
+            raise CommandExecutionError(
+                'RabbitMQ command failed: {0}'.format(response)
+            )
     return {
         msg: response
     }
@@ -53,15 +117,14 @@ def _get_rabbitmq_plugin():
     This works by taking the rabbitmq-server version and looking for where it
     seems to be hidden in /usr/lib.
     '''
-    rabbitmq = salt.utils.which('rabbitmq-plugins')
+    global RABBITMQ_PLUGINS
 
-    if rabbitmq is None:
+    if RABBITMQ_PLUGINS is None:
         version = __salt__['pkg.version']('rabbitmq-server').split('-')[0]
+        RABBITMQ_PLUGINS = ('/usr/lib/rabbitmq/lib/rabbitmq_server-{0}'
+                            '/sbin/rabbitmq-plugins').format(version)
 
-        rabbitmq = ('/usr/lib/rabbitmq/lib/rabbitmq_server-{0}'
-                    '/sbin/rabbitmq-plugins').format(version)
-
-    return rabbitmq
+    return RABBITMQ_PLUGINS
 
 
 def _safe_output(line):
@@ -94,6 +157,13 @@ def _output_to_dict(cmdoutput, values_mapper=None):
     cmdoutput: string output of rabbitmqctl commands
     values_mapper: function object to process the values part of each line
     '''
+    if isinstance(cmdoutput, dict):
+        if cmdoutput['retcode'] != 0 or cmdoutput['stderr']:
+            raise CommandExecutionError(
+                'RabbitMQ command failed: {0}'.format(cmdoutput['stderr'])
+            )
+        cmdoutput = cmdoutput['stdout']
+
     ret = {}
     if values_mapper is None:
         values_mapper = lambda string: string.split('\t')
@@ -118,6 +188,24 @@ def _output_to_dict(cmdoutput, values_mapper=None):
     return ret
 
 
+def _output_to_list(cmdoutput):
+    '''
+    Convert rabbitmqctl output to a list of strings (assuming whitespace-delimited output).
+    Ignores output lines that shouldn't be parsed, like warnings.
+    cmdoutput: string output of rabbitmqctl commands
+    '''
+    return [item for line in cmdoutput.splitlines() if _safe_output(line) for item in line.split()]
+
+
+def _output_lines_to_list(cmdoutput):
+    '''
+    Convert rabbitmqctl output to a list of strings (assuming newline-delimited output).
+    Ignores output lines that shouldn't be parsed, like warnings.
+    cmdoutput: string output of rabbitmqctl commands
+    '''
+    return [line.strip() for line in cmdoutput.splitlines() if _safe_output(line)]
+
+
 def list_users(runas=None):
     '''
     Return a list of users based off of rabbitmqctl user_list.
@@ -128,10 +216,14 @@ def list_users(runas=None):
 
         salt '*' rabbitmq.list_users
     '''
-    if runas is None:
+    # Windows runas currently requires a password.
+    # Due to this, don't use a default value for
+    # runas in Windows.
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'list_users'],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'list_users', '-q'],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
 
@@ -151,11 +243,15 @@ def list_vhosts(runas=None):
 
         salt '*' rabbitmq.list_vhosts
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run']('rabbitmqctl list_vhosts -q',
-                              runas=runas).splitlines()
-    return res
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'list_vhosts', '-q'],
+        reset_system_locale=False,
+        runas=runas,
+        python_shell=False)
+    _check_response(res)
+    return _output_to_list(res['stdout'])
 
 
 def user_exists(name, runas=None):
@@ -168,12 +264,9 @@ def user_exists(name, runas=None):
 
         salt '*' rabbitmq.user_exists rabbit_user
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    user_list = list_users(runas=runas)
-    log.debug(user_list)
-
-    return name in user_list
+    return name in list_users(runas=runas)
 
 
 def vhost_exists(name, runas=None):
@@ -186,7 +279,7 @@ def vhost_exists(name, runas=None):
 
         salt '*' rabbitmq.vhost_exists rabbit_host
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
     return name in list_vhosts(runas=runas)
 
@@ -209,24 +302,40 @@ def add_user(name, password=None, runas=None):
         password = ''.join(random.SystemRandom().choice(
             string.ascii_uppercase + string.digits) for x in range(15))
 
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'add_user', name, password],
+
+    if salt.utils.is_windows():
+        # On Windows, if the password contains a special character
+        # such as '|', normal execution will fail. For example:
+        # cmd: rabbitmq.add_user abc "asdf|def"
+        # stderr: 'def' is not recognized as an internal or external
+        #         command,\r\noperable program or batch file.
+        # Work around this by using a shell and a quoted command.
+        python_shell = True
+        cmd = '"{0}" add_user "{1}" "{2}"'.format(
+            RABBITMQCTL, name, password
+        )
+    else:
+        python_shell = False
+        cmd = [RABBITMQCTL, 'add_user', name, password]
+
+    res = __salt__['cmd.run_all'](
+        cmd,
+        reset_system_locale=False,
         output_loglevel='quiet',
         runas=runas,
-        python_shell=False)
+        python_shell=python_shell)
 
     if clear_pw:
         # Now, Clear the random password from the account, if necessary
-        res2 = clear_password(name, runas)
-
-        if 'Error' in res2:
+        try:
+            clear_password(name, runas)
+        except Exception:
             # Clearing the password failed. We should try to cleanup
             # and rerun and error.
             delete_user(name, runas)
-            msg = 'Error'
-            return _format_response(res2, msg)
+            raise
 
     msg = 'Added'
     return _format_response(res, msg)
@@ -242,10 +351,11 @@ def delete_user(name, runas=None):
 
         salt '*' rabbitmq.delete_user rabbit_user
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'delete_user', name],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'delete_user', name],
+        reset_system_locale=False,
         python_shell=False,
         runas=runas)
     msg = 'Deleted'
@@ -263,13 +373,28 @@ def change_password(name, password, runas=None):
 
         salt '*' rabbitmq.change_password rabbit_user password
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'change_password', name, password],
+    if salt.utils.is_windows():
+        # On Windows, if the password contains a special character
+        # such as '|', normal execution will fail. For example:
+        # cmd: rabbitmq.add_user abc "asdf|def"
+        # stderr: 'def' is not recognized as an internal or external
+        #         command,\r\noperable program or batch file.
+        # Work around this by using a shell and a quoted command.
+        python_shell = True
+        cmd = '"{0}" change_password "{1}" "{2}"'.format(
+            RABBITMQCTL, name, password
+        )
+    else:
+        python_shell = False
+        cmd = [RABBITMQCTL, 'change_password', name, password]
+    res = __salt__['cmd.run_all'](
+        cmd,
+        reset_system_locale=False,
         runas=runas,
         output_loglevel='quiet',
-        python_shell=False)
+        python_shell=python_shell)
     msg = 'Password Changed'
 
     return _format_response(res, msg)
@@ -285,10 +410,11 @@ def clear_password(name, runas=None):
 
         salt '*' rabbitmq.clear_password rabbit_user
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'clear_password', name],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'clear_password', name],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
     msg = 'Password Cleared'
@@ -310,11 +436,11 @@ def check_password(name, password, runas=None):
     '''
     # try to get the rabbitmq-version - adapted from _get_rabbitmq_plugin
 
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
 
     try:
-        res = __salt__['cmd.run'](['rabbitmqctl', 'status'], runas=runas, python_shell=False)
+        res = __salt__['cmd.run']([RABBITMQCTL, 'status'], reset_system_locale=False, runas=runas, python_shell=False)
         server_version = re.search(r'\{rabbit,"RabbitMQ","(.+)"\}', res)
 
         if server_version is None:
@@ -329,21 +455,40 @@ def check_password(name, password, runas=None):
 
     # rabbitmq introduced a native api to check a username and password in version 3.5.7.
     if tuple(version) >= (3, 5, 7):
-        res = __salt__['cmd.run'](
-            ['rabbitmqctl', 'authenticate_user', name, password],
+        if salt.utils.is_windows():
+            # On Windows, if the password contains a special character
+            # such as '|', normal execution will fail. For example:
+            # cmd: rabbitmq.add_user abc "asdf|def"
+            # stderr: 'def' is not recognized as an internal or external
+            #         command,\r\noperable program or batch file.
+            # Work around this by using a shell and a quoted command.
+            python_shell = True
+            cmd = '"{0}" authenticate_user "{1}" "{2}"'.format(
+                RABBITMQCTL, name, password
+            )
+        else:
+            python_shell = False
+            cmd = [RABBITMQCTL, 'authenticate_user', name, password]
+
+        res = __salt__['cmd.run_all'](
+            cmd,
+            reset_system_locale=False,
             runas=runas,
             output_loglevel='quiet',
-            python_shell=False)
+            python_shell=python_shell)
 
-        return 'Error:' not in res
+        if res['retcode'] != 0 or res['stderr']:
+            return False
+        return True
 
     cmd = ('rabbit_auth_backend_internal:check_user_login'
         '(<<"{0}">>, [{{password, <<"{1}">>}}]).').format(
         name.replace('"', '\\"'),
         password.replace('"', '\\"'))
 
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'eval', cmd],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'eval', cmd],
+        reset_system_locale=False,
         runas=runas,
         output_loglevel='quiet',
         python_shell=False)
@@ -368,10 +513,11 @@ def add_vhost(vhost, runas=None):
 
         salt '*' rabbitmq add_vhost '<vhost_name>'
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'add_vhost', vhost],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'add_vhost', vhost],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
 
@@ -389,10 +535,11 @@ def delete_vhost(vhost, runas=None):
 
         salt '*' rabbitmq.delete_vhost '<vhost_name>'
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'delete_vhost', vhost],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'delete_vhost', vhost],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
     msg = 'Deleted'
@@ -407,13 +554,14 @@ def set_permissions(vhost, user, conf='.*', write='.*', read='.*', runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.set_permissions 'myvhost' 'myuser'
+        salt '*' rabbitmq.set_permissions myvhost myuser
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'set_permissions', '-p',
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'set_permissions', '-p',
          vhost, user, conf, write, read],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
     msg = 'Permissions Set'
@@ -428,12 +576,13 @@ def list_permissions(vhost, runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.list_permissions '/myvhost'
+        salt '*' rabbitmq.list_permissions /myvhost
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'list_permissions', '-p', vhost],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'list_permissions', '-q', '-p', vhost],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
 
@@ -448,12 +597,13 @@ def list_user_permissions(name, runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.list_user_permissions 'user'.
+        salt '*' rabbitmq.list_user_permissions user
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'list_user_permissions', name],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'list_user_permissions', name, '-q'],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
 
@@ -467,16 +617,17 @@ def set_user_tags(name, tags, runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.set_user_tags 'myadmin' 'administrator'
+        salt '*' rabbitmq.set_user_tags myadmin administrator
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
 
     if not isinstance(tags, (list, tuple)):
         tags = [tags]
 
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'set_user_tags', name] + list(tags),
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'set_user_tags', name] + list(tags),
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
     msg = "Tag(s) set"
@@ -493,13 +644,15 @@ def status(runas=None):
 
         salt '*' rabbitmq.status
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'status'],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'status'],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
-    return res
+    _check_response(res)
+    return res['stdout']
 
 
 def cluster_status(runas=None):
@@ -512,14 +665,15 @@ def cluster_status(runas=None):
 
         salt '*' rabbitmq.cluster_status
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'cluster_status'],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'cluster_status'],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
-
-    return res
+    _check_response(res)
+    return res['stdout']
 
 
 def join_cluster(host, user='rabbit', ram_node=None, runas=None):
@@ -530,17 +684,17 @@ def join_cluster(host, user='rabbit', ram_node=None, runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.join_cluster 'rabbit.example.com' 'rabbit'
+        salt '*' rabbitmq.join_cluster rabbit.example.com rabbit
     '''
-    cmd = ['rabbitmqctl', 'join_cluster']
+    cmd = [RABBITMQCTL, 'join_cluster']
     if ram_node:
         cmd.append('--ram')
     cmd.append('{0}@{1}'.format(user, host))
 
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
     stop_app(runas)
-    res = __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
+    res = __salt__['cmd.run_all'](cmd, reset_system_locale=False, runas=runas, python_shell=False)
     start_app(runas)
 
     return _format_response(res, 'Join')
@@ -556,12 +710,15 @@ def stop_app(runas=None):
 
         salt '*' rabbitmq.stop_app
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    return __salt__['cmd.run'](
-        ['rabbitmqctl', 'stop_app'],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'stop_app'],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
+    _check_response(res)
+    return res['stdout']
 
 
 def start_app(runas=None):
@@ -574,12 +731,15 @@ def start_app(runas=None):
 
         salt '*' rabbitmq.start_app
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    return __salt__['cmd.run'](
-        ['rabbitmqctl', 'start_app'],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'start_app'],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
+    _check_response(res)
+    return res['stdout']
 
 
 def reset(runas=None):
@@ -592,12 +752,15 @@ def reset(runas=None):
 
         salt '*' rabbitmq.reset
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    return __salt__['cmd.run'](
-        ['rabbitmqctl', 'reset'],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'reset'],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
+    _check_response(res)
+    return res['stdout']
 
 
 def force_reset(runas=None):
@@ -610,12 +773,15 @@ def force_reset(runas=None):
 
         salt '*' rabbitmq.force_reset
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    return __salt__['cmd.run'](
-        ['rabbitmqctl', 'force_reset'],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'force_reset'],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
+    _check_response(res)
+    return res['stdout']
 
 
 def list_queues(runas=None, *args):
@@ -628,11 +794,13 @@ def list_queues(runas=None, *args):
 
         salt '*' rabbitmq.list_queues messages consumers
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    cmd = ['rabbitmqctl', 'list_queues']
+    cmd = [RABBITMQCTL, 'list_queues', '-q']
     cmd.extend(args)
-    return __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
+    res = __salt__['cmd.run_all'](cmd, reset_system_locale=False, runas=runas, python_shell=False)
+    _check_response(res)
+    return _output_to_dict(res['stdout'])
 
 
 def list_queues_vhost(vhost, runas=None, *args):
@@ -648,11 +816,13 @@ def list_queues_vhost(vhost, runas=None, *args):
 
         salt '*' rabbitmq.list_queues messages consumers
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    cmd = ['rabbitmqctl', 'list_queues', '-p', vhost]
+    cmd = [RABBITMQCTL, 'list_queues', '-q', '-p', vhost]
     cmd.extend(args)
-    return __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
+    res = __salt__['cmd.run_all'](cmd, reset_system_locale=False, runas=runas, python_shell=False)
+    _check_response(res)
+    return _output_to_dict(res['stdout'])
 
 
 def list_policies(vhost="/", runas=None):
@@ -666,35 +836,37 @@ def list_policies(vhost="/", runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.list_policies'
+        salt '*' rabbitmq.list_policies
     '''
     ret = {}
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    output = __salt__['cmd.run'](
-        ['rabbitmqctl', 'list_policies', '-p', vhost],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'list_policies', '-q', '-p', vhost],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
-    for line in salt.utils.itertools.split(output, '\n'):
-        if '...' not in line:
-            parts = line.split('\t')
-            if len(parts) not in (5, 6):
-                continue
-            vhost, name = parts[0], parts[1]
-            if vhost not in ret:
-                ret[vhost] = {}
-            ret[vhost][name] = {}
-            # How many fields are there? - 'apply_to' was inserted in position
-            # 2 at some point
-            offset = len(parts) - 5
-            if len(parts) == 6:
-                ret[vhost][name]['apply_to'] = parts[2]
-            ret[vhost][name].update({
-                'pattern': parts[offset+2],
-                'definition': parts[offset+3],
-                'priority': parts[offset+4]
-            })
-    log.debug('Listing policies: {0}'.format(ret))
+    _check_response(res)
+    output = res['stdout']
+    for line in _output_lines_to_list(output):
+        parts = line.split('\t')
+        if len(parts) not in (5, 6):
+            continue
+        vhost, name = parts[0], parts[1]
+        if vhost not in ret:
+            ret[vhost] = {}
+        ret[vhost][name] = {}
+        # How many fields are there? - 'apply_to' was inserted in position
+        # 2 at some point
+        offset = len(parts) - 5
+        if len(parts) == 6:
+            ret[vhost][name]['apply_to'] = parts[2]
+        ret[vhost][name].update({
+            'pattern': parts[offset+2],
+            'definition': parts[offset+3],
+            'priority': parts[offset+4]
+        })
+
     return ret
 
 
@@ -710,7 +882,7 @@ def set_policy(vhost, name, pattern, definition, priority=None, runas=None):
 
         salt '*' rabbitmq.set_policy / HA '.*' '{"ha-mode":"all"}'
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
     if isinstance(definition, dict):
         definition = json.dumps(definition)
@@ -718,12 +890,12 @@ def set_policy(vhost, name, pattern, definition, priority=None, runas=None):
         raise SaltInvocationError(
             'The \'definition\' argument must be a dictionary or JSON string'
         )
-    cmd = ['rabbitmqctl', 'set_policy', '-p', vhost]
+    cmd = [RABBITMQCTL, 'set_policy', '-p', vhost]
     if priority:
         cmd.extend(['--priority', priority])
     cmd.extend([name, pattern, definition])
-    res = __salt__['cmd.run'](cmd, runas=runas, python_shell=False)
-    log.debug('Set policy: {0}'.format(res))
+    res = __salt__['cmd.run_all'](cmd, reset_system_locale=False, runas=runas, python_shell=False)
+    log.debug('Set policy: %s', res['stdout'])
     return _format_response(res, 'Set')
 
 
@@ -737,15 +909,16 @@ def delete_policy(vhost, name, runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.delete_policy / HA'
+        salt '*' rabbitmq.delete_policy / HA
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    res = __salt__['cmd.run'](
-        ['rabbitmqctl', 'clear_policy', '-p', vhost, name],
+    res = __salt__['cmd.run_all'](
+        [RABBITMQCTL, 'clear_policy', '-p', vhost, name],
+        reset_system_locale=False,
         runas=runas,
         python_shell=False)
-    log.debug('Delete policy: {0}'.format(res))
+    log.debug('Delete policy: {0}'.format(res['stdout']))
     return _format_response(res, 'Deleted')
 
 
@@ -761,12 +934,48 @@ def policy_exists(vhost, name, runas=None):
 
         salt '*' rabbitmq.policy_exists / HA
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
     policies = list_policies(runas=runas)
     return bool(vhost in policies and name in policies[vhost])
 
 
+def list_available_plugins(runas=None):
+    '''
+        Returns a list of the names of all available plugins (enabled and disabled).
+
+        CLI Example:
+
+        .. code-block:: bash
+
+            salt '*' rabbitmq.list_available_plugins
+        '''
+    if runas is None and not salt.utils.is_windows():
+        runas = salt.utils.get_user()
+    cmd = [_get_rabbitmq_plugin(), 'list', '-m']
+    ret = __salt__['cmd.run_all'](cmd, reset_system_locale=False, python_shell=False, runas=runas)
+    _check_response(ret)
+    return _output_to_list(ret['stdout'])
+
+
+def list_enabled_plugins(runas=None):
+    '''
+        Returns a list of the names of the enabled plugins.
+
+        CLI Example:
+
+        .. code-block:: bash
+
+            salt '*' rabbitmq.list_enabled_plugins
+        '''
+    if runas is None and not salt.utils.is_windows():
+        runas = salt.utils.get_user()
+    cmd = [_get_rabbitmq_plugin(), 'list', '-m', '-e']
+    ret = __salt__['cmd.run_all'](cmd, reset_system_locale=False, python_shell=False, runas=runas)
+    _check_response(ret)
+    return _output_to_list(ret['stdout'])
+
+
 def plugin_is_enabled(name, runas=None):
     '''
     Return whether the plugin is enabled.
@@ -775,17 +984,11 @@ def plugin_is_enabled(name, runas=None):
 
     .. code-block:: bash
 
-        salt '*' rabbitmq.plugin_is_enabled foo
+        salt '*' rabbitmq.plugin_is_enabled rabbitmq_plugin_name
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
-    cmd = [_get_rabbitmq_plugin(), 'list', '-m', '-e']
-    ret = __salt__['cmd.run_all'](cmd, python_shell=False, runas=runas)
-    if ret['retcode'] != 0:
-        raise CommandExecutionError(
-            'RabbitMQ command failed: {0}'.format(ret['stderr'])
-        )
-    return bool(name in ret['stdout'])
+    return name in list_enabled_plugins(runas)
 
 
 def enable_plugin(name, runas=None):
@@ -798,10 +1001,10 @@ def enable_plugin(name, runas=None):
 
         salt '*' rabbitmq.enable_plugin foo
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
     cmd = [_get_rabbitmq_plugin(), 'enable', name]
-    ret = __salt__['cmd.run_all'](cmd, runas=runas, python_shell=False)
+    ret = __salt__['cmd.run_all'](cmd, reset_system_locale=False, runas=runas, python_shell=False)
     return _format_response(ret, 'Enabled')
 
 
@@ -815,8 +1018,8 @@ def disable_plugin(name, runas=None):
 
         salt '*' rabbitmq.disable_plugin foo
     '''
-    if runas is None:
+    if runas is None and not salt.utils.is_windows():
         runas = salt.utils.get_user()
     cmd = [_get_rabbitmq_plugin(), 'disable', name]
-    ret = __salt__['cmd.run_all'](cmd, runas=runas, python_shell=False)
+    ret = __salt__['cmd.run_all'](cmd, reset_system_locale=False, runas=runas, python_shell=False)
     return _format_response(ret, 'Disabled')
