@@ -19,6 +19,7 @@ Module to provide redis functionality to Salt
 from __future__ import absolute_import
 from salt.ext.six.moves import zip
 from salt.ext import six
+from salt.utils import clean_kwargs
 from datetime import datetime
 
 # Import third party libs
@@ -395,7 +396,7 @@ def hmset(key, **fieldsvals):
     database = fieldsvals.pop('db', None)
     password = fieldsvals.pop('password', None)
     server = _connect(host, port, database, password)
-    return server.hmset(key, **fieldsvals)
+    return server.hmset(key, clean_kwargs(**fieldsvals))
 
 
 def hset(key, field, value, host=None, port=None, db=None, password=None):
