@@ -91,7 +91,7 @@ def _get_top_file_envs():
         return envs
 
 
-def _sync(form, saltenv=None):
+def _sync(form, saltenv=None, extmod_whitelist=None, extmod_blacklist=None):
     '''
     Sync the given directory in the given environment
     '''
@@ -99,7 +99,8 @@ def _sync(form, saltenv=None):
         saltenv = _get_top_file_envs()
     if isinstance(saltenv, six.string_types):
         saltenv = saltenv.split(',')
-    ret, touched = salt.utils.extmods.sync(__opts__, form, saltenv=saltenv)
+    ret, touched = salt.utils.extmods.sync(__opts__, form, saltenv=saltenv, extmod_whitelist=extmod_whitelist,
+                                           extmod_blacklist=extmod_blacklist)
     # Dest mod_dir is touched? trigger reload if requested
     if touched:
         mod_file = os.path.join(__opts__['cachedir'], 'module_refresh')
@@ -181,7 +182,7 @@ def update(version=None):
     return ret
 
 
-def sync_beacons(saltenv=None, refresh=True):
+def sync_beacons(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.5.1
 
@@ -200,6 +201,12 @@ def sync_beacons(saltenv=None, refresh=True):
         will be performed even if no new beacons are synced. Set to ``False``
         to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Example:
 
     .. code-block:: bash
@@ -208,13 +215,13 @@ def sync_beacons(saltenv=None, refresh=True):
         salt '*' saltutil.sync_beacons saltenv=dev
         salt '*' saltutil.sync_beacons saltenv=base,dev
     '''
-    ret = _sync('beacons', saltenv)
+    ret = _sync('beacons', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_beacons()
     return ret
 
 
-def sync_sdb(saltenv=None):
+def sync_sdb(saltenv=None, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.5.8,2015.8.3
 
@@ -232,6 +239,12 @@ def sync_sdb(saltenv=None):
         This argument has no affect and is included for consistency with the
         other sync functions.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Example:
 
     .. code-block:: bash
@@ -240,11 +253,11 @@ def sync_sdb(saltenv=None):
         salt '*' saltutil.sync_sdb saltenv=dev
         salt '*' saltutil.sync_sdb saltenv=base,dev
     '''
-    ret = _sync('sdb', saltenv)
+    ret = _sync('sdb', saltenv, extmod_whitelist, extmod_blacklist)
     return ret
 
 
-def sync_modules(saltenv=None, refresh=True):
+def sync_modules(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
@@ -280,6 +293,12 @@ def sync_modules(saltenv=None, refresh=True):
         See :ref:`here <reloading-modules>` for a more detailed explanation of
         why this is necessary.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Example:
 
     .. code-block:: bash
@@ -288,13 +307,13 @@ def sync_modules(saltenv=None, refresh=True):
         salt '*' saltutil.sync_modules saltenv=dev
         salt '*' saltutil.sync_modules saltenv=base,dev
     '''
-    ret = _sync('modules', saltenv)
+    ret = _sync('modules', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_states(saltenv=None, refresh=True):
+def sync_states(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
@@ -313,6 +332,12 @@ def sync_states(saltenv=None, refresh=True):
         will be performed even if no new state modules are synced. Set to
         ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -321,7 +346,7 @@ def sync_states(saltenv=None, refresh=True):
         salt '*' saltutil.sync_states saltenv=dev
         salt '*' saltutil.sync_states saltenv=base,dev
     '''
-    ret = _sync('states', saltenv)
+    ret = _sync('states', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
@@ -329,7 +354,7 @@ def sync_states(saltenv=None, refresh=True):
 
 def refresh_grains(**kwargs):
     '''
-    .. versionadded:: 2016.3.6,2016.11.4,Nitrogen
+    .. versionadded:: 2016.3.6,2016.11.4,2017.7.0
 
     Refresh the minion's grains without syncing custom grains modules from
     ``salt://_grains``.
@@ -360,7 +385,7 @@ def refresh_grains(**kwargs):
     return True
 
 
-def sync_grains(saltenv=None, refresh=True):
+def sync_grains(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
@@ -380,6 +405,12 @@ def sync_grains(saltenv=None, refresh=True):
         new grains modules are synced. Set to ``False`` to prevent this
         refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -388,14 +419,14 @@ def sync_grains(saltenv=None, refresh=True):
         salt '*' saltutil.sync_grains saltenv=dev
         salt '*' saltutil.sync_grains saltenv=base,dev
     '''
-    ret = _sync('grains', saltenv)
+    ret = _sync('grains', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
         refresh_pillar()
     return ret
 
 
-def sync_renderers(saltenv=None, refresh=True):
+def sync_renderers(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
@@ -415,6 +446,12 @@ def sync_renderers(saltenv=None, refresh=True):
         Set to ``False`` to prevent this refresh. Set to ``False`` to prevent
         this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -423,13 +460,13 @@ def sync_renderers(saltenv=None, refresh=True):
         salt '*' saltutil.sync_renderers saltenv=dev
         salt '*' saltutil.sync_renderers saltenv=base,dev
     '''
-    ret = _sync('renderers', saltenv)
+    ret = _sync('renderers', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_returners(saltenv=None, refresh=True):
+def sync_returners(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 0.10.0
 
@@ -448,6 +485,12 @@ def sync_returners(saltenv=None, refresh=True):
         This refresh will be performed even if no new returners are synced. Set
         to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -455,13 +498,13 @@ def sync_returners(saltenv=None, refresh=True):
         salt '*' saltutil.sync_returners
         salt '*' saltutil.sync_returners saltenv=dev
     '''
-    ret = _sync('returners', saltenv)
+    ret = _sync('returners', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_proxymodules(saltenv=None, refresh=False):
+def sync_proxymodules(saltenv=None, refresh=False, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.8.2
 
@@ -480,6 +523,12 @@ def sync_proxymodules(saltenv=None, refresh=False):
         This refresh will be performed even if no new proxy modules are synced.
         Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -488,13 +537,13 @@ def sync_proxymodules(saltenv=None, refresh=False):
         salt '*' saltutil.sync_proxymodules saltenv=dev
         salt '*' saltutil.sync_proxymodules saltenv=base,dev
     '''
-    ret = _sync('proxy', saltenv)
+    ret = _sync('proxy', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_engines(saltenv=None, refresh=False):
+def sync_engines(saltenv=None, refresh=False, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2016.3.0
 
@@ -513,6 +562,12 @@ def sync_engines(saltenv=None, refresh=False):
         This refresh will be performed even if no new engine modules are synced.
         Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -520,13 +575,13 @@ def sync_engines(saltenv=None, refresh=False):
         salt '*' saltutil.sync_engines
         salt '*' saltutil.sync_engines saltenv=base,dev
     '''
-    ret = _sync('engines', saltenv)
+    ret = _sync('engines', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_output(saltenv=None, refresh=True):
+def sync_output(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     Sync outputters from ``salt://_output`` to the minion
 
@@ -543,6 +598,12 @@ def sync_output(saltenv=None, refresh=True):
         This refresh will be performed even if no new outputters are synced.
         Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -551,7 +612,7 @@ def sync_output(saltenv=None, refresh=True):
         salt '*' saltutil.sync_output saltenv=dev
         salt '*' saltutil.sync_output saltenv=base,dev
     '''
-    ret = _sync('output', saltenv)
+    ret = _sync('output', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
@@ -559,7 +620,42 @@ def sync_output(saltenv=None, refresh=True):
 sync_outputters = salt.utils.alias_function(sync_output, 'sync_outputters')
 
 
-def sync_utils(saltenv=None, refresh=True):
+def sync_clouds(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
+    '''
+    .. versionadded:: 2017.7.0
+
+    Sync utility modules from ``salt://_cloud`` to the minion
+
+    saltenv : base
+        The fileserver environment from which to sync. To sync from more than
+        one environment, pass a comma-separated list.
+
+    refresh : True
+        If ``True``, refresh the available execution modules on the minion.
+        This refresh will be performed even if no new utility modules are
+        synced. Set to ``False`` to prevent this refresh.
+
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.sync_clouds
+        salt '*' saltutil.sync_clouds saltenv=dev
+        salt '*' saltutil.sync_clouds saltenv=base,dev
+    '''
+    ret = _sync('clouds', saltenv, extmod_whitelist, extmod_blacklist)
+    if refresh:
+        refresh_modules()
+    return ret
+
+
+def sync_utils(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2014.7.0
 
@@ -578,6 +674,12 @@ def sync_utils(saltenv=None, refresh=True):
         This refresh will be performed even if no new utility modules are
         synced. Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -586,13 +688,37 @@ def sync_utils(saltenv=None, refresh=True):
         salt '*' saltutil.sync_utils saltenv=dev
         salt '*' saltutil.sync_utils saltenv=base,dev
     '''
-    ret = _sync('utils', saltenv)
+    ret = _sync('utils', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_log_handlers(saltenv=None, refresh=True):
+def list_extmods():
+    '''
+    .. versionadded:: 2017.7.0
+
+    List Salt modules which have been synced externally
+
+    CLI Examples:
+
+    .. code-block:: bash
+
+        salt '*' saltutil.list_extmods
+    '''
+    ret = {}
+    ext_dir = os.path.join(__opts__['cachedir'], 'extmods')
+    mod_types = os.listdir(ext_dir)
+    for mod_type in mod_types:
+        ret[mod_type] = set()
+        for _, _, files in os.walk(os.path.join(ext_dir, mod_type)):
+            for fh_ in files:
+                ret[mod_type].add(fh_.split('.')[0])
+        ret[mod_type] = list(ret[mod_type])
+    return ret
+
+
+def sync_log_handlers(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.8.0
 
@@ -611,6 +737,12 @@ def sync_log_handlers(saltenv=None, refresh=True):
         This refresh will be performed even if no new log handlers are synced.
         Set to ``False`` to prevent this refresh.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -619,13 +751,13 @@ def sync_log_handlers(saltenv=None, refresh=True):
         salt '*' saltutil.sync_log_handlers saltenv=dev
         salt '*' saltutil.sync_log_handlers saltenv=base,dev
     '''
-    ret = _sync('log_handlers', saltenv)
+    ret = _sync('log_handlers', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
     return ret
 
 
-def sync_pillar(saltenv=None, refresh=True):
+def sync_pillar(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionadded:: 2015.8.11,2016.3.2
 
@@ -638,6 +770,12 @@ def sync_pillar(saltenv=None, refresh=True):
         Also refresh the execution modules available to the minion, and refresh
         pillar data.
 
+    extmod_whitelist : None
+        comma-seperated list of modules to sync
+
+    extmod_blacklist : None
+        comma-seperated list of modules to blacklist based on type
+
     .. note::
         This function will raise an error if executed on a traditional (i.e.
         not masterless) minion
@@ -653,14 +791,14 @@ def sync_pillar(saltenv=None, refresh=True):
         raise CommandExecutionError(
             'Pillar modules can only be synced to masterless minions'
         )
-    ret = _sync('pillar', saltenv)
+    ret = _sync('pillar', saltenv, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
         refresh_pillar()
     return ret
 
 
-def sync_all(saltenv=None, refresh=True):
+def sync_all(saltenv=None, refresh=True, extmod_whitelist=None, extmod_blacklist=None):
     '''
     .. versionchanged:: 2015.8.11,2016.3.2
         On masterless minions, pillar modules are now synced, and refreshed
@@ -692,6 +830,12 @@ def sync_all(saltenv=None, refresh=True):
         See :ref:`here <reloading-modules>` for a more detailed explanation of
         why this is necessary.
 
+    extmod_whitelist : None
+        dictionary of modules to sync based on type
+
+    extmod_blacklist : None
+        dictionary of modules to blacklist based on type
+
     CLI Examples:
 
     .. code-block:: bash
@@ -699,23 +843,25 @@ def sync_all(saltenv=None, refresh=True):
         salt '*' saltutil.sync_all
         salt '*' saltutil.sync_all saltenv=dev
         salt '*' saltutil.sync_all saltenv=base,dev
+        salt '*' saltutil.sync_all extmod_whitelist={'modules': ['custom_module']}
     '''
     log.debug('Syncing all')
     ret = {}
-    ret['beacons'] = sync_beacons(saltenv, False)
-    ret['modules'] = sync_modules(saltenv, False)
-    ret['states'] = sync_states(saltenv, False)
-    ret['sdb'] = sync_sdb(saltenv)
-    ret['grains'] = sync_grains(saltenv, False)
-    ret['renderers'] = sync_renderers(saltenv, False)
-    ret['returners'] = sync_returners(saltenv, False)
-    ret['output'] = sync_output(saltenv, False)
-    ret['utils'] = sync_utils(saltenv, False)
-    ret['log_handlers'] = sync_log_handlers(saltenv, False)
-    ret['proxymodules'] = sync_proxymodules(saltenv, False)
-    ret['engines'] = sync_engines(saltenv, False)
+    ret['clouds'] = sync_clouds(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['beacons'] = sync_beacons(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['modules'] = sync_modules(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['states'] = sync_states(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['sdb'] = sync_sdb(saltenv, extmod_whitelist, extmod_blacklist)
+    ret['grains'] = sync_grains(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['renderers'] = sync_renderers(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['returners'] = sync_returners(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['output'] = sync_output(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['utils'] = sync_utils(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['log_handlers'] = sync_log_handlers(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['proxymodules'] = sync_proxymodules(saltenv, False, extmod_whitelist, extmod_blacklist)
+    ret['engines'] = sync_engines(saltenv, False, extmod_whitelist, extmod_blacklist)
     if __opts__['file_client'] == 'local':
-        ret['pillar'] = sync_pillar(saltenv, False)
+        ret['pillar'] = sync_pillar(saltenv, False, extmod_whitelist, extmod_blacklist)
     if refresh:
         refresh_modules()
         refresh_pillar()
@@ -1104,28 +1250,44 @@ def _get_ssh_or_api_client(cfgfile, ssh=False):
     return client
 
 
-def _exec(client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs):
+def _exec(client, tgt, fun, arg, timeout, tgt_type, ret, kwarg, **kwargs):
+    if 'expr_form' in kwargs:
+        salt.utils.warn_until(
+            'Fluorine',
+            'The target type should be passed using the \'tgt_type\' '
+            'argument instead of \'expr_form\'. Support for using '
+            '\'expr_form\' will be removed in Salt Fluorine.'
+        )
+        tgt_type = kwargs.pop('expr_form')
+
     fcn_ret = {}
     seen = 0
     if 'batch' in kwargs:
         _cmd = client.cmd_batch
         cmd_kwargs = {
-            'tgt': tgt, 'fun': fun, 'arg': arg, 'expr_form': expr_form,
+            'tgt': tgt, 'fun': fun, 'arg': arg, 'tgt_type': tgt_type,
             'ret': ret, 'kwarg': kwarg, 'batch': kwargs['batch'],
         }
         del kwargs['batch']
+    elif 'subset' in kwargs:
+        _cmd = client.cmd_subset
+        cmd_kwargs = {
+            'tgt': tgt, 'fun': fun, 'arg': arg, 'tgt_type': tgt_type,
+            'ret': ret, 'cli': True, 'kwarg': kwarg, 'sub': kwargs['subset'],
+        }
+        del kwargs['subset']
     else:
         _cmd = client.cmd_iter
         cmd_kwargs = {
             'tgt': tgt, 'fun': fun, 'arg': arg, 'timeout': timeout,
-            'expr_form': expr_form, 'ret': ret, 'kwarg': kwarg,
+            'tgt_type': tgt_type, 'ret': ret, 'kwarg': kwarg,
         }
     cmd_kwargs.update(kwargs)
     for ret_comp in _cmd(**cmd_kwargs):
         fcn_ret.update(ret_comp)
         seen += 1
         # fcn_ret can be empty, so we cannot len the whole return dict
-        if expr_form == 'list' and len(tgt) == seen:
+        if tgt_type == 'list' and len(tgt) == seen:
             # do not wait for timeout when explicit list matching
             # and all results are there
             break
@@ -1136,12 +1298,16 @@ def cmd(tgt,
         fun,
         arg=(),
         timeout=None,
-        expr_form='glob',
+        tgt_type='glob',
         ret='',
         kwarg=None,
         ssh=False,
         **kwargs):
     '''
+    .. versionchanged:: 2017.7.0
+        The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
+        releases must use ``expr_form``.
+
     Assuming this minion is a master, execute a salt command
 
     CLI Example:
@@ -1153,7 +1319,7 @@ def cmd(tgt,
     cfgfile = __opts__['conf_file']
     client = _get_ssh_or_api_client(cfgfile, ssh)
     fcn_ret = _exec(
-        client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
+        client, tgt, fun, arg, timeout, tgt_type, ret, kwarg, **kwargs)
     # if return is empty, we may have not used the right conf,
     # try with the 'minion relative master configuration counter part
     # if available
@@ -1165,7 +1331,7 @@ def cmd(tgt,
     ):
         client = _get_ssh_or_api_client(master_cfgfile, ssh)
         fcn_ret = _exec(
-            client, tgt, fun, arg, timeout, expr_form, ret, kwarg, **kwargs)
+            client, tgt, fun, arg, timeout, tgt_type, ret, kwarg, **kwargs)
 
     if 'batch' in kwargs:
         old_ret, fcn_ret = fcn_ret, {}
@@ -1182,12 +1348,16 @@ def cmd_iter(tgt,
              fun,
              arg=(),
              timeout=None,
-             expr_form='glob',
+             tgt_type='glob',
              ret='',
              kwarg=None,
              ssh=False,
              **kwargs):
     '''
+    .. versionchanged:: 2017.7.0
+        The ``expr_form`` argument has been renamed to ``tgt_type``, earlier
+        releases must use ``expr_form``.
+
     Assuming this minion is a master, execute a salt command
 
     CLI Example:
@@ -1205,14 +1375,14 @@ def cmd_iter(tgt,
             fun,
             arg,
             timeout,
-            expr_form,
+            tgt_type,
             ret,
             kwarg,
             **kwargs):
         yield ret
 
 
-def runner(name, **kwargs):
+def runner(name, arg=None, kwarg=None, full_return=False, saltenv='base', jid=None, **kwargs):
     '''
     Execute a runner function. This function must be run on the master,
     either by targeting a minion running on a master or by using
@@ -1234,11 +1404,17 @@ def runner(name, **kwargs):
     .. code-block:: bash
 
         salt master_minion saltutil.runner jobs.list_jobs
+        salt master_minion saltutil.runner test.arg arg="['baz']" kwarg="{'foo': 'bar'}"
     '''
-    jid = kwargs.pop('__orchestration_jid__', None)
-    saltenv = kwargs.pop('__env__', 'base')
-    full_return = kwargs.pop('full_return', False)
+    if arg is None:
+        arg = []
+    if kwarg is None:
+        kwarg = {}
+    jid = kwargs.pop('__orchestration_jid__', jid)
+    saltenv = kwargs.pop('__env__', saltenv)
     kwargs = salt.utils.clean_kwargs(**kwargs)
+    if kwargs:
+        kwarg.update(kwargs)
 
     if 'master_job_cache' not in __opts__:
         master_config = os.path.join(os.path.dirname(__opts__['conf_file']),
@@ -1251,18 +1427,22 @@ def runner(name, **kwargs):
     if name in rclient.functions:
         aspec = salt.utils.args.get_function_argspec(rclient.functions[name])
         if 'saltenv' in aspec.args:
-            kwargs['saltenv'] = saltenv
+            kwarg['saltenv'] = saltenv
+
+    if name in ['state.orchestrate', 'state.orch', 'state.sls']:
+        kwarg['orchestration_jid'] = jid
 
     if jid:
         salt.utils.event.fire_args(
             __opts__,
             jid,
-            {'type': 'runner', 'name': name, 'args': kwargs},
+            {'type': 'runner', 'name': name, 'args': arg, 'kwargs': kwarg},
             prefix='run'
         )
 
     return rclient.cmd(name,
-                       kwarg=kwargs,
+                       arg=arg,
+                       kwarg=kwarg,
                        print_event=False,
                        full_return=full_return)
 
