@@ -35,6 +35,7 @@ def __virtual__():
         'Devuan',
         'Arch',
         'Arch ARM',
+        'Manjaro',
         'ALT',
         'SUSE  Enterprise Server',
         'SUSE',
@@ -104,7 +105,7 @@ def start(name):
 
         salt '*' service.start <service name>
     '''
-    return __salt__['service.run'](name, 'start')
+    return run(name, 'start')
 
 
 def stop(name):
@@ -117,7 +118,7 @@ def stop(name):
 
         salt '*' service.stop <service name>
     '''
-    return __salt__['service.run'](name, 'stop')
+    return run(name, 'stop')
 
 
 def restart(name):
@@ -130,7 +131,7 @@ def restart(name):
 
         salt '*' service.restart <service name>
     '''
-    return __salt__['service.run'](name, 'restart')
+    return run(name, 'restart')
 
 
 def status(name, sig=None):
@@ -159,7 +160,7 @@ def reload_(name):
 
         salt '*' service.reload <service name>
     '''
-    return __salt__['service.run'](name, 'reload')
+    return run(name, 'reload')
 
 
 def available(name):
