@@ -1,19 +1,29 @@
 # -*- coding: utf-8 -*-
 '''
 Manage users with the useradd command
+
+.. important::
+
+    If you feel that Salt should be using this module to manage users on a
+    minion, and it is using a different module (or gives an error similar to
+    *'user.info' is not available*), see :ref:`here
+    <module-provider-override>`.
+
 '''
 
 # Import python libs
+from __future__ import absolute_import
 try:
     import pwd
+    HAS_PWD = True
 except ImportError:
-    pass
+    HAS_PWD = False
+import copy
 import logging
-from copy import deepcopy
 
 # Import salt libs
 import salt.utils
-from salt.ext.six import string_types
+import salt.ext.six as six
 from salt.exceptions import CommandExecutionError
 
 log = logging.getLogger(__name__)
@@ -26,8 +36,10 @@ def __virtual__():
     '''
     Set the user module if the kernel is SunOS
     '''
-
-    return __virtualname__ if __grains__['kernel'] == 'SunOS' else False
+    if __grains__['kernel'] == 'SunOS' and HAS_PWD:
+        return __virtualname__
+    return (False, 'The solaris_user execution module failed to load: '
+            'only available on Solaris systems with pwd module installed.')
 
 
 def _get_gecos(name):
@@ -58,6 +70,25 @@ def _build_gecos(gecos_dict):
                                     gecos_dict.get('homephone', ''))
 
 
+def _update_gecos(name, key, value):
+    '''
+    Common code to change a user's GECOS information
+    '''
+    if not isinstance(value, six.string_types):
+        value = str(value)
+    pre_info = _get_gecos(name)
+    if not pre_info:
+        return False
+    if value == pre_info[key]:
+        return True
+    gecos_data = copy.deepcopy(pre_info)
+    gecos_data[key] = value
+    cmd = ['usermod', '-c', _build_gecos(gecos_data), name]
+    __salt__['cmd.run'](cmd, python_shell=False)
+    post_info = info(name)
+    return _get_gecos(name).get(key) == value
+
+
 def add(name,
         uid=None,
         gid=None,
@@ -86,32 +117,26 @@ def add(name,
     if kwargs:
         log.warning('Invalid kwargs passed to user.add')
 
-    if isinstance(groups, string_types):
+    if isinstance(groups, six.string_types):
         groups = groups.split(',')
-    cmd = 'useradd '
+    cmd = ['useradd']
     if shell:
-        cmd += '-s {0} '.format(shell)
+        cmd.extend(['-s', shell])
     if uid:
-        cmd += '-u {0} '.format(uid)
+        cmd.extend(['-u', uid])
     if gid:
-        cmd += '-g {0} '.format(gid)
+        cmd.extend(['-g', gid])
     if groups:
-        cmd += '-G {0} '.format(','.join(groups))
-
-    if home is None:
-        if createhome:
-            cmd += '-m '
-    else:
-        if createhome:
-            cmd += '-m -d {0} '.format(home)
-        else:
-            cmd += '-d {0} '.format(home)
-
+        cmd.extend(['-G', ','.join(groups)])
+    if createhome:
+        cmd.append('-m')
+    if home is not None:
+        cmd.extend(['-d', home])
     if not unique:
-        cmd += '-o '
-    cmd += name
-    ret = __salt__['cmd.retcode'](cmd, python_shell=False)
-    if ret != 0:
+        cmd.append('-o')
+    cmd.append(name)
+
+    if __salt__['cmd.retcode'](cmd, python_shell=False) != 0:
         return False
     else:
         # At this point, the user was successfully created, so return true
@@ -144,16 +169,15 @@ def delete(name, remove=False, force=False):
         salt '*' user.delete name remove=True force=True
     '''
     if salt.utils.is_true(force):
-        log.error('userdel does not support force-deleting user while '
-                  'user is logged in')
-    cmd = 'userdel '
+        log.warning(
+            'userdel does not support force-deleting user while user is '
+            'logged in'
+        )
+    cmd = ['userdel']
     if remove:
-        cmd += '-r '
-    cmd += name
-
-    ret = __salt__['cmd.run_all'](cmd, python_shell=False)
-
-    return not ret['retcode']
+        cmd.append('-r')
+    cmd.append(name)
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 def getent(refresh=False):
@@ -187,14 +211,15 @@ def chuid(name, uid):
         salt '*' user.chuid foo 4376
     '''
     pre_info = info(name)
+    if not pre_info:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if uid == pre_info['uid']:
         return True
-    cmd = 'usermod -u {0} {1}'.format(uid, name)
+    cmd = ['usermod', '-u', uid, name]
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['uid'] != pre_info['uid']:
-        return post_info['uid'] == uid
-    return False
+    return info(name).get('uid') == uid
 
 
 def chgid(name, gid):
@@ -208,14 +233,15 @@ def chgid(name, gid):
         salt '*' user.chgid foo 4376
     '''
     pre_info = info(name)
+    if not pre_info:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if gid == pre_info['gid']:
         return True
-    cmd = 'usermod -g {0} {1}'.format(gid, name)
+    cmd = ['usermod', '-g', gid, name]
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['gid'] != pre_info['gid']:
-        return post_info['gid'] == gid
-    return False
+    return info(name).get('gid') == gid
 
 
 def chshell(name, shell):
@@ -229,20 +255,31 @@ def chshell(name, shell):
         salt '*' user.chshell foo /bin/zsh
     '''
     pre_info = info(name)
+    if not pre_info:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if shell == pre_info['shell']:
         return True
-    cmd = 'usermod -s {0} {1}'.format(shell, name)
+    cmd = ['usermod', '-s', shell, name]
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['shell'] != pre_info['shell']:
-        return post_info['shell'] == shell
-    return False
+    return info(name).get('shell') == shell
 
 
 def chhome(name, home, persist=False):
     '''
-    Change the home directory of the user, pass true for persist to copy files
-    to the new home dir
+    Set a new home directory for an existing user
+
+    name
+        Username to modify
+
+    home
+        New home directory to set
+
+    persist : False
+        Set to ``True`` to prevent configuration files in the new home
+        directory from being overwritten by the files from the skeleton
+        directory.
 
     CLI Example:
 
@@ -251,23 +288,35 @@ def chhome(name, home, persist=False):
         salt '*' user.chhome foo /home/users/foo True
     '''
     pre_info = info(name)
+    if not pre_info:
+        raise CommandExecutionError(
+            'User \'{0}\' does not exist'.format(name)
+        )
     if home == pre_info['home']:
         return True
-    cmd = 'usermod -d {0} '.format(home)
+    cmd = ['usermod', '-d', home]
     if persist:
-        cmd += ' -m '
-    cmd += name
+        cmd.append('-m')
+    cmd.append(name)
     __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['home'] != pre_info['home']:
-        return post_info['home'] == home
-    return False
+    return info(name).get('home') == home
 
 
 def chgroups(name, groups, append=False):
     '''
-    Change the groups this user belongs to, add append to append the specified
+    Change the groups to which a user belongs
+
+    name
+        Username to modify
+
     groups
+        List of groups to set for the user. Can be passed as a comma-separated
+        list or a Python list.
+
+    append : False
+        Set to ``True`` to append these groups to the user's existing list of
+        groups. Otherwise, the specified groups will replace any existing
+        groups for the user.
 
     CLI Example:
 
@@ -275,15 +324,15 @@ def chgroups(name, groups, append=False):
 
         salt '*' user.chgroups foo wheel,root True
     '''
-    if isinstance(groups, string_types):
+    if isinstance(groups, six.string_types):
         groups = groups.split(',')
     ugrps = set(list_groups(name))
     if ugrps == set(groups):
         return True
     if append:
-        groups += ugrps
-    cmd = 'usermod -G {0} {1} '.format(','.join(groups), name)
-    return not __salt__['cmd.retcode'](cmd, python_shell=False)
+        groups.update(ugrps)
+    cmd = ['usermod', '-G', ','.join(groups), name]
+    return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
 
 
 def chfullname(name, fullname):
@@ -296,20 +345,7 @@ def chfullname(name, fullname):
 
         salt '*' user.chfullname foo "Foo Bar"
     '''
-    fullname = str(fullname)
-    pre_info = _get_gecos(name)
-    if not pre_info:
-        return False
-    if fullname == pre_info['fullname']:
-        return True
-    gecos_field = deepcopy(pre_info)
-    gecos_field['fullname'] = fullname
-    cmd = 'usermod -c "{0}" {1}'.format(_build_gecos(gecos_field), name)
-    __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['fullname'] != pre_info['fullname']:
-        return post_info['fullname'] == fullname
-    return False
+    return _update_gecos(name, 'fullname', fullname)
 
 
 def chroomnumber(name, roomnumber):
@@ -322,20 +358,7 @@ def chroomnumber(name, roomnumber):
 
         salt '*' user.chroomnumber foo 123
     '''
-    roomnumber = str(roomnumber)
-    pre_info = _get_gecos(name)
-    if not pre_info:
-        return False
-    if roomnumber == pre_info['roomnumber']:
-        return True
-    gecos_field = deepcopy(pre_info)
-    gecos_field['roomnumber'] = roomnumber
-    cmd = 'usermod -c "{0}" {1}'.format(_build_gecos(gecos_field), name)
-    __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['roomnumber'] != pre_info['roomnumber']:
-        return post_info['roomnumber'] == roomnumber
-    return False
+    return _update_gecos(name, 'roomnumber', roomnumber)
 
 
 def chworkphone(name, workphone):
@@ -348,20 +371,7 @@ def chworkphone(name, workphone):
 
         salt '*' user.chworkphone foo "7735550123"
     '''
-    workphone = str(workphone)
-    pre_info = _get_gecos(name)
-    if not pre_info:
-        return False
-    if workphone == pre_info['workphone']:
-        return True
-    gecos_field = deepcopy(pre_info)
-    gecos_field['workphone'] = workphone
-    cmd = 'usermod -c "{0}" {1}'.format(_build_gecos(gecos_field), name)
-    __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['workphone'] != pre_info['workphone']:
-        return post_info['workphone'] == workphone
-    return False
+    return _update_gecos(name, 'workphone', workphone)
 
 
 def chhomephone(name, homephone):
@@ -374,20 +384,7 @@ def chhomephone(name, homephone):
 
         salt '*' user.chhomephone foo "7735551234"
     '''
-    homephone = str(homephone)
-    pre_info = _get_gecos(name)
-    if not pre_info:
-        return False
-    if homephone == pre_info['homephone']:
-        return True
-    gecos_field = deepcopy(pre_info)
-    gecos_field['homephone'] = homephone
-    cmd = 'usermod -c "{0}" {1}'.format(_build_gecos(gecos_field), name)
-    __salt__['cmd.run'](cmd, python_shell=False)
-    post_info = info(name)
-    if post_info['homephone'] != pre_info['homephone']:
-        return post_info['homephone'] == homephone
-    return False
+    return _update_gecos(name, 'homephone', homephone)
 
 
 def info(name):
@@ -437,6 +434,19 @@ def list_groups(name):
     return salt.utils.get_group_list(name)
 
 
+def list_users():
+    '''
+    Return a list of all users
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' user.list_users
+    '''
+    return sorted([user.pw_name for user in pwd.getpwall()])
+
+
 def rename(name, new_name):
     '''
     Change the username for a named user
@@ -449,13 +459,12 @@ def rename(name, new_name):
     '''
     current_info = info(name)
     if not current_info:
-        raise CommandExecutionError('User {0!r} does not exist'.format(name))
+        raise CommandExecutionError('User \'{0}\' does not exist'.format(name))
     new_info = info(new_name)
     if new_info:
-        raise CommandExecutionError('User {0!r} already exists'.format(new_name))
-    cmd = 'usermod -l {0} {1}'.format(new_name, name)
-    __salt__['cmd.run'](cmd)
-    post_info = info(new_name)
-    if post_info['name'] != current_info['name']:
-        return post_info['name'] == new_name
-    return False
+        raise CommandExecutionError(
+            'User \'{0}\' already exists'.format(new_name)
+        )
+    cmd = ['usermod', '-l', new_name, name]
+    __salt__['cmd.run'](cmd, python_shell=False)
+    return info(new_name).get('name') == new_name
