@@ -5,6 +5,7 @@ Module for managing timezone on POSIX-like systems.
 from __future__ import absolute_import
 
 # Import python libs
+import filecmp
 import os
 import errno
 import logging
@@ -32,7 +33,7 @@ def __virtual__():
 
     if salt.utils.is_darwin():
         return (False, 'The timezone execution module failed to load: '
-                       'mac_timezone.py should replace this module on OS X.'
+                       'mac_timezone.py should replace this module on macOS.'
                        'There was a problem loading mac_timezone.py.')
 
     return __virtualname__
@@ -61,6 +62,23 @@ def _get_zone_solaris():
     raise CommandExecutionError('Unable to get timezone from ' + tzfile)
 
 
+def _get_adjtime_timezone():
+    '''
+    Return the timezone in /etc/adjtime of the system clock
+    '''
+    adjtime_file = '/etc/adjtime'
+    if os.path.exists(adjtime_file):
+        cmd = ['tail', '-n', '1', adjtime_file]
+        return __salt__['cmd.run'](cmd, python_shell=False)
+    elif os.path.exists('/dev/rtc'):
+        raise CommandExecutionError(
+            'Unable to get hwclock timezone from ' + adjtime_file
+        )
+    else:
+        # There is no RTC.
+        return None
+
+
 def _get_zone_sysconfig():
     tzfile = '/etc/sysconfig/clock'
     with salt.utils.fopen(tzfile, 'r') as fp_:
@@ -89,6 +107,8 @@ def _get_zone_etc_localtime():
                 return get_zonecode()
             raise CommandExecutionError(tzfile + ' does not exist')
         elif exc.errno == errno.EINVAL:
+            if 'FreeBSD' in __grains__['os_family']:
+                return get_zonecode()
             log.warning(
                 tzfile + ' is not a symbolic link, attempting to match ' +
                 tzfile + ' to zoneinfo files'
@@ -110,14 +130,38 @@ def _get_zone_etc_localtime():
 
 
 def _get_zone_etc_timezone():
-    with salt.utils.fopen('/etc/timezone', 'r') as fp_:
-        return fp_.read().strip()
+    tzfile = '/etc/timezone'
+    try:
+        with salt.utils.fopen(tzfile, 'r') as fp_:
+            return fp_.read().strip()
+    except IOError as exc:
+        raise CommandExecutionError(
+            'Problem reading timezone file {0}: {1}'
+            .format(tzfile, exc.strerror)
+        )
+
+
+def _get_zone_aix():
+    tzfile = '/etc/environment'
+    with salt.utils.fopen(tzfile, 'r') as fp_:
+        for line in fp_:
+            if 'TZ=' in line:
+                zonepart = line.rstrip('\n').split('=')[-1]
+                return zonepart.strip('\'"') or 'UTC'
+    raise CommandExecutionError('Unable to get timezone from ' + tzfile)
 
 
 def get_zone():
     '''
     Get current timezone (i.e. America/Denver)
 
+    .. versionchanged:: 2016.11.4
+
+    .. note::
+
+        On AIX operating systems, Posix values can also be returned
+        'CST6CDT,M3.2.0/2:00:00,M11.1.0/2:00:00'
+
     CLI Example:
 
     .. code-block:: bash
@@ -141,7 +185,7 @@ def get_zone():
         if __grains__['os'].lower() == 'centos':
             return _get_zone_etc_localtime()
         os_family = __grains__['os_family']
-        for family in ('RedHat', 'SUSE'):
+        for family in ('RedHat', 'Suse'):
             if family in os_family:
                 return _get_zone_sysconfig()
         for family in ('Debian', 'Gentoo'):
@@ -151,6 +195,8 @@ def get_zone():
             return _get_zone_etc_localtime()
         elif 'Solaris' in os_family:
             return _get_zone_solaris()
+        elif 'AIX' in os_family:
+            return _get_zone_aix()
     raise CommandExecutionError('Unable to get timezone')
 
 
@@ -177,7 +223,15 @@ def get_offset():
 
         salt '*' timezone.get_offset
     '''
-    return __salt__['cmd.run'](['date', '+%z'], python_shell=False)
+    if 'AIX' not in __grains__['os_family']:
+        return __salt__['cmd.run'](['date', '+%z'], python_shell=False)
+
+    salt_path = '/opt/salt/bin/date'
+
+    if not os.path.exists(salt_path):
+        return 'date in salt binaries does not exist: {0}'.format(salt_path)
+
+    return __salt__['cmd.run']([salt_path, '+%z'], python_shell=False)
 
 
 def set_zone(timezone):
@@ -194,6 +248,17 @@ def set_zone(timezone):
     .. code-block:: bash
 
         salt '*' timezone.set_zone 'America/Denver'
+
+    .. versionchanged:: 2016.11.4
+
+    .. note::
+
+        On AIX operating systems, Posix values are also allowed, see below
+
+    .. code-block:: bash
+
+        salt '*' timezone.set_zone 'CST6CDT,M3.2.0/2:00:00,M11.1.0/2:00:00'
+
     '''
     if salt.utils.which('timedatectl'):
         try:
@@ -201,11 +266,12 @@ def set_zone(timezone):
         except CommandExecutionError:
             pass
 
-    if 'Solaris' in __grains__['os_family']:
+    if 'Solaris' in __grains__['os_family'] or 'AIX' in __grains__['os_family']:
         zonepath = '/usr/share/lib/zoneinfo/{0}'.format(timezone)
     else:
         zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)
-    if not os.path.exists(zonepath):
+
+    if not os.path.exists(zonepath) and 'AIX' not in __grains__['os_family']:
         return 'Zone does not exist: {0}'.format(zonepath)
 
     if os.path.exists('/etc/localtime'):
@@ -214,22 +280,31 @@ def set_zone(timezone):
     if 'Solaris' in __grains__['os_family']:
         __salt__['file.sed'](
             '/etc/default/init', '^TZ=.*', 'TZ={0}'.format(timezone))
+    elif 'AIX' in __grains__['os_family']:
+        ## timezone could be Olson or Posix
+        curtzstring = get_zone()
+        cmd = ['chtz', timezone]
+        result = __salt__['cmd.retcode'](cmd, python_shell=False)
+        if result == 0:
+            return True
+
+        # restore orig timezone, since AIX chtz failure sets UTC
+        cmd = ['chtz', curtzstring]
+        __salt__['cmd.retcode'](cmd, python_shell=False)
+        return False
     else:
         os.symlink(zonepath, '/etc/localtime')
 
     if 'RedHat' in __grains__['os_family']:
         __salt__['file.sed'](
             '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
-    elif 'SUSE' in __grains__['os_family']:
+    elif 'Suse' in __grains__['os_family']:
         __salt__['file.sed'](
             '/etc/sysconfig/clock', '^TIMEZONE=.*', 'TIMEZONE="{0}"'.format(timezone))
-    elif 'Debian' in __grains__['os_family']:
+    elif 'Debian' in __grains__['os_family'] or 'Gentoo' in __grains__['os_family']:
         with salt.utils.fopen('/etc/timezone', 'w') as ofh:
             ofh.write(timezone.strip())
             ofh.write('\n')
-    elif 'Gentoo' in __grains__['os_family']:
-        with salt.utils.fopen('/etc/timezone', 'w') as ofh:
-            ofh.write(timezone)
 
     return True
 
@@ -247,43 +322,47 @@ def zone_compare(timezone):
 
         On Solaris-link operating systems only a string comparison is done.
 
+    .. versionchanged:: 2016.11.4
+
+    .. note::
+
+        On AIX operating systems only a string comparison is done.
+
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' timezone.zone_compare 'America/Denver'
     '''
-    if 'Solaris' in __grains__['os_family']:
+    if 'Solaris' in __grains__['os_family'] or 'AIX' in __grains__['os_family']:
         return timezone == get_zone()
 
-    curtzstring = get_zone()
-    if curtzstring != timezone:
-        return False
-
-    tzfile = '/etc/localtime'
-    zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)
-
-    if not os.path.exists(tzfile):
-        return 'Error: {0} does not exist.'.format(tzfile)
-
-    hash_type = __opts__.get('hash_type', 'md5')
+    if 'FreeBSD' in __grains__['os_family']:
+        if not os.path.isfile(_get_etc_localtime_path()):
+            return timezone == get_zone()
 
+    tzfile = _get_etc_localtime_path()
+    zonepath = _get_zone_file(timezone)
     try:
-        usrzone = salt.utils.get_hash(zonepath, hash_type)
-    except IOError as exc:
-        raise SaltInvocationError('Invalid timezone \'{0}\''.format(timezone))
+        return filecmp.cmp(tzfile, zonepath, shallow=False)
+    except OSError as exc:
+        problematic_file = exc.filename
+        if problematic_file == zonepath:
+            raise SaltInvocationError(
+                'Can\'t find a local timezone "{0}"'.format(timezone))
+        elif problematic_file == tzfile:
+            raise CommandExecutionError(
+                'Failed to read {0} to determine current timezone: {1}'
+                .format(tzfile, exc.strerror))
+        raise
 
-    try:
-        etczone = salt.utils.get_hash(tzfile, hash_type)
-    except IOError as exc:
-        raise CommandExecutionError(
-            'Problem reading timezone file {0}: {1}'
-            .format(tzfile, exc.strerror)
-        )
 
-    if usrzone == etczone:
-        return True
-    return False
+def _get_etc_localtime_path():
+    return '/etc/localtime'
+
+
+def _get_zone_file(timezone):
+    return '/usr/share/zoneinfo/{0}'.format(timezone)
 
 
 def get_hwclock():
@@ -314,10 +393,9 @@ def get_hwclock():
 
     else:
         os_family = __grains__['os_family']
-        for family in ('RedHat', 'SUSE'):
+        for family in ('RedHat', 'Suse'):
             if family in os_family:
-                cmd = ['tail', '-n', '1', '/etc/adjtime']
-                return __salt__['cmd.run'](cmd, python_shell=False)
+                return _get_adjtime_timezone()
 
         if 'Debian' in __grains__['os_family']:
             # Original way to look up hwclock on Debian-based systems
@@ -335,8 +413,7 @@ def get_hwclock():
             except IOError as exc:
                 pass
             # Since Wheezy
-            cmd = ['tail', '-n', '1', '/etc/adjtime']
-            return __salt__['cmd.run'](cmd, python_shell=False)
+            return _get_adjtime_timezone()
 
         if 'Gentoo' in __grains__['os_family']:
             if not os.path.exists('/etc/adjtime'):
@@ -360,8 +437,7 @@ def get_hwclock():
                         'Problem reading offset file {0}: {1}'
                         .format(offset_file, exc.strerror)
                     )
-            cmd = ['tail', '-n', '1', '/etc/adjtime']
-            return __salt__['cmd.run'](cmd, python_shell=False)
+            return _get_adjtime_timezone()
 
         if 'Solaris' in __grains__['os_family']:
             offset_file = '/etc/rtc_config'
@@ -380,6 +456,23 @@ def get_hwclock():
                     .format(offset_file, exc.strerror)
                 )
 
+        if 'AIX' in __grains__['os_family']:
+            offset_file = '/etc/environment'
+            try:
+                with salt.utils.fopen(offset_file, 'r') as fp_:
+                    for line in fp_:
+                        if line.startswith('TZ=UTC'):
+                            return 'UTC'
+                    return 'localtime'
+            except IOError as exc:
+                if exc.errno == errno.ENOENT:
+                    # offset file does not exist
+                    return 'UTC'
+                raise CommandExecutionError(
+                    'Problem reading offset file {0}: {1}'
+                    .format(offset_file, exc.strerror)
+                )
+
 
 def set_hwclock(clock):
     '''
@@ -391,53 +484,67 @@ def set_hwclock(clock):
 
         salt '*' timezone.set_hwclock UTC
     '''
-    timezone = get_zone()
 
-    if 'Solaris' in __grains__['os_family']:
-        if clock.lower() not in ('localtime', 'utc'):
-            raise SaltInvocationError(
-                'localtime and UTC are the only permitted values'
-            )
-        if 'sparc' in __grains__['cpuarch']:
-            raise SaltInvocationError(
-                'UTC is the only choice for SPARC architecture'
-            )
-        cmd = ['rtc', '-z', 'GMT' if clock.lower() == 'utc' else timezone]
-        return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
-
-    zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)
-
-    if not os.path.exists(zonepath):
-        raise CommandExecutionError(
-            'Zone \'{0}\' does not exist'.format(zonepath)
-        )
-
-    os.unlink('/etc/localtime')
-    os.symlink(zonepath, '/etc/localtime')
-
-    if 'Arch' in __grains__['os_family']:
-        cmd = ['timezonectl', 'set-local-rtc',
+    if salt.utils.which('timedatectl'):
+        cmd = ['timedatectl', 'set-local-rtc',
                'true' if clock == 'localtime' else 'false']
         return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
-    elif 'RedHat' in __grains__['os_family']:
-        __salt__['file.sed'](
-            '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
-    elif 'SUSE' in __grains__['os_family']:
-        __salt__['file.sed'](
-            '/etc/sysconfig/clock', '^TIMEZONE=.*', 'TIMEZONE="{0}"'.format(timezone))
-    elif 'Debian' in __grains__['os_family']:
-        if clock == 'UTC':
-            __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=yes')
-        elif clock == 'localtime':
-            __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=no')
-    elif 'Gentoo' in __grains__['os_family']:
-        if clock not in ('UTC', 'localtime'):
-            raise SaltInvocationError(
-                'Only \'UTC\' and \'localtime\' are allowed'
+    else:
+        os_family = __grains__['os_family']
+        if os_family in ('AIX', 'NILinuxRT'):
+            if clock.lower() != 'utc':
+                raise SaltInvocationError(
+                    'UTC is the only permitted value'
+                )
+            return True
+
+        timezone = get_zone()
+
+        if 'Solaris' in __grains__['os_family']:
+            if clock.lower() not in ('localtime', 'utc'):
+                raise SaltInvocationError(
+                    'localtime and UTC are the only permitted values'
+                )
+            if 'sparc' in __grains__['cpuarch']:
+                raise SaltInvocationError(
+                    'UTC is the only choice for SPARC architecture'
+                )
+            cmd = ['rtc', '-z', 'GMT' if clock.lower() == 'utc' else timezone]
+            return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
+
+        zonepath = '/usr/share/zoneinfo/{0}'.format(timezone)
+
+        if not os.path.exists(zonepath):
+            raise CommandExecutionError(
+                'Zone \'{0}\' does not exist'.format(zonepath)
             )
-        if clock == 'localtime':
-            clock = 'local'
-        __salt__['file.sed'](
-            '/etc/conf.d/hwclock', '^clock=.*', 'clock="{0}"'.format(clock))
+
+        os.unlink('/etc/localtime')
+        os.symlink(zonepath, '/etc/localtime')
+
+        if 'Arch' in __grains__['os_family']:
+            cmd = ['timezonectl', 'set-local-rtc',
+                   'true' if clock == 'localtime' else 'false']
+            return __salt__['cmd.retcode'](cmd, python_shell=False) == 0
+        elif 'RedHat' in __grains__['os_family']:
+            __salt__['file.sed'](
+                '/etc/sysconfig/clock', '^ZONE=.*', 'ZONE="{0}"'.format(timezone))
+        elif 'Suse' in __grains__['os_family']:
+            __salt__['file.sed'](
+                '/etc/sysconfig/clock', '^TIMEZONE=.*', 'TIMEZONE="{0}"'.format(timezone))
+        elif 'Debian' in __grains__['os_family']:
+            if clock == 'UTC':
+                __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=yes')
+            elif clock == 'localtime':
+                __salt__['file.sed']('/etc/default/rcS', '^UTC=.*', 'UTC=no')
+        elif 'Gentoo' in __grains__['os_family']:
+            if clock not in ('UTC', 'localtime'):
+                raise SaltInvocationError(
+                    'Only \'UTC\' and \'localtime\' are allowed'
+                )
+            if clock == 'localtime':
+                clock = 'local'
+            __salt__['file.sed'](
+                '/etc/conf.d/hwclock', '^clock=.*', 'clock="{0}"'.format(clock))
 
     return True
