@@ -5,10 +5,13 @@ Support for Tomcat
 This module uses the manager webapp to manage Apache tomcat webapps.
 If the manager webapp is not configured some of the functions won't work.
 
-.. note::
-
-    The config format was changed in 2014.7.0, but backwards compatibility for
-    the old-style config will be in the 2014.7.1 release.
+:configuration:
+    - Java bin path should be in default path
+    - If ipv6 is enabled make sure you permit manager access to ipv6 interface
+      "0:0:0:0:0:0:0:1"
+    - If you are using tomcat.tar.gz it has to be installed or symlinked under
+      ``/opt``, preferably using name tomcat
+    - "tomcat.signal start/stop" works but it does not use the startup scripts
 
 The following grains/pillar should be set:
 
@@ -60,27 +63,31 @@ Also configure a user in the conf/tomcat-users.xml file:
 from __future__ import absolute_import
 
 # Import python libs
+import os
+import re
 import glob
 import hashlib
 import tempfile
-import os
-import re
+import logging
 
 # Import 3rd-party libs
 # pylint: disable=no-name-in-module,import-error
+from salt.ext.six import string_types as _string_types
 from salt.ext.six.moves.urllib.parse import urlencode as _urlencode
 from salt.ext.six.moves.urllib.request import (
-        urlopen as _urlopen,
-        HTTPBasicAuthHandler as _HTTPBasicAuthHandler,
-        HTTPDigestAuthHandler as _HTTPDigestAuthHandler,
-        build_opener as _build_opener,
-        install_opener as _install_opener
+    urlopen as _urlopen,
+    HTTPBasicAuthHandler as _HTTPBasicAuthHandler,
+    HTTPDigestAuthHandler as _HTTPDigestAuthHandler,
+    build_opener as _build_opener,
+    install_opener as _install_opener
 )
 # pylint: enable=no-name-in-module,import-error
 
 # Import Salt libs
 import salt.utils
 
+log = logging.getLogger(__name__)
+
 __func_alias__ = {
     'reload_': 'reload'
 }
@@ -105,7 +112,8 @@ def __virtual__():
     '''
     if __catalina_home() or _auth('dummy'):
         return 'tomcat'
-    return False
+    return (False,
+            'Tomcat execution module not loaded: neither Tomcat installed locally nor tomcat-manager credentials set in grains/pillar/config.')
 
 
 def __catalina_home():
@@ -162,19 +170,31 @@ def _auth(uri):
 
     basic = _HTTPBasicAuthHandler()
     basic.add_password(realm='Tomcat Manager Application', uri=uri,
-            user=user, passwd=password)
+                       user=user, passwd=password)
     digest = _HTTPDigestAuthHandler()
     digest.add_password(realm='Tomcat Manager Application', uri=uri,
-            user=user, passwd=password)
+                        user=user, passwd=password)
     return _build_opener(basic, digest)
 
 
-def _extract_version(war):
+def extract_war_version(war):
     '''
-    extract the version from the war name
+    Extract the version from the war file name. There does not seem to be a
+    standard for encoding the version into the `war file name`_
+
+    .. _`war file name`: https://tomcat.apache.org/tomcat-6.0-doc/deployer-howto.html
+
+    Examples:
+
+    .. code-block:: bash
+
+        /path/salt-2015.8.6.war -> 2015.8.6
+        /path/V6R2013xD5.war -> None
     '''
-    version = re.findall("-([\\d.-]+)$", os.path.basename(war).replace('.war', ''))
-    return version[0] if len(version) == 1 else None
+    basename = os.path.basename(war)
+    war_package = os.path.splitext(basename)[0]  # remove '.war'
+    version = re.findall("-([\\d.-]+)$", war_package)  # try semver
+    return version[0] if version and len(version) == 1 else None  # default to none
 
 
 def _wget(cmd, opts=None, url='http://localhost:8080/manager', timeout=180):
@@ -184,15 +204,18 @@ def _wget(cmd, opts=None, url='http://localhost:8080/manager', timeout=180):
 
     cmd
         the command to execute
+
     url
-        the URL of the server manager webapp
-        example: http://localhost:8080/manager
+        The URL of the server manager webapp (example:
+        http://localhost:8080/manager)
+
     opts
         a dict of arguments
+
     timeout
         timeout for HTTP request
 
-    return value is a dict in the from of::
+    Return value is a dict in the from of::
 
         {
             res: [True|False]
@@ -274,7 +297,7 @@ def leaks(url='http://localhost:8080/manager', timeout=180):
     '''
 
     return _wget('findleaks', {'statusLine': 'true'},
-        url, timeout=timeout)['msg']
+                 url, timeout=timeout)['msg']
 
 
 def status(url='http://localhost:8080/manager', timeout=180):
@@ -507,8 +530,8 @@ def deploy_war(war,
                url='http://localhost:8080/manager',
                saltenv='base',
                timeout=180,
-               env=None,
-               temp_war_location=None):
+               temp_war_location=None,
+               version=True):
     '''
     Deploy a WAR file
 
@@ -529,6 +552,17 @@ def deploy_war(war,
     temp_war_location : None
         use another location to temporarily copy to war file
         by default the system's temp directory is used
+    version : ''
+        Specify the war version.  If this argument is provided, it overrides
+        the version encoded in the war file name, if one is present.
+
+        Examples:
+
+        .. code-block:: bash
+
+            salt '*' tomcat.deploy_war salt://salt-2015.8.6.war version=2015.08.r6
+
+        .. versionadded:: 2015.8.6
 
     CLI Examples:
 
@@ -548,15 +582,6 @@ def deploy_war(war,
         salt '*' tomcat.deploy_war /tmp/application.war /api no
         salt '*' tomcat.deploy_war /tmp/application.war /api yes http://localhost:8080/manager
     '''
-    if env is not None:
-        salt.utils.warn_until(
-            'Boron',
-            'Passing a salt environment should be done using \'saltenv\' '
-            'not \'env\'. This functionality will be removed in Salt Boron.'
-        )
-        # Backwards compatibility
-        saltenv = env
-
     # Decide the location to copy the war for the deployment
     tfile = 'salt.{0}'.format(os.path.basename(war))
     if temp_war_location is not None:
@@ -580,14 +605,21 @@ def deploy_war(war,
     else:
         tfile = war
 
-    version_string = _extract_version(war)
-
     # Prepare options
     opts = {
         'war': 'file:{0}'.format(tfile),
         'path': context,
-        'version': version_string,
     }
+
+    # If parallel versions are desired or not disabled
+    if version:
+        # Set it to defined version or attempt extract
+        version = extract_war_version(war) if version is True else version
+
+        if isinstance(version, _string_types):
+            # Only pass version to Tomcat if not undefined
+            opts['version'] = version
+
     if force == 'yes':
         opts['update'] = 'true'
 
@@ -604,7 +636,7 @@ def deploy_war(war,
 
 def passwd(passwd,
            user='',
-           alg='md5',
+           alg='sha1',
            realm=None):
     '''
     This function replaces the $CATALINA_HOME/bin/digest.sh script
@@ -619,23 +651,15 @@ def passwd(passwd,
         salt '*' tomcat.passwd secret tomcat sha1
         salt '*' tomcat.passwd secret tomcat sha1 'Protected Realm'
     '''
-    if alg == 'md5':
-        m = hashlib.md5()
-    elif alg == 'sha1':
-        m = hashlib.sha1()
-    else:
-        return False
+    # Shouldn't it be SHA265 instead of SHA1?
+    digest = hasattr(hashlib, alg) and getattr(hashlib, alg) or None
+    if digest:
+        if realm:
+            digest.update('{0}:{1}:{2}'.format(user, realm, passwd, ))
+        else:
+            digest.update(passwd)
 
-    if realm:
-        m.update('{0}:{1}:{2}'.format(
-            user,
-            realm,
-            passwd,
-            ))
-    else:
-        m.update(passwd)
-
-    return m.hexdigest()
+    return digest and digest.hexdigest() or False
 
 
 # Non-Manager functions
@@ -728,6 +752,6 @@ if __name__ == '__main__':
     new_format_creds = _get_credentials()
 
     if old_format_creds == new_format_creds:
-        print 'Config backwards compatible'
+        log.info('Config backwards compatible')
     else:
-        print 'Config not backwards compatible'
+        log.ifno('Config not backwards compatible')
