@@ -4,26 +4,27 @@ Create virtualenv environments.
 
 .. versionadded:: 0.17.0
 '''
-from __future__ import absolute_import
 
 # Import python libs
+from __future__ import absolute_import
 import glob
 import shutil
 import logging
 import os
+import sys
 
 # Import salt libs
 import salt.utils
+import salt.utils.files
+import salt.utils.verify
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 from salt.ext.six import string_types
 
-KNOWN_BINARY_NAMES = frozenset(
-    ['virtualenv',
-     'virtualenv2',
-     'virtualenv-2.6',
-     'virtualenv-2.7'
-     ]
-)
+KNOWN_BINARY_NAMES = frozenset([
+    'virtualenv-{0}.{1}'.format(*sys.version_info[:2]),
+    'virtualenv{0}'.format(sys.version_info[0]),
+    'virtualenv',
+])
 
 log = logging.getLogger(__name__)
 
@@ -55,7 +56,8 @@ def create(path,
            upgrade=None,
            user=None,
            use_vt=False,
-           saltenv='base'):
+           saltenv='base',
+           **kwargs):
     '''
     Create a virtualenv
 
@@ -101,6 +103,11 @@ def create(path,
     user : None
         Set ownership for the virtualenv
 
+        .. note::
+            On Windows you must also pass a ``password`` parameter. Additionally,
+            the user must have permissions to the location where the virtual
+            environment is being created
+
     runas : None
         Set ownership for the virtualenv
 
@@ -160,7 +167,7 @@ def create(path,
             # Unable to import?? Let's parse the version from the console
             version_cmd = [venv_bin, '--version']
             ret = __salt__['cmd.run_all'](
-                    version_cmd, runas=user, python_shell=False
+                    version_cmd, runas=user, python_shell=False, **kwargs
                 )
             if ret['retcode'] > 0 or not ret['stdout'].strip():
                 raise CommandExecutionError(
@@ -250,7 +257,7 @@ def create(path,
     cmd.append(path)
 
     # Let's create the virtualenv
-    ret = __salt__['cmd.run_all'](cmd, runas=user, python_shell=False)
+    ret = __salt__['cmd.run_all'](cmd, runas=user, python_shell=False, **kwargs)
     if ret['retcode'] != 0:
         # Something went wrong. Let's bail out now!
         return ret
@@ -315,7 +322,7 @@ def get_site_packages(venv):
     ret = __salt__['cmd.exec_code_all'](
         bin_path,
         'from distutils import sysconfig; '
-            'print sysconfig.get_python_lib()'
+            'print(sysconfig.get_python_lib())'
     )
 
     if ret['retcode'] != 0:
@@ -360,13 +367,13 @@ def get_distribution_path(venv, distribution):
 
 
 def get_resource_path(venv,
-                      package_or_requirement=None,
-                      resource_name=None,
                       package=None,
                       resource=None):
     '''
     Return the path to a package resource installed inside a virtualenv
 
+    .. versionadded:: 2015.5.0
+
     venv
         Path to the virtualenv
 
@@ -375,63 +382,17 @@ def get_resource_path(venv,
 
         .. versionadded:: 2016.3.0
 
-    package_or_requirement
-        Name of the package in which the resource resides
-
-        .. deprecated:: 2016.3.0
-            Use ``package`` instead.
-
     resource
         Name of the resource of which the path is to be returned
 
         .. versionadded:: 2016.3.0
 
-    resource_name
-        Name of the resource of which the path is to be returned
-
-        .. deprecated:: 2016.3.0
-
-
-    .. versionadded:: 2015.5.0
-
-    venv
-        Path to the virtualenv.
-    package_or_requirement
-        Name of the package where the resource resides in.
-    resource_name
-        Name of the resource of which the path is to be returned.
-
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' virtualenv.get_resource_path /path/to/my/venv my_package my/resource.xml
     '''
-    if package_or_requirement is not None:
-        salt.utils.warn_until(
-            'Nitrogen',
-            'The \'package_or_requirement\' argument to '
-            'virtualenv.get_resource_path is deprecated. Please use '
-            '\'package\' instead.'
-        )
-        if package is not None:
-            raise CommandExecutionError(
-                'Only one of \'package\' and \'package_or_requirement\' is '
-                'permitted.'
-            )
-        package = package_or_requirement
-    if resource_name is not None:
-        salt.utils.warn_until(
-            'Nitrogen',
-            'The \'resource_name\' argument to virtualenv.get_resource_path '
-            'is deprecated. Please use \'resource\' instead.'
-        )
-        if resource is not None:
-            raise CommandExecutionError(
-                'Only one of \'resource\' and \'resource_name\' is permitted.'
-            )
-        resource = resource_name
-
     _verify_safe_py_code(package, resource)
     bin_path = _verify_virtualenv(venv)
 
@@ -451,13 +412,13 @@ def get_resource_path(venv,
 
 
 def get_resource_content(venv,
-                         package_or_requirement=None,
-                         resource_name=None,
                          package=None,
                          resource=None):
     '''
     Return the content of a package resource installed inside a virtualenv
 
+    .. versionadded:: 2015.5.0
+
     venv
         Path to the virtualenv
 
@@ -466,65 +427,17 @@ def get_resource_content(venv,
 
         .. versionadded:: 2016.3.0
 
-    package_or_requirement
-        Name of the package in which the resource resides
-
-        .. deprecated:: 2016.3.0
-            Use ``package`` instead.
-
     resource
         Name of the resource of which the content is to be returned
 
         .. versionadded:: 2016.3.0
 
-    resource_name
-        Name of the resource of which the content is to be returned
-
-        .. deprecated:: 2016.3.0
-
-
-    .. versionadded:: 2015.5.0
-
-    venv
-        Path to the virtualenv.
-    package_or_requirement
-        Name of the package where the resource resides in.
-    resource_name
-        Name of the resource of which the content is to be returned.
-
-
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' virtualenv.get_resource_content /path/to/my/venv my_package my/resource.xml
     '''
-    if package_or_requirement is not None:
-        salt.utils.warn_until(
-            'Nitrogen',
-            'The \'package_or_requirement\' argument to '
-            'virtualenv.get_resource_content is deprecated. Please use '
-            '\'package\' instead.'
-        )
-        if package is not None:
-            raise CommandExecutionError(
-                'Only one of \'package\' and \'package_or_requirement\' is '
-                'permitted.'
-            )
-        package = package_or_requirement
-    if resource_name is not None:
-        salt.utils.warn_until(
-            'Nitrogen',
-            'The \'resource_name\' argument to '
-            'virtualenv.get_resource_content is deprecated. Please use '
-            '\'resource\' instead.'
-        )
-        if resource is not None:
-            raise CommandExecutionError(
-                'Only one of \'resource\' and \'resource_name\' is permitted.'
-            )
-        resource = resource_name
-
     _verify_safe_py_code(package, resource)
     bin_path = _verify_virtualenv(venv)
 
@@ -545,7 +458,7 @@ def get_resource_content(venv,
 
 def _install_script(source, cwd, python, user, saltenv='base', use_vt=False):
     if not salt.utils.is_windows():
-        tmppath = salt.utils.mkstemp(dir=cwd)
+        tmppath = salt.utils.files.mkstemp(dir=cwd)
     else:
         tmppath = __salt__['cp.cache_file'](source, saltenv)
 
