@@ -648,57 +648,56 @@ def set_lcm_config(config_mode=None,
     For detailed descriptions of the parameters see:
     https://msdn.microsoft.com/en-us/PowerShell/DSC/metaConfig
 
-    Args:
+    config_mode (str): How the LCM applies the configuration. Valid values
+        are:
 
-        config_mode (str): How the LCM applies the configuration. Valid values
-            are:
+        - ApplyOnly
+        - ApplyAndMonitor
+        - ApplyAndAutoCorrect
 
-            - ApplyOnly
-            - ApplyAndMonitor
-            - ApplyAndAutoCorrect
+    config_mode_freq (int): How often, in minutes, the current configuration
+        is checked and applied. Ignored if config_mode is set to ApplyOnly.
+        Default is 15.
 
-        config_mode_freq (int): How often, in minutes, the current configuration
-            is checked and applied. Ignored if config_mode is set to ApplyOnly.
-            Default is 15.
+    refresh_mode (str): How the LCM gets configurations. Valid values are:
 
-        refresh_mode (str): How the LCM gets configurations. Valid values are:
+        - Disabled
+        - Push
+        - Pull
 
-            - Disabled
-            - Push
-            - Pull
+    refresh_freq (int): How often, in minutes, the LCM checks for updated
+        configurations. (pull mode only) Default is 30.
 
-        refresh_freq (int): How often, in minutes, the LCM checks for updated
-            configurations. (pull mode only) Default is 30.
+    reboot_if_needed (bool): Reboot the machine if needed after a
+        configuration is applied. Default is False.
 
-        .. note:: Either `config_mode_freq` or `refresh_freq` needs to be a
-            multiple of the other. See documentation on MSDN for more details.
+    action_after_reboot (str): Action to take after reboot. Valid values
+        are:
 
-        reboot_if_needed (bool): Reboot the machine if needed after a
-            configuration is applied. Default is False.
+        - ContinueConfiguration
+        - StopConfiguration
 
-        action_after_reboot (str): Action to take after reboot. Valid values
-            are:
+    certificate_id (guid): A GUID that specifies a certificate used to
+        access the configuration: (pull mode)
 
-            - ContinueConfiguration
-            - StopConfiguration
+    configuration_id (guid): A GUID that identifies the config file to get
+        from a pull server. (pull mode)
 
-        certificate_id (guid): A GUID that specifies a certificate used to
-            access the configuration: (pull mode)
+    allow_module_overwrite (bool): New configs are allowed to overwrite old
+        ones on the target node.
 
-        configuration_id (guid): A GUID that identifies the config file to get
-            from a pull server. (pull mode)
+    debug_mode (str): Sets the debug level. Valid values are:
 
-        allow_module_overwrite (bool): New configs are allowed to overwrite old
-            ones on the target node.
+        - None
+        - ForceModuleImport
+        - All
 
-        debug_mode (str): Sets the debug level. Valid values are:
+    status_retention_days (int): Number of days to keep status of the
+        current config.
 
-            - None
-            - ForceModuleImport
-            - All
-
-        status_retention_days (int): Number of days to keep status of the
-            current config.
+    .. note::
+        Either ``config_mode_freq`` or ``refresh_freq`` needs to be a
+        multiple of the other. See documentation on MSDN for more details.
 
     Returns:
         bool: True if successful, otherwise False
