@@ -56,16 +56,7 @@ def __virtual__():
 
 
 def _get_local_repo_dir(saltenv='base'):
-    if 'win_repo_source_dir' in __opts__:
-        salt.utils.warn_until(
-            'Nitrogen',
-            'The \'win_repo_source_dir\' config option is deprecated, please '
-            'use \'winrepo_source_dir\' instead.'
-        )
-        winrepo_source_dir = __opts__['win_repo_source_dir']
-    else:
-        winrepo_source_dir = __opts__['winrepo_source_dir']
-
+    winrepo_source_dir = __opts__['winrepo_source_dir']
     dirs = []
     dirs.append(salt.syspaths.CACHE_DIR)
     dirs.extend(['minion', 'files'])
@@ -134,6 +125,7 @@ def update_git_repos(clean=False):
 def show_sls(name, saltenv='base'):
     r'''
     .. versionadded:: 2015.8.0
+
     Display the rendered software definition from a specific sls file in the
     local winrepo cache. This will parse all Jinja. Run pkg.refresh_db to pull
     the latest software definitions from the master.
