@@ -4,8 +4,8 @@ Module for managing Windows Updates using the Windows Update Agent.
 
 List updates on the system using the following functions:
 
-- :ref:`available`
-- :ref:`list`
+- :py:func:`win_wua.available <salt.modules.win_wua.available>`
+- :py:func:`win_wua.list <salt.modules.win_wua.list_>`
 
 This is an easy way to find additional information about updates available to
 to the system, such as the GUID, KB number, or description.
@@ -13,27 +13,33 @@ to the system, such as the GUID, KB number, or description.
 Once you have the GUID or a KB number for the update you can get information
 about the update, download, install, or uninstall it using these functions:
 
-- :ref:`get`
-- :ref:`download`
-- :ref:`install`
-- :ref:`uninstall`
+- :py:func:`win_wua.get <salt.modules.win_wua.get>`
+- :py:func:`win_wua.download <salt.modules.win_wua.download>`
+- :py:func:`win_wua.install <salt.modules.win_wua.install>`
+- :py:func:`win_wua.uninstall <salt.modules.win_wua.uninstall>`
 
 The get function expects a name in the form of a GUID, KB, or Title and should
 return information about a single update. The other functions accept either a
 single item or a list of items for downloading/installing/uninstalling a
 specific list of items.
 
-The :ref:`list` and :ref:`get` functions are utility functions. In addition to
-returning information about updates they can also download and install updates
-by setting ``download=True`` or ``install=True``. So, with :ref:`list` for
-example, you could run the function with the filters you want to see what is
-available. Then just add ``install=True`` to install everything on that list.
+The :py:func:`win_wua.list <salt.modules.win_wua.list_>` and
+:py:func:`win_wua.get <salt.modules.win_wua.get>` functions are utility
+functions. In addition to returning information about updates they can also
+download and install updates by setting ``download=True`` or ``install=True``.
+So, with py:func:`win_wua.list <salt.modules.win_wua.list_>` for example, you
+could run the function with the filters you want to see what is available. Then
+just add ``install=True`` to install everything on that list.
 
 If you want to download, install, or uninstall specific updates, use
-:ref:`download`, :ref:`install`, or :ref:`uninstall`. To update your system
-with the latest updates use :ref:`list` and set ``install=True``
+:py:func:`win_wua.download <salt.modules.win_wua.download>`,
+:py:func:`win_wua.install <salt.modules.win_wua.install>`, or
+:py:func:`win_wua.uninstall <salt.modules.win_wua.uninstall>`. To update your
+system with the latest updates use :py:func:`win_wua.list
+<salt.modules.win_wua.list_>` and set ``install=True``
 
-You can also adjust the Windows Update settings using the :ref:`set_wu_settings`
+You can also adjust the Windows Update settings using the
+:py:func:`win_wua.set_wu_settings <salt.modules.win_wua.set_wu_settings>`
 function. This function is only supported on the following operating systems:
 
 - Windows Vista / Server 2008
@@ -47,8 +53,7 @@ Group Policy using the ``lgpo`` module.
 
 .. versionadded:: 2015.8.0
 
-:depends:
-        - salt.utils.win_update
+:depends: salt.utils.win_update
 '''
 
 # Import Python libs
@@ -72,6 +77,10 @@ except ImportError:
 
 log = logging.getLogger(__name__)
 
+__func_alias__ = {
+    'list_': 'list',
+}
+
 
 def __virtual__():
     '''
@@ -538,14 +547,14 @@ def list_updates(software=True,
                 severities, download, install)
 
 
-def list(software=True,
-         drivers=False,
-         summary=False,
-         skip_installed=True,
-         categories=None,
-         severities=None,
-         download=False,
-         install=False):
+def list_(software=True,
+          drivers=False,
+          summary=False,
+          skip_installed=True,
+          categories=None,
+          severities=None,
+          download=False,
+          install=False):
     '''
     .. versionadded:: 2017.7.0
 
