@@ -75,12 +75,12 @@ A REST API for Salt
     log_access_file
         Path to a file to write HTTP access logs.
 
-        .. versionaddedd:: 2016.11.0
+        .. versionadded:: 2016.11.0
 
     log_error_file
         Path to a file to write HTTP error logs.
 
-        .. versionaddedd:: 2016.11.0
+        .. versionadded:: 2016.11.0
 
     ssl_crt
         The path to a SSL certificate. (See below)
@@ -272,8 +272,8 @@ command:
 3.  Fill out the remaining parameters needed for the chosen client.
 
 The ``client`` field is a reference to the main Python classes used in Salt's
-Python API. Read the full :ref:`client interfaces <netapi-clients>`
-documentation, but in short:
+Python API. Read the full :ref:`Client APIs <client-apis>` documentation, but
+in short:
 
 * "local" uses :py:class:`LocalClient <salt.client.LocalClient>` which sends
   commands to Minions. Equivalent to the ``salt`` CLI command.
@@ -289,7 +289,7 @@ documentation, but in short:
 
 Most clients have variants like synchronous or asynchronous execution as well as
 others like batch execution. See the :ref:`full list of client interfaces
-<netapi-clients>`.
+<client-interfaces>`.
 
 Each client requires different arguments and sometimes has different syntax.
 For example, ``LocalClient`` requires the ``tgt`` argument because it forwards
@@ -1078,6 +1078,13 @@ class LowDataAdapter(object):
                 if cherrypy.session.get('groups'):
                     chunk['__current_eauth_groups'] = cherrypy.session.get('groups')
 
+            if 'token' in chunk:
+                # Make sure that auth token is hex
+                try:
+                    int(chunk['token'], 16)
+                except (TypeError, ValueError):
+                    raise cherrypy.HTTPError(401, 'Invalid token')
+
             if client:
                 chunk['client'] = client
 
@@ -1114,7 +1121,7 @@ class LowDataAdapter(object):
 
             curl -i localhost:8000
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET / HTTP/1.1
             Host: localhost:8000
@@ -1122,7 +1129,7 @@ class LowDataAdapter(object):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Type: application/json
@@ -1166,7 +1173,7 @@ class LowDataAdapter(object):
                 -H "Content-type: application/json" \\
                 -d '[{"client": "local", "tgt": "*", "fun": "test.ping"}]'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST / HTTP/1.1
             Host: localhost:8000
@@ -1178,7 +1185,7 @@ class LowDataAdapter(object):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 200
@@ -1226,7 +1233,7 @@ class Minions(LowDataAdapter):
 
             curl -i localhost:8000/minions/ms-3
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /minions/ms-3 HTTP/1.1
             Host: localhost:8000
@@ -1234,7 +1241,7 @@ class Minions(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 129005
@@ -1270,8 +1277,8 @@ class Minions(LowDataAdapter):
             :status 401: |401|
             :status 406: |406|
 
-            :term:`lowstate` data describing Salt commands must be sent in the
-            request body. The ``client`` option will be set to
+            Lowstate data describing Salt commands must be sent in the request
+            body. The ``client`` option will be set to
             :py:meth:`~salt.client.LocalClient.local_async`.
 
         **Example request:**
@@ -1283,7 +1290,7 @@ class Minions(LowDataAdapter):
                 -H "Accept: application/x-yaml" \\
                 -d '[{"tgt": "*", "fun": "status.diskusage"}]'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST /minions HTTP/1.1
             Host: localhost:8000
@@ -1294,7 +1301,7 @@ class Minions(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 202 Accepted
             Content-Length: 86
@@ -1347,7 +1354,7 @@ class Jobs(LowDataAdapter):
 
             curl -i localhost:8000/jobs
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /jobs HTTP/1.1
             Host: localhost:8000
@@ -1355,7 +1362,7 @@ class Jobs(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 165
@@ -1376,7 +1383,7 @@ class Jobs(LowDataAdapter):
 
             curl -i localhost:8000/jobs/20121130104633606931
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /jobs/20121130104633606931 HTTP/1.1
             Host: localhost:8000
@@ -1384,7 +1391,7 @@ class Jobs(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 73
@@ -1466,7 +1473,7 @@ class Keys(LowDataAdapter):
 
             curl -i localhost:8000/keys
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /keys HTTP/1.1
             Host: localhost:8000
@@ -1474,7 +1481,7 @@ class Keys(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 165
@@ -1495,7 +1502,7 @@ class Keys(LowDataAdapter):
 
             curl -i localhost:8000/keys/jerry
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /keys/jerry HTTP/1.1
             Host: localhost:8000
@@ -1503,7 +1510,7 @@ class Keys(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 73
@@ -1580,14 +1587,14 @@ class Keys(LowDataAdapter):
                     -d eauth=pam \
                     -o jerry-salt-keys.tar
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST /keys HTTP/1.1
             Host: localhost:8000
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 10240
@@ -1662,7 +1669,7 @@ class Login(LowDataAdapter):
 
             curl -i localhost:8000/login
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /login HTTP/1.1
             Host: localhost:8000
@@ -1670,7 +1677,7 @@ class Login(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Type: text/html
@@ -1714,7 +1721,7 @@ class Login(LowDataAdapter):
                     "eauth": "auto"
                 }'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST / HTTP/1.1
             Host: localhost:8000
@@ -1727,7 +1734,7 @@ class Login(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Type: application/json
@@ -1866,7 +1873,7 @@ class Token(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Type: application/json
@@ -1927,8 +1934,8 @@ class Run(LowDataAdapter):
 
         .. http:post:: /run
 
-            An array of :term:`lowstate` data describing Salt commands must be
-            sent in the request body.
+            An array of lowstate data describing Salt commands must be sent in
+            the request body.
 
             :status 200: |200|
             :status 400: |400|
@@ -1965,7 +1972,7 @@ class Run(LowDataAdapter):
                     "token": "<salt eauth token here>"
                 }]'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST /run HTTP/1.1
             Host: localhost:8000
@@ -1977,7 +1984,7 @@ class Run(LowDataAdapter):
 
         **Example response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 73
@@ -2011,7 +2018,7 @@ class Run(LowDataAdapter):
                 -d tgt='*' \\
                 -d fun='test.ping'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST /run HTTP/1.1
             Host: localhost:8000
@@ -2023,7 +2030,7 @@ class Run(LowDataAdapter):
 
         **Example SSH response:**
 
-        .. code-block:: http
+        .. code-block:: text
 
                 return:
                 - silver:
@@ -2078,7 +2085,11 @@ class Events(object):
 
         :return bool: True if valid, False if not valid.
         '''
-        if auth_token is None:
+        # Make sure that auth token is hex. If it's None, or something other
+        # than hex, this will raise a ValueError.
+        try:
+            int(auth_token, 16)
+        except (TypeError, ValueError):
             return False
 
         # First check if the given token is in our session table; if so it's a
@@ -2123,7 +2134,7 @@ class Events(object):
 
             curl -NsS localhost:8000/events
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /events HTTP/1.1
             Host: localhost:8000
@@ -2135,7 +2146,7 @@ class Events(object):
         clients to only watch for certain tags without having to deserialze the
         JSON object each time.
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Connection: keep-alive
@@ -2321,7 +2332,7 @@ class WebsocketEndpoint(object):
                 -H 'Sec-WebSocket-Key: '"$(echo -n $RANDOM | base64)" \\
                 localhost:8000/ws
 
-        .. code-block:: http
+        .. code-block:: text
 
             GET /ws HTTP/1.1
             Connection: Upgrade
@@ -2334,7 +2345,7 @@ class WebsocketEndpoint(object):
 
         **Example response**:
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 101 Switching Protocols
             Upgrade: websocket
@@ -2491,7 +2502,7 @@ class Webhook(object):
                         -d branch="${TRAVIS_BRANCH}" \
                         -d commit="${TRAVIS_COMMIT}"
 
-    .. seealso:: :ref:`events`, :ref:`reactor`
+    .. seealso:: :ref:`events`, :ref:`reactor <reactor>`
     '''
     exposed = True
     tag_base = ['salt', 'netapi', 'hook']
@@ -2535,7 +2546,7 @@ class Webhook(object):
                 -H 'Content-type: application/json' \\
                 -d '{"foo": "Foo!", "bar": "Bar!"}'
 
-        .. code-block:: http
+        .. code-block:: text
 
             POST /hook HTTP/1.1
             Host: localhost:8000
@@ -2546,7 +2557,7 @@ class Webhook(object):
 
         **Example response**:
 
-        .. code-block:: http
+        .. code-block:: text
 
             HTTP/1.1 200 OK
             Content-Length: 14
