@@ -35,7 +35,7 @@ Return data to a cassandra server
 
     Use the following cassandra database schema:
 
-    .. code-block:: sql
+    .. code-block:: text
 
         CREATE KEYSPACE IF NOT EXISTS salt
             WITH replication = {'class': 'SimpleStrategy', 'replication_factor' : 1};
@@ -83,11 +83,39 @@ Return data to a cassandra server
 
 Required python modules: cassandra-driver
 
-To use the cassandra returner, append '--return cassandra' to the salt command. ex:
+To use the cassandra returner, append '--return cassandra_cql' to the salt command. ex:
 
 .. code-block:: bash
 
-    salt '*' test.ping --return cassandra
+    salt '*' test.ping --return_cql cassandra
+
+Note: if your Cassandra instance has not been tuned much you may benefit from
+altering some timeouts in `cassandra.yaml` like so:
+
+.. code-block:: yaml
+
+    # How long the coordinator should wait for read operations to complete
+    read_request_timeout_in_ms: 5000
+    # How long the coordinator should wait for seq or index scans to complete
+    range_request_timeout_in_ms: 20000
+    # How long the coordinator should wait for writes to complete
+    write_request_timeout_in_ms: 20000
+    # How long the coordinator should wait for counter writes to complete
+    counter_write_request_timeout_in_ms: 10000
+    # How long a coordinator should continue to retry a CAS operation
+    # that contends with other proposals for the same row
+    cas_contention_timeout_in_ms: 5000
+    # How long the coordinator should wait for truncates to complete
+    # (This can be much longer, because unless auto_snapshot is disabled
+    # we need to flush first so we can snapshot before removing the data.)
+    truncate_request_timeout_in_ms: 60000
+    # The default timeout for other, miscellaneous operations
+    request_timeout_in_ms: 20000
+
+As always, your mileage may vary and your Cassandra cluster may have different
+needs.  SaltStack has seen situations where these timeouts can resolve
+some stacktraces that appear to come from the Datastax Python driver.
+
 '''
 from __future__ import absolute_import
 # Let's not allow PyLint complain about string substitution
@@ -148,7 +176,8 @@ __virtualname__ = 'cassandra_cql'
 
 def __virtual__():
     if not HAS_CASSANDRA_DRIVER:
-        return False
+        return False, 'Could not import cassandra_cql returner; ' \
+                      'cassandra-driver is not installed.'
 
     return True
 
@@ -159,21 +188,24 @@ def returner(ret):
     '''
     query = '''INSERT INTO salt.salt_returns (
                  jid, minion_id, fun, alter_time, full_ret, return, success
-               ) VALUES (
-                 '{0}', '{1}', '{2}', '{3}', '{4}', '{5}', {6}
-               );'''.format(
-                 ret['jid'],
-                 ret['id'],
-                 ret['fun'],
-                 int(time.time() * 1000),
-                 json.dumps(ret).replace("'", "''"),
-                 json.dumps(ret['return']).replace("'", "''"),
-                 ret.get('success', False)
-               )
+               ) VALUES (?, ?, ?, ?, ?, ?, ?)'''
+
+    statement_arguments = []
+
+    statement_arguments.append('{0}'.format(ret['jid']))
+    statement_arguments.append('{0}'.format(ret['id']))
+    statement_arguments.append('{0}'.format(ret['fun']))
+    statement_arguments.append(int(time.time() * 1000))
+    statement_arguments.append('{0}'.format(json.dumps(ret).replace("'", "''")))
+    statement_arguments.append('{0}'.format(json.dumps(ret['return']).replace("'", "''")))
+    statement_arguments.append(ret.get('success', False))
 
     # cassandra_cql.cql_query may raise a CommandExecutionError
     try:
-        __salt__['cassandra_cql.cql_query'](query)
+        __salt__['cassandra_cql.cql_query_with_prepare'](query,
+                                                         'returner_return',
+                                                         tuple(statement_arguments),
+                                                         async=True)
     except CommandExecutionError:
         log.critical('Could not insert into salt_returns with Cassandra returner.')
         raise
@@ -185,13 +217,19 @@ def returner(ret):
     # The data in salt.minions will be used by get_fun and get_minions
     query = '''INSERT INTO salt.minions (
                  minion_id, last_fun
-               ) VALUES (
-                 '{0}', '{1}'
-               );'''.format(ret['id'], ret['fun'])
+               ) VALUES (?, ?)'''
+
+    statement_arguments = []
+
+    statement_arguments.append('{0}'.format(ret['id']))
+    statement_arguments.append('{0}'.format(ret['fun']))
 
     # cassandra_cql.cql_query may raise a CommandExecutionError
     try:
-        __salt__['cassandra_cql.cql_query'](query)
+        __salt__['cassandra_cql.cql_query_with_prepare'](query,
+                                                         'returner_minion',
+                                                         tuple(statement_arguments),
+                                                         async=True)
     except CommandExecutionError:
         log.critical('Could not store minion ID with Cassandra returner.')
         raise
@@ -218,16 +256,19 @@ def event_return(events):
         query = '''INSERT INTO salt.salt_events (
                      id, alter_time, data, master_id, tag
                    ) VALUES (
-                     {0}, {1}, '{2}', '{3}', '{4}'
-                   );'''.format(str(uuid.uuid1()),
-                                int(time.time() * 1000),
-                                json.dumps(data).replace("'", "''"),
-                                __opts__['id'],
-                                tag)
+                     ?, ?, ?, ?, ?)
+                 '''
+        statement_arguments = [str(uuid.uuid1()),
+                               int(time.time() * 1000),
+                               json.dumps(data).replace("'", "''"),
+                               __opts__['id'],
+                               tag]
 
         # cassandra_cql.cql_query may raise a CommandExecutionError
         try:
-            __salt__['cassandra_cql.cql_query'](query)
+            __salt__['cassandra_cql.cql_query_with_prepare'](query, 'salt_events',
+                                                             statement_arguments,
+                                                             async=True)
         except CommandExecutionError:
             log.critical('Could not store events with Cassandra returner.')
             raise
@@ -245,13 +286,18 @@ def save_load(jid, load, minions=None):
     # json.dumps(load) must be escaped Cassandra style.
     query = '''INSERT INTO salt.jids (
                  jid, load
-               ) VALUES (
-                 '{0}', '{1}'
-               );'''.format(jid, json.dumps(load).replace("'", "''"))
+               ) VALUES (?, ?)'''
+
+    statement_arguments = [
+        jid,
+        json.dumps(load).replace("'", "''")
+    ]
 
     # cassandra_cql.cql_query may raise a CommandExecutionError
     try:
-        __salt__['cassandra_cql.cql_query'](query)
+        __salt__['cassandra_cql.cql_query_with_prepare'](query, 'save_load',
+                                                         statement_arguments,
+                                                         async=True)
     except CommandExecutionError:
         log.critical('Could not save load in jids table.')
         raise
@@ -260,7 +306,7 @@ def save_load(jid, load, minions=None):
         raise
 
 
-def save_minions(jid, minions):  # pylint: disable=unused-argument
+def save_minions(jid, minions, syndic_id=None):  # pylint: disable=unused-argument
     '''
     Included for API consistency
     '''
@@ -272,13 +318,13 @@ def get_load(jid):
     '''
     Return the load data that marks a specified jid
     '''
-    query = '''SELECT load FROM salt.jids WHERE jid = '{0}';'''.format(jid)
+    query = '''SELECT load FROM salt.jids WHERE jid = ?;'''
 
     ret = {}
 
     # cassandra_cql.cql_query may raise a CommandExecutionError
     try:
-        data = __salt__['cassandra_cql.cql_query'](query)
+        data = __salt__['cassandra_cql.cql_query_with_prepare'](query, 'get_load', [jid])
         if data:
             load = data[0].get('load')
             if load:
@@ -298,13 +344,13 @@ def get_jid(jid):
     '''
     Return the information returned when the specified job id was executed
     '''
-    query = '''SELECT minion_id, full_ret FROM salt.salt_returns WHERE jid = '{0}';'''.format(jid)
+    query = '''SELECT minion_id, full_ret FROM salt.salt_returns WHERE jid = ?;'''
 
     ret = {}
 
     # cassandra_cql.cql_query may raise a CommandExecutionError
     try:
-        data = __salt__['cassandra_cql.cql_query'](query)
+        data = __salt__['cassandra_cql.cql_query_with_prepare'](query, 'get_jid', [jid])
         if data:
             for row in data:
                 minion = row.get('minion_id')
@@ -326,13 +372,13 @@ def get_fun(fun):
     '''
     Return a dict of the last function called for all minions
     '''
-    query = '''SELECT minion_id, last_fun FROM salt.minions where last_fun = '{0}';'''.format(fun)
+    query = '''SELECT minion_id, last_fun FROM salt.minions where last_fun = ?;'''
 
     ret = {}
 
     # cassandra_cql.cql_query may raise a CommandExecutionError
     try:
-        data = __salt__['cassandra_cql.cql_query'](query)
+        data = __salt__['cassandra_cql.cql_query'](query, 'get_fun', [fun])
         if data:
             for row in data:
                 minion = row.get('minion_id')
@@ -354,9 +400,9 @@ def get_jids():
     '''
     Return a list of all job ids
     '''
-    query = '''SELECT DISTINCT jid FROM salt.jids;'''
+    query = '''SELECT jid, load FROM salt.jids;'''
 
-    ret = []
+    ret = {}
 
     # cassandra_cql.cql_query may raise a CommandExecutionError
     try:
@@ -364,8 +410,9 @@ def get_jids():
         if data:
             for row in data:
                 jid = row.get('jid')
-                if jid:
-                    ret.append(jid)
+                load = row.get('load')
+                if jid and load:
+                    ret[jid] = salt.utils.jid.format_jid_instance(jid, json.loads(load))
     except CommandExecutionError:
         log.critical('Could not get a list of all job ids.')
         raise
