@@ -60,19 +60,19 @@ Configuration
         Special fields:
 
         - ``vrf``: return the name of the VRF.
-        - ``connection_stats``: returning an output of the form
-        ``<State> <Active>/<Received>/<Accepted>/<Damped>``, e.g.
-        ``Established 398/399/399/0`` similar to the usual output
-        from network devices.
+        - ``connection_stats``: returning an output of the form ``<State>
+          <Active>/<Received>/<Accepted>/<Damped>``, e.g.  ``Established
+          398/399/399/0`` similar to the usual output from network devices.
         - ``interface_description``: matches the neighbor details with the
-        corresponding interface and returns its description. This will reuse
-        functionality from the :mod:`net runner <salt.runners.net.interfaces>`,
-        so the user needs to enable the mines as specified in the documentation.
+          corresponding interface and returns its description. This will reuse
+          functionality from the :mod:`net runner
+          <salt.runners.net.interfaces>`, so the user needs to enable the mines
+          as specified in the documentation.
         - ``interface_name``: matches the neighbor details with the
-        corresponding interface and returns the name.
-        Similar to ``interface_description``, this will reuse
-        functionality from the :mod:`net runner <salt.runners.net.interfaces>`,
-        so the user needs to enable the mines as specified in the documentation.
+          corresponding interface and returns the name.  Similar to
+          ``interface_description``, this will reuse functionality from the
+          :mod:`net runner <salt.runners.net.interfaces>`, so the user needs to
+          enable the mines as specified in the documentation.
 
     display: ``True``
         Display on the screen or return structured object? Default: ``True`` (return on the CLI).
@@ -237,7 +237,7 @@ def neighbors(*asns, **kwargs):
 
     Arguments:
 
-    *asns
+    asns
         A list of AS numbers to search for.
         The runner will return only the neighbors of these AS numbers.
 
