@@ -30,7 +30,7 @@ except ImportError:
 log = logging.getLogger(__name__)
 
 
-def active(outputter=None, display_progress=False):
+def active(display_progress=False):
     '''
     Return a report on all actively running jobs from a job id centric
     perspective
@@ -73,23 +73,13 @@ def active(outputter=None, display_progress=False):
             if minion not in ret[jid]['Returned']:
                 ret[jid]['Returned'].append(minion)
 
-    if outputter:
-        salt.utils.warn_until(
-            'Carbon',
-            'The \'outputter\' argument to the jobs.active runner '
-            'has been deprecated. Please specify an outputter using --out. '
-            'See the output of \'salt-run -h\' for more information.'
-        )
-        return {'outputter': outputter, 'data': ret}
-    else:
-        return ret
+    return ret
 
 
 def lookup_jid(jid,
                ext_source=None,
                returned=True,
                missing=False,
-               outputter=None,
                display_progress=False):
     '''
     Return the printout from a previously executed job
@@ -155,23 +145,14 @@ def lookup_jid(jid,
         for minion_id in (x for x in targeted_minions if x not in returns):
             ret[minion_id] = 'Minion did not return'
 
-    # Once we remove the outputter argument in a couple releases, we still
-    # need to check to see if the 'out' key is present and use it to specify
+    # We need to check to see if the 'out' key is present and use it to specify
     # the correct outputter, so we get highstate output for highstate runs.
-    if outputter is None:
-        try:
-            # Check if the return data has an 'out' key. We'll use that as the
-            # outputter in the absence of one being passed on the CLI.
-            outputter = data[next(iter(data))].get('out')
-        except (StopIteration, AttributeError):
-            outputter = None
-    else:
-        salt.utils.warn_until(
-            'Carbon',
-            'The \'outputter\' argument to the jobs.lookup_jid runner '
-            'has been deprecated. Please specify an outputter using --out. '
-            'See the output of \'salt-run -h\' for more information.'
-        )
+    try:
+        # Check if the return data has an 'out' key. We'll use that as the
+        # outputter in the absence of one being passed on the CLI.
+        outputter = returns[next(iter(returns))].get('out')
+    except (StopIteration, AttributeError):
+        outputter = None
 
     if outputter:
         return {'outputter': outputter, 'data': ret}
@@ -179,7 +160,7 @@ def lookup_jid(jid,
         return ret
 
 
-def list_job(jid, ext_source=None, outputter=None, display_progress=False):
+def list_job(jid, ext_source=None, display_progress=False):
     '''
     List a specific job given by its jid
 
@@ -222,16 +203,7 @@ def list_job(jid, ext_source=None, outputter=None, display_progress=False):
         if endtime:
             ret['EndTime'] = endtime
 
-    if outputter:
-        salt.utils.warn_until(
-            'Carbon',
-            'The \'outputter\' argument to the jobs.list_job runner '
-            'has been deprecated. Please specify an outputter using --out. '
-            'See the output of \'salt-run -h\' for more information.'
-        )
-        return {'outputter': outputter, 'data': ret}
-    else:
-        return ret
+    return ret
 
 
 def list_jobs(ext_source=None,
@@ -374,7 +346,7 @@ def list_jobs(ext_source=None,
             else:
                 log.error(
                     '\'dateutil\' library not available, skipping start_time '
-                    'comparision.'
+                    'comparison.'
                 )
 
         if end_time and _match:
@@ -387,7 +359,7 @@ def list_jobs(ext_source=None,
             else:
                 log.error(
                     '\'dateutil\' library not available, skipping end_time '
-                    'comparision.'
+                    'comparison.'
                 )
 
         if _match:
@@ -443,7 +415,7 @@ def list_jobs_filter(count,
         return ret
 
 
-def print_job(jid, ext_source=None, outputter=None):
+def print_job(jid, ext_source=None):
     '''
     Print a specific job's detail given by it's jid, including the return data.
 
@@ -480,16 +452,7 @@ def print_job(jid, ext_source=None, outputter=None):
         if endtime:
             ret[jid]['EndTime'] = endtime
 
-    if outputter:
-        salt.utils.warn_until(
-            'Carbon',
-            'The \'outputter\' argument to the jobs.print_job runner '
-            'has been deprecated. Please specify an outputter using --out. '
-            'See the output of \'salt-run -h\' for more information.'
-        )
-        return {'outputter': outputter, 'data': ret}
-    else:
-        return ret
+    return ret
 
 
 def exit_success(jid, ext_source=None):
@@ -502,20 +465,30 @@ def exit_success(jid, ext_source=None):
         The external job cache to use. Default: `None`.
 
     CLI Example:
+
     .. code-block:: bash
+
         salt-run jobs.exit_success 20160520145827701627
     '''
     ret = dict()
 
-    data = lookup_jid(
+    data = list_job(
         jid,
         ext_source=ext_source
     )
 
-    for minion in data:
-        if "retcode" in data[minion]:
-            ret[minion] = True if not data[minion]['retcode'] else False
+    minions = data.get('Minions', [])
+    result = data.get('Result', {})
 
+    for minion in minions:
+        if minion in result and 'return' in result[minion]:
+            ret[minion] = True if result[minion]['return'] else False
+        else:
+            ret[minion] = False
+
+    for minion in result:
+        if 'return' in result[minion] and result[minion]['return']:
+            ret[minion] = True
     return ret
 
 
@@ -549,7 +522,7 @@ def last_run(ext_source=None,
                           function, target, display_progress)
     if _all_jobs:
         last_job = sorted(_all_jobs)[-1]
-        return print_job(last_job, ext_source, outputter)
+        return print_job(last_job, ext_source)
     else:
         return False
 
@@ -571,7 +544,7 @@ def _format_job_instance(job):
            'Arguments': list(job.get('arg', [])),
            # unlikely but safeguard from invalid returns
            'Target': job.get('tgt', 'unknown-target'),
-           'Target-type': job.get('tgt_type', []),
+           'Target-type': job.get('tgt_type', 'list'),
            'User': job.get('user', 'root')}
 
     if 'metadata' in job:
@@ -606,12 +579,14 @@ def _walk_through(job_dir, display_progress=False):
 
         for final in os.listdir(t_path):
             load_path = os.path.join(t_path, final, '.load.p')
-            job = serial.load(salt.utils.fopen(load_path, 'rb'))
+            with salt.utils.fopen(load_path, 'rb') as rfh:
+                job = serial.load(rfh)
 
             if not os.path.isfile(load_path):
                 continue
 
-            job = serial.load(salt.utils.fopen(load_path, 'rb'))
+            with salt.utils.fopen(load_path, 'rb') as rfh:
+                job = serial.load(rfh)
             jid = job['jid']
             if display_progress:
                 __jid_event__.fire_event(
