@@ -69,10 +69,10 @@ import hashlib
 import json
 
 # Import Salt Libs
+import salt.ext.six as six
 import salt.utils.dictupdate as dictupdate
 import salt.utils
 from salt.exceptions import SaltInvocationError
-from salt.ext.six import string_types
 
 log = logging.getLogger(__name__)
 
@@ -84,11 +84,12 @@ def __virtual__():
     return 'boto_lambda' if 'boto_lambda.function_exists' in __salt__ else False
 
 
-def function_present(name, FunctionName, Runtime, Role, Handler, ZipFile=None, S3Bucket=None,
-            S3Key=None, S3ObjectVersion=None,
-            Description='', Timeout=3, MemorySize=128,
-            Permissions=None, RoleRetries=5,
-            region=None, key=None, keyid=None, profile=None):
+def function_present(name, FunctionName, Runtime, Role, Handler, ZipFile=None,
+                     S3Bucket=None, S3Key=None, S3ObjectVersion=None,
+                     Description='', Timeout=3, MemorySize=128,
+                     Permissions=None, RoleRetries=5, region=None, key=None,
+                     keyid=None, profile=None, VpcConfig=None,
+                     Environment=None):
     '''
     Ensure function exists.
 
@@ -145,6 +146,27 @@ def function_present(name, FunctionName, Runtime, Role, Handler, ZipFile=None, S
         to an image processing function. The default value is 128 MB. The value must be a multiple of
         64 MB.
 
+    VpcConfig
+        If your Lambda function accesses resources in a VPC, you must provide this parameter
+        identifying the list of security group IDs/Names and subnet IDs/Name.  These must all belong
+        to the same VPC.  This is a dict of the form:
+
+        .. code-block:: yaml
+
+            VpcConfig:
+              SecurityGroupNames:
+                - mysecgroup1
+                - mysecgroup2
+              SecurityGroupIds:
+                - sg-abcdef1234
+              SubnetNames:
+                - mysubnet1
+              SubnetIds:
+                - subnet-1234abcd
+                - subnet-abcd1234
+
+        If VpcConfig is provided at all, you MUST pass at least one security group and one subnet.
+
     Permissions
         A list of permission definitions to be added to the function's policy
 
@@ -153,6 +175,20 @@ def function_present(name, FunctionName, Runtime, Role, Handler, ZipFile=None, S
         During that time function creation may fail; this state will
         atuomatically retry this number of times. The default is 5.
 
+    Environment
+        The parent object that contains your environment's configuration
+        settings.  This is a dictionary of the form:
+
+        .. code-block:: python
+
+            {
+                'Variables': {
+                    'VariableName': 'VariableValue'
+                }
+            }
+
+        .. versionadded:: 2017.7.0
+
     region
         Region to connect to.
 
@@ -173,74 +209,82 @@ def function_present(name, FunctionName, Runtime, Role, Handler, ZipFile=None, S
            }
 
     if Permissions is not None:
-        if isinstance(Permissions, string_types):
+        if isinstance(Permissions, six.string_types):
             Permissions = json.loads(Permissions)
         required_keys = set(('Action', 'Principal'))
         optional_keys = set(('SourceArn', 'SourceAccount'))
-        for sid, permission in Permissions.iteritems():
+        for sid, permission in six.iteritems(Permissions):
             keyset = set(permission.keys())
             if not keyset.issuperset(required_keys):
                 raise SaltInvocationError('{0} are required for each permission '
-                           'specification'.format(', '.join(required_keys)))
+                                          'specification'.format(', '.join(required_keys)))
             keyset = keyset - required_keys
             keyset = keyset - optional_keys
             if bool(keyset):
-                raise SaltInvocationError('Invalid permission value {0}'.format(', '.join(keyset)))
+                raise SaltInvocationError(
+                    'Invalid permission value {0}'.format(', '.join(keyset)))
 
-    r = __salt__['boto_lambda.function_exists'](FunctionName=FunctionName, region=region,
-                                                key=key, keyid=keyid, profile=profile)
+    r = __salt__['boto_lambda.function_exists'](
+        FunctionName=FunctionName, region=region,
+        key=key, keyid=keyid, profile=profile)
 
     if 'error' in r:
         ret['result'] = False
-        ret['comment'] = 'Failed to create function: {0}.'.format(r['error']['message'])
+        ret['comment'] = ('Failed to create function: '
+                          '{0}.'.format(r['error']['message']))
         return ret
 
     if not r.get('exists'):
         if __opts__['test']:
-            ret['comment'] = 'Function {0} is set to be created.'.format(FunctionName)
+            ret['comment'] = 'Function {0} is set to be created.'.format(
+                FunctionName)
             ret['result'] = None
             return ret
-        r = __salt__['boto_lambda.create_function'](FunctionName=FunctionName, Runtime=Runtime,
-                                                    Role=Role, Handler=Handler,
-                                                    ZipFile=ZipFile, S3Bucket=S3Bucket,
-                                                    S3Key=S3Key,
-                                                    S3ObjectVersion=S3ObjectVersion,
-                                                    Description=Description,
-                                                    Timeout=Timeout, MemorySize=MemorySize,
-                                                    WaitForRole=True,
-                                                    RoleRetries=RoleRetries,
-                                                    region=region, key=key,
-                                                    keyid=keyid, profile=profile)
+        r = __salt__['boto_lambda.create_function'](
+            FunctionName=FunctionName, Runtime=Runtime, Role=Role,
+            Handler=Handler, ZipFile=ZipFile, S3Bucket=S3Bucket, S3Key=S3Key,
+            S3ObjectVersion=S3ObjectVersion, Description=Description,
+            Timeout=Timeout, MemorySize=MemorySize, VpcConfig=VpcConfig,
+            Environment=Environment, WaitForRole=True, RoleRetries=RoleRetries,
+            region=region, key=key, keyid=keyid, profile=profile)
         if not r.get('created'):
             ret['result'] = False
-            ret['comment'] = 'Failed to create function: {0}.'.format(r['error']['message'])
+            ret['comment'] = ('Failed to create function: '
+                              '{0}.'.format(r['error']['message']))
             return ret
 
         if Permissions:
-            for sid, permission in Permissions.iteritems():
-                r = __salt__['boto_lambda.add_permission'](FunctionName=FunctionName,
-                                                       StatementId=sid,
-                                                       **permission)
+            for sid, permission in six.iteritems(Permissions):
+                r = __salt__['boto_lambda.add_permission'](
+                    FunctionName=FunctionName, StatementId=sid,
+                    region=region, key=key, keyid=keyid, profile=profile,
+                    **permission)
                 if not r.get('updated'):
                     ret['result'] = False
-                    ret['comment'] = 'Failed to create function: {0}.'.format(r['error']['message'])
+                    ret['comment'] = ('Failed to create function: '
+                                      '{0}.'.format(r['error']['message']))
 
-        _describe = __salt__['boto_lambda.describe_function'](FunctionName,
-                           region=region, key=key, keyid=keyid, profile=profile)
-        _describe['function']['Permissions'] = __salt__['boto_lambda.get_permissions'](FunctionName,
-                           region=region, key=key, keyid=keyid, profile=profile)['permissions']
+        _describe = __salt__['boto_lambda.describe_function'](
+            FunctionName, region=region, key=key, keyid=keyid, profile=profile)
+        _describe['function']['Permissions'] = (
+            __salt__['boto_lambda.get_permissions'](
+                FunctionName, region=region, key=key, keyid=keyid,
+                profile=profile)['permissions'])
         ret['changes']['old'] = {'function': None}
         ret['changes']['new'] = _describe
         ret['comment'] = 'Function {0} created.'.format(FunctionName)
         return ret
 
-    ret['comment'] = os.linesep.join([ret['comment'], 'Function {0} is present.'.format(FunctionName)])
+    ret['comment'] = os.linesep.join(
+        [ret['comment'], 'Function {0} is present.'.format(FunctionName)])
     ret['changes'] = {}
     # function exists, ensure config matches
-    _ret = _function_config_present(FunctionName, Role, Handler, Description, Timeout,
-                                    MemorySize, region, key, keyid, profile)
+    _ret = _function_config_present(FunctionName, Role, Handler, Description,
+                                    Timeout, MemorySize, VpcConfig,
+                                    Environment, region, key, keyid,
+                                    profile, RoleRetries)
     if not _ret.get('result'):
-        ret['result'] = False
+        ret['result'] = _ret.get('result', False)
         ret['comment'] = _ret['comment']
         ret['changes'] = {}
         return ret
@@ -249,7 +293,7 @@ def function_present(name, FunctionName, Runtime, Role, Handler, ZipFile=None, S
     _ret = _function_code_present(FunctionName, ZipFile, S3Bucket, S3Key, S3ObjectVersion,
                                   region, key, keyid, profile)
     if not _ret.get('result'):
-        ret['result'] = False
+        ret['result'] = _ret.get('result', False)
         ret['comment'] = _ret['comment']
         ret['changes'] = {}
         return ret
@@ -258,7 +302,7 @@ def function_present(name, FunctionName, Runtime, Role, Handler, ZipFile=None, S
     _ret = _function_permissions_present(FunctionName, Permissions,
                                          region, key, keyid, profile)
     if not _ret.get('result'):
-        ret['result'] = False
+        ret['result'] = _ret.get('result', False)
         ret['comment'] = _ret['comment']
         ret['changes'] = {}
         return ret
@@ -277,48 +321,88 @@ def _get_role_arn(name, region=None, key=None, keyid=None, profile=None):
     return 'arn:aws:iam::{0}:role/{1}'.format(account_id, name)
 
 
+def _resolve_vpcconfig(conf, region=None, key=None, keyid=None, profile=None):
+    if isinstance(conf, six.string_types):
+        conf = json.loads(conf)
+    if not conf:
+        return None
+    if not isinstance(conf, dict):
+        raise SaltInvocationError('VpcConfig must be a dict.')
+    sns = [__salt__['boto_vpc.get_resource_id']('subnet', s, region=region, key=key,
+            keyid=keyid, profile=profile).get('id') for s in conf.pop('SubnetNames', [])]
+    sgs = [__salt__['boto_secgroup.get_group_id'](s, region=region, key=key, keyid=keyid,
+            profile=profile) for s in conf.pop('SecurityGroupNames', [])]
+    conf.setdefault('SubnetIds', []).extend(sns)
+    conf.setdefault('SecurityGroupIds', []).extend(sgs)
+    return conf
+
+
 def _function_config_present(FunctionName, Role, Handler, Description, Timeout,
-                           MemorySize, region, key, keyid, profile):
+                             MemorySize, VpcConfig, Environment, region,
+                             key, keyid, profile, RoleRetries):
     ret = {'result': True, 'comment': '', 'changes': {}}
-    func = __salt__['boto_lambda.describe_function'](FunctionName,
-           region=region, key=key, keyid=keyid, profile=profile)['function']
+    func = __salt__['boto_lambda.describe_function'](
+        FunctionName, region=region,
+        key=key, keyid=keyid, profile=profile)['function']
     role_arn = _get_role_arn(Role, region, key, keyid, profile)
     need_update = False
-    for val, var in {
-        'Role': 'role_arn',
-        'Handler': 'Handler',
-        'Description': 'Description',
-        'Timeout': 'Timeout',
-        'MemorySize': 'MemorySize',
-    }.iteritems():
+    options = {'Role': 'role_arn',
+               'Handler': 'Handler',
+               'Description': 'Description',
+               'Timeout': 'Timeout',
+               'MemorySize': 'MemorySize'}
+
+    for val, var in six.iteritems(options):
         if func[val] != locals()[var]:
             need_update = True
             ret['changes'].setdefault('new', {})[var] = locals()[var]
             ret['changes'].setdefault('old', {})[var] = func[val]
+    # VpcConfig returns the extra value 'VpcId' so do a special compare
+    oldval = func.get('VpcConfig')
+    if oldval is not None:
+        oldval.pop('VpcId', None)
+    fixed_VpcConfig = _resolve_vpcconfig(VpcConfig, region, key, keyid, profile)
+    if __utils__['boto3.ordered'](oldval) != __utils__['boto3.ordered'](fixed_VpcConfig):
+        need_update = True
+        ret['changes'].setdefault('new', {})['VpcConfig'] = VpcConfig
+        ret['changes'].setdefault(
+            'old', {})['VpcConfig'] = func.get('VpcConfig')
+
+    if Environment is not None:
+        if func.get('Environment') != Environment:
+            need_update = True
+            ret['changes'].setdefault('new', {})['Environment'] = Environment
+            ret['changes'].setdefault('old', {})['Environment'] = func.get(
+                'Environment')
+
     if need_update:
-        ret['comment'] = os.linesep.join([ret['comment'], 'Function config to be modified'])
+        ret['comment'] = os.linesep.join(
+            [ret['comment'], 'Function config to be modified'])
         if __opts__['test']:
             msg = 'Function {0} set to be modified.'.format(FunctionName)
             ret['comment'] = msg
             ret['result'] = None
             return ret
-        _r = __salt__['boto_lambda.update_function_config'](FunctionName=FunctionName,
-                                        Role=Role, Handler=Handler, Description=Description,
-                                        Timeout=Timeout, MemorySize=MemorySize,
-                                        region=region, key=key,
-                                        keyid=keyid, profile=profile)
+        _r = __salt__['boto_lambda.update_function_config'](
+            FunctionName=FunctionName, Role=Role, Handler=Handler,
+            Description=Description, Timeout=Timeout, MemorySize=MemorySize,
+            VpcConfig=VpcConfig, Environment=Environment, region=region,
+            key=key, keyid=keyid, profile=profile, WaitForRole=True,
+            RoleRetries=RoleRetries)
         if not _r.get('updated'):
             ret['result'] = False
-            ret['comment'] = 'Failed to update function: {0}.'.format(_r['error']['message'])
+            ret['comment'] = ('Failed to update function: '
+                              '{0}.'.format(_r['error']['message']))
             ret['changes'] = {}
     return ret
 
 
-def _function_code_present(FunctionName, ZipFile, S3Bucket, S3Key, S3ObjectVersion,
-                         region, key, keyid, profile):
+def _function_code_present(FunctionName, ZipFile, S3Bucket, S3Key,
+                           S3ObjectVersion, region, key, keyid, profile):
     ret = {'result': True, 'comment': '', 'changes': {}}
-    func = __salt__['boto_lambda.describe_function'](FunctionName,
-           region=region, key=key, keyid=keyid, profile=profile)['function']
+    func = __salt__['boto_lambda.describe_function'](
+        FunctionName, region=region,
+        key=key, keyid=keyid, profile=profile)['function']
     update = False
     if ZipFile:
         size = os.path.getsize(ZipFile)
@@ -346,18 +430,21 @@ def _function_code_present(FunctionName, ZipFile, S3Bucket, S3Key, S3ObjectVersi
             'CodeSha256': func['CodeSha256'],
             'CodeSize': func['CodeSize'],
         }
-        func = __salt__['boto_lambda.update_function_code'](FunctionName, ZipFile, S3Bucket,
+        func = __salt__['boto_lambda.update_function_code'](
+            FunctionName, ZipFile, S3Bucket,
             S3Key, S3ObjectVersion,
             region=region, key=key, keyid=keyid, profile=profile)
         if not func.get('updated'):
             ret['result'] = False
-            ret['comment'] = 'Failed to update function: {0}.'.format(func['error']['message'])
+            ret['comment'] = ('Failed to update function: '
+                              '{0}.'.format(func['error']['message']))
             ret['changes'] = {}
             return ret
         func = func['function']
         if func['CodeSha256'] != ret['changes']['old']['CodeSha256'] or \
                 func['CodeSize'] != ret['changes']['old']['CodeSize']:
-            ret['comment'] = os.linesep.join([ret['comment'], 'Function code to be modified'])
+            ret['comment'] = os.linesep.join(
+                [ret['comment'], 'Function code to be modified'])
             ret['changes']['new'] = {
                 'CodeSha256': func['CodeSha256'],
                 'CodeSize': func['CodeSize'],
@@ -370,44 +457,47 @@ def _function_code_present(FunctionName, ZipFile, S3Bucket, S3Key, S3ObjectVersi
 def _function_permissions_present(FunctionName, Permissions,
                                   region, key, keyid, profile):
     ret = {'result': True, 'comment': '', 'changes': {}}
-    curr_permissions = __salt__['boto_lambda.get_permissions'](FunctionName,
-           region=region, key=key, keyid=keyid, profile=profile)['permissions']
+    curr_permissions = __salt__['boto_lambda.get_permissions'](
+        FunctionName, region=region,
+        key=key, keyid=keyid, profile=profile).get('permissions')
+    if curr_permissions is None:
+        curr_permissions = {}
     need_update = False
     diffs = salt.utils.compare_dicts(curr_permissions, Permissions or {})
     if bool(diffs):
-        ret['comment'] = os.linesep.join([ret['comment'], 'Function permissions to be modified'])
+        ret['comment'] = os.linesep.join(
+            [ret['comment'], 'Function permissions to be modified'])
         if __opts__['test']:
             msg = 'Function {0} set to be modified.'.format(FunctionName)
             ret['comment'] = msg
             ret['result'] = None
             return ret
-        for sid, diff in diffs.iteritems():
+        for sid, diff in six.iteritems(diffs):
             if diff.get('old', '') != '':
                 # There's a permssion that needs to be removed
-                _r = __salt__['boto_lambda.remove_permission'](FunctionName=FunctionName,
-                                        StatementId=sid,
-                                        region=region, key=key,
-                                        keyid=keyid, profile=profile)
-                ret['changes'].setdefault('new', {}).setdefault('Permissions',
-                                 {})[sid] = {}
-                ret['changes'].setdefault('old', {}).setdefault('Permissions',
-                                 {})[sid] = diff['old']
+                _r = __salt__['boto_lambda.remove_permission'](
+                    FunctionName=FunctionName, StatementId=sid,
+                    region=region, key=key, keyid=keyid, profile=profile)
+                ret['changes'].setdefault(
+                    'new', {}).setdefault('Permissions', {})[sid] = {}
+                ret['changes'].setdefault(
+                    'old', {}).setdefault('Permissions', {})[sid] = diff['old']
             if diff.get('new', '') != '':
                 # New permission information needs to be added
-                _r = __salt__['boto_lambda.add_permission'](FunctionName=FunctionName,
-                                        StatementId=sid,
-                                        region=region, key=key,
-                                        keyid=keyid, profile=profile,
-                                        **diff['new'])
-                ret['changes'].setdefault('new', {}).setdefault('Permissions',
-                                 {})[sid] = diff['new']
-                oldperms = ret['changes'].setdefault('old', {}).setdefault('Permissions',
-                                 {})
+                _r = __salt__['boto_lambda.add_permission'](
+                    FunctionName=FunctionName, StatementId=sid,
+                    region=region, key=key, keyid=keyid, profile=profile,
+                    **diff['new'])
+                ret['changes'].setdefault(
+                    'new', {}).setdefault('Permissions', {})[sid] = diff['new']
+                oldperms = ret['changes'].setdefault(
+                    'old', {}).setdefault('Permissions', {})
                 if sid not in oldperms:
                     oldperms[sid] = {}
             if not _r.get('updated'):
                 ret['result'] = False
-                ret['comment'] = 'Failed to update function: {0}.'.format(_r['error']['message'])
+                ret['comment'] = ('Failed to update function: '
+                                  '{0}.'.format(_r['error']['message']))
                 ret['changes'] = {}
     return ret
 
@@ -442,11 +532,12 @@ def function_absent(name, FunctionName, region=None, key=None, keyid=None, profi
            'changes': {}
            }
 
-    r = __salt__['boto_lambda.function_exists'](FunctionName, region=region,
-                                    key=key, keyid=keyid, profile=profile)
+    r = __salt__['boto_lambda.function_exists'](
+        FunctionName, region=region, key=key, keyid=keyid, profile=profile)
     if 'error' in r:
         ret['result'] = False
-        ret['comment'] = 'Failed to delete function: {0}.'.format(r['error']['message'])
+        ret['comment'] = 'Failed to delete function: {0}.'.format(r['error'][
+            'message'])
         return ret
 
     if r and not r['exists']:
@@ -454,15 +545,17 @@ def function_absent(name, FunctionName, region=None, key=None, keyid=None, profi
         return ret
 
     if __opts__['test']:
-        ret['comment'] = 'Function {0} is set to be removed.'.format(FunctionName)
+        ret['comment'] = 'Function {0} is set to be removed.'.format(
+            FunctionName)
         ret['result'] = None
         return ret
     r = __salt__['boto_lambda.delete_function'](FunctionName,
-                                    region=region, key=key,
-                                    keyid=keyid, profile=profile)
+                                                region=region, key=key,
+                                                keyid=keyid, profile=profile)
     if not r['deleted']:
         ret['result'] = False
-        ret['comment'] = 'Failed to delete function: {0}.'.format(r['error']['message'])
+        ret['comment'] = ('Failed to delete function: '
+                          '{0}.'.format(r['error']['message']))
         return ret
     ret['changes']['old'] = {'function': FunctionName}
     ret['changes']['new'] = {'function': None}
@@ -471,7 +564,7 @@ def function_absent(name, FunctionName, region=None, key=None, keyid=None, profi
 
 
 def alias_present(name, FunctionName, Name, FunctionVersion, Description='',
-            region=None, key=None, keyid=None, profile=None):
+                  region=None, key=None, keyid=None, profile=None):
     '''
     Ensure alias exists.
 
@@ -510,12 +603,14 @@ def alias_present(name, FunctionName, Name, FunctionVersion, Description='',
            'changes': {}
            }
 
-    r = __salt__['boto_lambda.alias_exists'](FunctionName=FunctionName, Name=Name, region=region,
-                                    key=key, keyid=keyid, profile=profile)
+    r = __salt__['boto_lambda.alias_exists'](
+        FunctionName=FunctionName, Name=Name, region=region,
+        key=key, keyid=keyid, profile=profile)
 
     if 'error' in r:
         ret['result'] = False
-        ret['comment'] = 'Failed to create alias: {0}.'.format(r['error']['message'])
+        ret['comment'] = ('Failed to create alias: '
+                          '{0}.'.format(r['error']['message']))
         return ret
 
     if not r.get('exists'):
@@ -524,48 +619,53 @@ def alias_present(name, FunctionName, Name, FunctionVersion, Description='',
             ret['result'] = None
             return ret
         r = __salt__['boto_lambda.create_alias'](FunctionName, Name,
-                FunctionVersion, Description,
-                region, key, keyid, profile)
+                                                 FunctionVersion, Description,
+                                                 region, key, keyid, profile)
         if not r.get('created'):
             ret['result'] = False
-            ret['comment'] = 'Failed to create alias: {0}.'.format(r['error']['message'])
+            ret['comment'] = ('Failed to create alias: '
+                              '{0}.'.format(r['error']['message']))
             return ret
-        _describe = __salt__['boto_lambda.describe_alias'](FunctionName, Name, region=region, key=key,
-                                                  keyid=keyid, profile=profile)
+        _describe = __salt__['boto_lambda.describe_alias'](
+            FunctionName, Name, region=region, key=key,
+            keyid=keyid, profile=profile)
         ret['changes']['old'] = {'alias': None}
         ret['changes']['new'] = _describe
         ret['comment'] = 'Alias {0} created.'.format(Name)
         return ret
 
-    ret['comment'] = os.linesep.join([ret['comment'], 'Alias {0} is present.'.format(Name)])
+    ret['comment'] = os.linesep.join(
+        [ret['comment'], 'Alias {0} is present.'.format(Name)])
     ret['changes'] = {}
-    _describe = __salt__['boto_lambda.describe_alias'](FunctionName, Name,
-                                                  region=region, key=key, keyid=keyid,
-                                                  profile=profile)['alias']
+    _describe = __salt__['boto_lambda.describe_alias'](
+        FunctionName, Name, region=region, key=key, keyid=keyid,
+        profile=profile)['alias']
 
     need_update = False
-    for val, var in {
-        'FunctionVersion': 'FunctionVersion',
-        'Description': 'Description',
-    }.iteritems():
+    options = {'FunctionVersion': 'FunctionVersion',
+               'Description': 'Description'}
+
+    for val, var in six.iteritems(options):
         if _describe[val] != locals()[var]:
             need_update = True
             ret['changes'].setdefault('new', {})[var] = locals()[var]
             ret['changes'].setdefault('old', {})[var] = _describe[val]
     if need_update:
-        ret['comment'] = os.linesep.join([ret['comment'], 'Alias config to be modified'])
+        ret['comment'] = os.linesep.join(
+            [ret['comment'], 'Alias config to be modified'])
         if __opts__['test']:
             msg = 'Alias {0} set to be modified.'.format(Name)
             ret['comment'] = msg
             ret['result'] = None
             return ret
-        _r = __salt__['boto_lambda.update_alias'](FunctionName=FunctionName, Name=Name,
-                                        FunctionVersion=FunctionVersion, Description=Description,
-                                        region=region, key=key,
-                                        keyid=keyid, profile=profile)
+        _r = __salt__['boto_lambda.update_alias'](
+            FunctionName=FunctionName, Name=Name,
+            FunctionVersion=FunctionVersion, Description=Description,
+            region=region, key=key, keyid=keyid, profile=profile)
         if not _r.get('updated'):
             ret['result'] = False
-            ret['comment'] = 'Failed to update alias: {0}.'.format(_r['error']['message'])
+            ret['comment'] = ('Failed to update alias: '
+                              '{0}.'.format(_r['error']['message']))
             ret['changes'] = {}
     return ret
 
@@ -603,11 +703,13 @@ def alias_absent(name, FunctionName, Name, region=None, key=None, keyid=None, pr
            'changes': {}
            }
 
-    r = __salt__['boto_lambda.alias_exists'](FunctionName, Name, region=region,
-                                    key=key, keyid=keyid, profile=profile)
+    r = __salt__['boto_lambda.alias_exists'](
+        FunctionName, Name, region=region, key=key, keyid=keyid,
+        profile=profile)
     if 'error' in r:
         ret['result'] = False
-        ret['comment'] = 'Failed to delete alias: {0}.'.format(r['error']['message'])
+        ret['comment'] = ('Failed to delete alias: '
+                          '{0}.'.format(r['error']['message']))
         return ret
 
     if r and not r['exists']:
@@ -618,12 +720,13 @@ def alias_absent(name, FunctionName, Name, region=None, key=None, keyid=None, pr
         ret['comment'] = 'Alias {0} is set to be removed.'.format(Name)
         ret['result'] = None
         return ret
-    r = __salt__['boto_lambda.delete_alias'](FunctionName, Name,
-                                    region=region, key=key,
-                                    keyid=keyid, profile=profile)
+    r = __salt__['boto_lambda.delete_alias'](
+        FunctionName, Name, region=region, key=key, keyid=keyid,
+        profile=profile)
     if not r['deleted']:
         ret['result'] = False
-        ret['comment'] = 'Failed to delete alias: {0}.'.format(r['error']['message'])
+        ret['comment'] = ('Failed to delete alias: '
+                          '{0}.'.format(r['error']['message']))
         return ret
     ret['changes']['old'] = {'alias': Name}
     ret['changes']['new'] = {'alias': None}
@@ -645,9 +748,10 @@ def _get_function_arn(name, region=None, key=None, keyid=None, profile=None):
     return 'arn:aws:lambda:{0}:{1}:function:{2}'.format(region, account_id, name)
 
 
-def event_source_mapping_present(name, EventSourceArn, FunctionName, StartingPosition,
-            Enabled=True, BatchSize=100,
-            region=None, key=None, keyid=None, profile=None):
+def event_source_mapping_present(name, EventSourceArn, FunctionName,
+                                 StartingPosition, Enabled=True, BatchSize=100,
+                                 region=None, key=None, keyid=None,
+                                 profile=None):
     '''
     Ensure event source mapping exists.
 
@@ -703,83 +807,91 @@ def event_source_mapping_present(name, EventSourceArn, FunctionName, StartingPos
            'changes': {}
            }
 
-    r = __salt__['boto_lambda.event_source_mapping_exists'](EventSourceArn=EventSourceArn,
-                                    FunctionName=FunctionName,
-                                    region=region, key=key, keyid=keyid, profile=profile)
+    r = __salt__['boto_lambda.event_source_mapping_exists'](
+        EventSourceArn=EventSourceArn, FunctionName=FunctionName,
+        region=region, key=key, keyid=keyid, profile=profile)
 
     if 'error' in r:
         ret['result'] = False
-        ret['comment'] = 'Failed to create event source mapping: {0}.'.format(r['error']['message'])
+        ret['comment'] = ('Failed to create event source mapping: '
+                          '{0}.'.format(r['error']['message']))
         return ret
 
     if not r.get('exists'):
         if __opts__['test']:
-            ret['comment'] = 'Event source mapping {0} is set to be created.'.format(FunctionName)
+            ret['comment'] = ('Event source mapping {0} is set '
+                              'to be created.'.format(FunctionName))
             ret['result'] = None
             return ret
-        r = __salt__['boto_lambda.create_event_source_mapping'](EventSourceArn=EventSourceArn,
-                    FunctionName=FunctionName, StartingPosition=StartingPosition,
-                    Enabled=Enabled, BatchSize=BatchSize,
-                    region=region, key=key, keyid=keyid, profile=profile)
+        r = __salt__['boto_lambda.create_event_source_mapping'](
+            EventSourceArn=EventSourceArn, FunctionName=FunctionName,
+            StartingPosition=StartingPosition, Enabled=Enabled,
+            BatchSize=BatchSize, region=region, key=key, keyid=keyid,
+            profile=profile)
         if not r.get('created'):
             ret['result'] = False
-            ret['comment'] = 'Failed to create event source mapping: {0}.'.format(r['error']['message'])
+            ret['comment'] = ('Failed to create event source mapping: '
+                              '{0}.'.format(r['error']['message']))
             return ret
         _describe = __salt__['boto_lambda.describe_event_source_mapping'](
-                                 EventSourceArn=EventSourceArn,
-                                 FunctionName=FunctionName,
-                                 region=region, key=key, keyid=keyid, profile=profile)
+            EventSourceArn=EventSourceArn, FunctionName=FunctionName,
+            region=region, key=key, keyid=keyid, profile=profile)
         ret['name'] = _describe['event_source_mapping']['UUID']
         ret['changes']['old'] = {'event_source_mapping': None}
         ret['changes']['new'] = _describe
-        ret['comment'] = 'Event source mapping {0} created.'.format(ret['name'])
+        ret['comment'] = ('Event source mapping {0} '
+                          'created.'.format(ret['name']))
         return ret
 
-    ret['comment'] = os.linesep.join([ret['comment'], 'Event source mapping is present.'])
+    ret['comment'] = os.linesep.join(
+        [ret['comment'], 'Event source mapping is present.'])
     ret['changes'] = {}
     _describe = __salt__['boto_lambda.describe_event_source_mapping'](
-                                 EventSourceArn=EventSourceArn,
-                                 FunctionName=FunctionName,
-                                 region=region, key=key, keyid=keyid, profile=profile)['event_source_mapping']
+        EventSourceArn=EventSourceArn, FunctionName=FunctionName,
+        region=region, key=key, keyid=keyid,
+        profile=profile)['event_source_mapping']
 
     need_update = False
-    for val, var in {
-        'BatchSize': 'BatchSize',
-    }.iteritems():
+    options = {'BatchSize': 'BatchSize'}
+
+    for val, var in six.iteritems(options):
         if _describe[val] != locals()[var]:
             need_update = True
             ret['changes'].setdefault('new', {})[var] = locals()[var]
             ret['changes'].setdefault('old', {})[var] = _describe[val]
     # verify FunctionName against FunctionArn
-    function_arn = _get_function_arn(FunctionName,
-                    region=region, key=key, keyid=keyid, profile=profile)
+    function_arn = _get_function_arn(FunctionName, region=region,
+                                     key=key, keyid=keyid, profile=profile)
     if _describe['FunctionArn'] != function_arn:
         need_update = True
         ret['changes'].setdefault('new', {})['FunctionArn'] = function_arn
-        ret['changes'].setdefault('old', {})['FunctionArn'] = _describe['FunctionArn']
-    # TODO check for 'Enabled', since it doesn't directly map to a specific state
+        ret['changes'].setdefault('old', {})['FunctionArn'] = _describe[
+            'FunctionArn']
+    # TODO check for 'Enabled', since it doesn't directly map to a specific
+    # state
     if need_update:
-        ret['comment'] = os.linesep.join([ret['comment'], 'Event source mapping to be modified'])
+        ret['comment'] = os.linesep.join(
+            [ret['comment'], 'Event source mapping to be modified'])
         if __opts__['test']:
-            msg = 'Event source mapping {0} set to be modified.'.format(_describe['UUID'])
+            msg = ('Event source mapping {0} set to be '
+                   'modified.'.format(_describe['UUID']))
             ret['comment'] = msg
             ret['result'] = None
             return ret
-        _r = __salt__['boto_lambda.update_event_source_mapping'](UUID=_describe['UUID'],
-                                        FunctionName=FunctionName,
-                                        Enabled=Enabled,
-                                        BatchSize=BatchSize,
-                                        region=region, key=key,
-                                        keyid=keyid, profile=profile)
+        _r = __salt__['boto_lambda.update_event_source_mapping'](
+            UUID=_describe['UUID'], FunctionName=FunctionName,
+            Enabled=Enabled, BatchSize=BatchSize,
+            region=region, key=key, keyid=keyid, profile=profile)
         if not _r.get('updated'):
             ret['result'] = False
-            ret['comment'] = 'Failed to update mapping: {0}.'.format(_r['error']['message'])
+            ret['comment'] = ('Failed to update mapping: '
+                              '{0}.'.format(_r['error']['message']))
             ret['changes'] = {}
     return ret
 
 
 def event_source_mapping_absent(name, EventSourceArn, FunctionName,
-                           region=None, key=None, keyid=None, profile=None):
+                                region=None, key=None, keyid=None, profile=None):
     '''
     Ensure event source mapping with passed properties is absent.
 
@@ -812,12 +924,13 @@ def event_source_mapping_absent(name, EventSourceArn, FunctionName,
            'changes': {}
            }
 
-    desc = __salt__['boto_lambda.describe_event_source_mapping'](EventSourceArn=EventSourceArn,
-                                    FunctionName=FunctionName,
-                                    region=region, key=key, keyid=keyid, profile=profile)
+    desc = __salt__['boto_lambda.describe_event_source_mapping'](
+        EventSourceArn=EventSourceArn, FunctionName=FunctionName,
+        region=region, key=key, keyid=keyid, profile=profile)
     if 'error' in desc:
         ret['result'] = False
-        ret['comment'] = 'Failed to delete event source mapping: {0}.'.format(desc['error']['message'])
+        ret['comment'] = ('Failed to delete event source mapping: '
+                          '{0}.'.format(desc['error']['message']))
         return ret
 
     if not desc.get('event_source_mapping'):
@@ -829,13 +942,13 @@ def event_source_mapping_absent(name, EventSourceArn, FunctionName,
         ret['comment'] = 'Event source mapping is set to be removed.'
         ret['result'] = None
         return ret
-    r = __salt__['boto_lambda.delete_event_source_mapping'](EventSourceArn=EventSourceArn,
-                                        FunctionName=FunctionName,
-                                        region=region, key=key,
-                                        keyid=keyid, profile=profile)
+    r = __salt__['boto_lambda.delete_event_source_mapping'](
+        EventSourceArn=EventSourceArn, FunctionName=FunctionName,
+        region=region, key=key, keyid=keyid, profile=profile)
     if not r['deleted']:
         ret['result'] = False
-        ret['comment'] = 'Failed to delete event source mapping: {0}.'.format(r['error']['message'])
+        ret['comment'] = 'Failed to delete event source mapping: {0}.'.format(r['error'][
+            'message'])
         return ret
     ret['changes']['old'] = desc
     ret['changes']['new'] = {'event_source_mapping': None}
