@@ -37,9 +37,9 @@ config:
     myprofile:
         keyid: GKTADJGHEIQSXMKKRBJ08H
         key: askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs
-            region: us-east-1
+        region: us-east-1
 
-.. code-block:: yaml
+.. code-block:: text
 
     Ensure bucket exists:
         boto_s3_bucket.present:
@@ -141,14 +141,11 @@ config:
 # Import Python Libs
 from __future__ import absolute_import
 import logging
-import os
-import os.path
 from copy import deepcopy
 import json
 
-# Import Salt Libs
-from salt.ext.six import string_types  # pylint: disable=import-error
-from salt.utils.boto3 import json_objs_equal
+# Import Salt libs
+from salt.ext import six
 
 log = logging.getLogger(__name__)
 
@@ -195,11 +192,10 @@ def _acl_to_grant(ACL, owner_canonical_id):
             item['Grantee'] = _normalize_user(item.get('Grantee'))
         # If AccessControlPolicy is set, other options are not allowed
         return ret
+    owner_canonical_grant = deepcopy(owner_canonical_id)
+    owner_canonical_grant.update({'Type': 'CanonicalUser'})
     ret = {
-        'Grants': [{
-            'Grantee': owner_canonical_id,
-            'Permission': 'FULL_CONTROL'
-        }],
+        'Grants': [],
         'Owner': owner_canonical_id
     }
     if 'ACL' in ACL:
@@ -208,6 +204,7 @@ def _acl_to_grant(ACL, owner_canonical_id):
         if acl in ('public-read', 'public-read-write'):
             ret['Grants'].append({
                 'Grantee': {
+                    'Type': 'Group',
                     'URI': 'http://acs.amazonaws.com/groups/global/AllUsers'
                 },
                 'Permission': 'READ'
@@ -215,6 +212,7 @@ def _acl_to_grant(ACL, owner_canonical_id):
         if acl == 'public-read-write':
             ret['Grants'].append({
                 'Grantee': {
+                    'Type': 'Group',
                     'URI': 'http://acs.amazonaws.com/groups/global/AllUsers'
                 },
                 'Permission': 'WRITE'
@@ -222,6 +220,7 @@ def _acl_to_grant(ACL, owner_canonical_id):
         if acl == 'aws-exec-read':
             ret['Grants'].append({
                 'Grantee': {
+                    'Type': 'CanonicalUser',
                     'DisplayName': 'za-team',
                     'ID': '6aa5a366c34c1cbe25dc49211496e913e0351eb0e8c37aa3477e40942ec6b97c'
                 },
@@ -230,6 +229,7 @@ def _acl_to_grant(ACL, owner_canonical_id):
         if acl == 'authenticated-read':
             ret['Grants'].append({
                 'Grantee': {
+                    'Type': 'Group',
                     'URI': 'http://acs.amazonaws.com/groups/global/AuthenticatedUsers'
                 },
                 'Permission': 'READ'
@@ -238,6 +238,7 @@ def _acl_to_grant(ACL, owner_canonical_id):
             for permission in ('WRITE', 'READ_ACP'):
                 ret['Grants'].append({
                     'Grantee': {
+                        'Type': 'Group',
                         'URI': 'http://acs.amazonaws.com/groups/s3/LogDelivery'
                     },
                     'Permission': permission
@@ -254,7 +255,8 @@ def _acl_to_grant(ACL, owner_canonical_id):
                 kind, val = item.split('=')
                 if kind == 'uri':
                     grantee = {
-                        'URI': val,
+                        'Type': 'Group',
+                        'URI': val
                     }
                 elif kind == 'id':
                     grantee = {
@@ -262,6 +264,7 @@ def _acl_to_grant(ACL, owner_canonical_id):
                         # match, and we will always update. Result is still
                         # idempotent
                         # 'DisplayName': ???,
+                        'Type': 'CanonicalUser',
                         'ID': val
                     }
                 else:
@@ -276,6 +279,12 @@ def _acl_to_grant(ACL, owner_canonical_id):
                     'Grantee': grantee,
                     'Permission': permission
                 })
+    # Boto only seems to list the default Grants when no other Grants are defined
+    if not ret['Grants']:
+        ret['Grants'] = [{
+                'Grantee': owner_canonical_grant,
+                'Permission': 'FULL_CONTROL'
+        }]
     return ret
 
 
@@ -294,7 +303,7 @@ def _get_role_arn(name, region=None, key=None, keyid=None, profile=None):
 
 
 def _compare_json(current, desired, region, key, keyid, profile):
-    return json_objs_equal(current, desired)
+    return __utils__['boto3.json_objs_equal'](current, desired)
 
 
 def _compare_acl(current, desired, region, key, keyid, profile):
@@ -305,24 +314,11 @@ def _compare_acl(current, desired, region, key, keyid, profile):
     rather than the input itself.
     '''
     ocid = _get_canonical_id(region, key, keyid, profile)
-    return json_objs_equal(_prep_acl_for_compare(current), _acl_to_grant(desired, ocid))
+    return __utils__['boto3.json_objs_equal'](current, _acl_to_grant(desired, ocid))
 
 
 def _compare_policy(current, desired, region, key, keyid, profile):
-    '''
-    Policy description is always returned as a JSON string. Comparison
-    should be object-to-object, since order is not significant in JSON
-    '''
-    if isinstance(desired, string_types):
-        desired = json.loads(desired)
-
-    if current is not None:
-        temp = current.get('Policy')
-        if isinstance(temp, string_types):
-            current = {'Policy': json.loads(temp)}
-        else:
-            current = None
-    return json_objs_equal(current, desired)
+    return current == desired
 
 
 def _compare_replication(current, desired, region, key, keyid, profile):
@@ -333,7 +329,7 @@ def _compare_replication(current, desired, region, key, keyid, profile):
         desired = deepcopy(desired)
         desired['Role'] = _get_role_arn(desired['Role'],
                                  region=region, key=key, keyid=keyid, profile=profile)
-    return json_objs_equal(current, desired)
+    return __utils__['boto3.json_objs_equal'](current, desired)
 
 
 def present(name, Bucket,
@@ -425,6 +421,10 @@ def present(name, Bucket,
         NotificationConfiguration = {}
     if RequestPayment is None:
         RequestPayment = {'Payer': 'BucketOwner'}
+    if Policy:
+        if isinstance(Policy, six.string_types):
+            Policy = json.loads(Policy)
+        Policy = __utils__['boto3.ordered'](Policy)
 
     r = __salt__['boto_s3_bucket.exists'](Bucket=Bucket,
            region=region, key=key, keyid=keyid, profile=profile)
@@ -479,7 +479,7 @@ def present(name, Bucket,
         return ret
 
     # bucket exists, ensure config matches
-    ret['comment'] = os.linesep.join([ret['comment'], 'S3 bucket {0} is present.'.format(Bucket)])
+    ret['comment'] = ' '.join([ret['comment'], 'S3 bucket {0} is present.'.format(Bucket)])
     ret['changes'] = {}
     _describe = __salt__['boto_s3_bucket.describe'](Bucket=Bucket,
                                  region=region, key=key, keyid=keyid, profile=profile)
@@ -543,42 +543,55 @@ def present(name, Bucket,
         config_items.append(replication_item)
         config_items.append(versioning_item)
 
+    update = False
     for varname, setter, current, comparator, desired, deleter in config_items:
+        if varname == 'Policy':
+            if current is not None:
+                temp = current.get('Policy')
+                # Policy description is always returned as a JSON string.
+                # Convert it to JSON now for ease of comparisons later.
+                if isinstance(temp, six.string_types):
+                    current = __utils__['boto3.ordered']({'Policy': json.loads(temp)})
         if not comparator(current, desired, region, key, keyid, profile):
-            # current state and desired state differ
-            if __opts__['test']:
-                msg = 'S3 bucket {0} set to be modified.'.format(Bucket)
-                ret['comment'] = msg
-                ret['result'] = None
-                return ret
-            ret['changes'].setdefault('new', {})[varname] = desired
+            update = True
+            if varname == 'ACL':
+                ret['changes'].setdefault('new', {})[varname] = _acl_to_grant(
+                        desired, _get_canonical_id(region, key, keyid, profile))
+            else:
+                ret['changes'].setdefault('new', {})[varname] = desired
             ret['changes'].setdefault('old', {})[varname] = current
 
-            if deleter and desired is None:
-                # Setting can be deleted, so use that to unset it
-                r = __salt__['boto_s3_bucket.{0}'.format(deleter)](Bucket=Bucket,
-                   region=region, key=key, keyid=keyid, profile=profile)
-                if not r.get('deleted'):
-                    ret['result'] = False
-                    ret['comment'] = 'Failed to update bucket: {0}.'.format(r['error']['message'])
-                    ret['changes'] = {}
-                    return ret
-            else:
-                r = __salt__['boto_s3_bucket.{0}'.format(setter)](Bucket=Bucket,
-                   region=region, key=key, keyid=keyid, profile=profile,
-                   **(desired or {}))
-                if not r.get('updated'):
-                    ret['result'] = False
-                    ret['comment'] = 'Failed to update bucket: {0}.'.format(r['error']['message'])
-                    ret['changes'] = {}
-                    return ret
+            if not __opts__['test']:
+                if deleter and desired is None:
+                    # Setting can be deleted, so use that to unset it
+                    r = __salt__['boto_s3_bucket.{0}'.format(deleter)](Bucket=Bucket,
+                      region=region, key=key, keyid=keyid, profile=profile)
+                    if not r.get('deleted'):
+                        ret['result'] = False
+                        ret['comment'] = 'Failed to update bucket: {0}.'.format(r['error']['message'])
+                        ret['changes'] = {}
+                        return ret
+                else:
+                    r = __salt__['boto_s3_bucket.{0}'.format(setter)](Bucket=Bucket,
+                      region=region, key=key, keyid=keyid, profile=profile,
+                      **(desired or {}))
+                    if not r.get('updated'):
+                        ret['result'] = False
+                        ret['comment'] = 'Failed to update bucket: {0}.'.format(r['error']['message'])
+                        ret['changes'] = {}
+                        return ret
+    if update and __opts__['test']:
+        msg = 'S3 bucket {0} set to be modified.'.format(Bucket)
+        ret['comment'] = msg
+        ret['result'] = None
+        return ret
 
     # Since location can't be changed, try that last so at least the rest of
     # the things are correct by the time we fail here. Fail so the user will
     # notice something mismatches their desired state.
     if _describe.get('Location', {}).get('LocationConstraint') != LocationConstraint:
         msg = 'Bucket {0} location does not match desired configuration, but cannot be changed'.format(LocationConstraint)
-        log.warn(msg)
+        log.warning(msg)
         ret['result'] = False
         ret['comment'] = 'Failed to update bucket: {0}.'.format(msg)
         return ret
@@ -586,7 +599,7 @@ def present(name, Bucket,
     return ret
 
 
-def absent(name, Bucket,
+def absent(name, Bucket, Force=False,
                   region=None, key=None, keyid=None, profile=None):
     '''
     Ensure bucket with passed properties is absent.
@@ -597,6 +610,9 @@ def absent(name, Bucket,
     Bucket
         Name of the bucket.
 
+    Force
+        Empty the bucket first if necessary - Boolean.
+
     region
         Region to connect to.
 
@@ -617,8 +633,8 @@ def absent(name, Bucket,
            'changes': {}
            }
 
-    r = __salt__['boto_s3_bucket.exists'](Bucket,
-                       region=region, key=key, keyid=keyid, profile=profile)
+    r = __salt__['boto_s3_bucket.exists'](Bucket, region=region, key=key,
+                                          keyid=keyid, profile=profile)
     if 'error' in r:
         ret['result'] = False
         ret['comment'] = 'Failed to delete bucket: {0}.'.format(r['error']['message'])
@@ -632,9 +648,8 @@ def absent(name, Bucket,
         ret['comment'] = 'S3 bucket {0} is set to be removed.'.format(Bucket)
         ret['result'] = None
         return ret
-    r = __salt__['boto_s3_bucket.delete'](Bucket,
-                                    region=region, key=key,
-                                    keyid=keyid, profile=profile)
+    r = __salt__['boto_s3_bucket.delete'](Bucket, Force=Force, region=region,
+                                          key=key, keyid=keyid, profile=profile)
     if not r['deleted']:
         ret['result'] = False
         ret['comment'] = 'Failed to delete bucket: {0}.'.format(r['error']['message'])
