@@ -4,7 +4,7 @@ Manage chassis via Salt Proxies.
 
 .. versionadded:: 2015.8.2
 
-Below is an example state that sets parameters just to show the basics.
+Below is an example state that sets basic parameters:
 
 .. code-block:: yaml
 
@@ -24,15 +24,17 @@ Below is an example state that sets parameters just to show the basics.
           - server-3: powercycle
 
 However, it is possible to place the entire set of chassis configuration
-data in pillar. Here's an example pillar
-structure:
+data in pillar. Here's an example pillar structure:
 
 .. code-block:: yaml
 
     proxy:
       host: 10.27.20.18
       admin_username: root
-      admin_password: saltstack
+      fallback_admin_username: root
+      passwords:
+        - super-secret
+        - old-secret
       proxytype: fx2
 
       chassis:
@@ -47,75 +49,50 @@ structure:
           - 'server-1': blade1
           - 'server-2': blade2
 
-        blades:
-           blade1:
-             idrac_password: saltstack1
-             ipmi_over_lan: True
-             ip: 172.17.17.1
-             subnet: 255.255.0.0
-             netmask: 172.17.255.255
-          blade2:
-             idrac_password: saltstack1
-             ipmi_over_lan: True
-             ip: 172.17.17.2
-             subnet: 255.255.0.0
-             netmask: 172.17.255.255
-          blade3:
-             idrac_password: saltstack1
-             ipmi_over_lan: True
-             ip: 172.17.17.2
-             subnet: 255.255.0.0
-             netmask: 172.17.255.255
-          blade4:
-             idrac_password: saltstack1
-             ipmi_over_lan: True
-             ip: 172.17.17.2
-             subnet: 255.255.0.0
-             netmask: 172.17.255.255
+        servers:
+          server-1:
+            idrac_password: saltstack1
+            ipmi_over_lan: True
+            ip: 172.17.17.132
+            netmask: 255.255.0.0
+            gateway: 172.17.17.1
+          server-2:
+            idrac_password: saltstack1
+            ipmi_over_lan: True
+            ip: 172.17.17.2
+            netmask: 255.255.0.0
+            gateway: 172.17.17.1
+          server-3:
+            idrac_password: saltstack1
+            ipmi_over_lan: True
+            ip: 172.17.17.20
+            netmask: 255.255.0.0
+            gateway: 172.17.17.1
+          server-4:
+            idrac_password: saltstack1
+            ipmi_over_lan: True
+            ip: 172.17.17.2
+            netmask: 255.255.0.0
+            gateway: 172.17.17.1
 
         switches:
           switch-1:
             ip: 192.168.1.2
             netmask: 255.255.255.0
-            broadcast: 192.168.1.255
+            gateway: 192.168.1.1
             snmp: nonpublic
             password: saltstack1
           switch-2:
             ip: 192.168.1.3
             netmask: 255.255.255.0
-            broadcast: 192.168.1.255
+            gateway: 192.168.1.1
             snmp: nonpublic
             password: saltstack1
 
-And to go with it, here's an example state that pulls the data from pillar.
-This example assumes that the pillar data would be structured like
+And to go with it, here's an example state that pulls the data from the
+pillar stated above:
 
-Pillar:
-
-.. code-block:: yaml
-
-    proxy:
-      host: 192.168.1.1
-      admin_username: root
-      admin_password: sekrit
-      fallback_admin_username: root
-      fallback_admin_password: old_sekrit
-      proxytype: fx2
-
-      chassis:
-        name: fx2-1
-        username: root
-        datacenter: UT1
-        location: UT1
-        management_mode: 2
-        idrac_launch: 0
-        slot_names:
-          1: blade1
-          2: blade2
-
-State:
-
-.. code-block:: yaml
+.. code-block:: jinja
 
     {% set details = pillar.get('proxy:chassis', {}) %}
     standup-step1:
@@ -124,15 +101,30 @@ State:
         - location: {{ details['location'] }}
         - mode: {{ details['management_mode'] }}
         - idrac_launch: {{ details['idrac_launch'] }}
-        - slot_names
-          {% for k, v in details['chassis']['slot_names'].iteritems() %}
-          - {{ k }}: {{ v }}
+        - slot_names:
+          {% for entry details['slot_names'] %}
+            - {{ entry.keys()[0] }}: {{ entry[entry.keys()[0]]  }}
           {% endfor %}
 
+    blade_powercycle:
+      dellchassis.chassis:
+        - blade_power_states:
+          - server-1: powercycle
+          - server-2: powercycle
+          - server-3: powercycle
+          - server-4: powercycle
 
-    {% for k, v in details['chassis']['switches'].iteritems() %}
-    standup-switches-{{ k }}:
-      dellchassis.dell_switch:
+    # Set idrac_passwords for blades.  racadm needs them to be called 'server-x'
+    {% for k, v in details['servers'].iteritems() %}
+    {{ k }}:
+      dellchassis.blade_idrac:
+        - idrac_password: {{ v['idrac_password'] }}
+    {% endfor %}
+
+    # Set management ip addresses, passwords, and snmp strings for switches
+    {% for k, v in details['switches'].iteritems() %}
+    {{ k }}-switch-setup:
+      dellchassis.switch:
         - name: {{ k }}
         - ip: {{ v['ip'] }}
         - netmask: {{ v['netmask'] }}
@@ -141,26 +133,37 @@ State:
         - snmp: {{ v['snmp'] }}
     {% endfor %}
 
-    dellchassis
-    {% for k, v in details['chassis']['slot_names'].iteritems() %}
+.. note::
 
-          - {{ k }}: {{ v }}
-          {% endfor %}
+    This state module relies on the dracr.py execution module, which runs racadm commands on
+    the chassis, blades, etc. The racadm command runs very slowly and, depending on your state,
+    the proxy minion return might timeout before the racadm commands have completed. If you
+    are repeatedly seeing minions timeout after state calls, please use the ``-t`` CLI argument
+    to increase the timeout variable.
 
-    blade_powercycle:
-      chassis.dell_chassis:
-        - blade_power_states:
-          - server-1: powercycle
-          - server-2: powercycle
-          - server-3: powercycle
-          - server-4: powercycle
+    For example:
+
+    .. code-block:: bash
+
+        salt '*' state.sls my-dell-chasis-state-name -t 60
+
+.. note::
+
+    The Dell CMC units perform adequately but many iDRACs are **excruciatingly**
+    slow.  Some functions can take minutes to execute.
 
 '''
 
 # Import python libs
 from __future__ import absolute_import
 import logging
+import os
 
+# Import Salt lobs
+import salt.ext.six as six
+from salt.exceptions import CommandExecutionError
+
+# Get logging started
 log = logging.getLogger(__name__)
 
 
@@ -168,54 +171,149 @@ def __virtual__():
     return 'chassis.cmd' in __salt__
 
 
-def blade_idrac(idrac_password=None, idrac_ipmi=None,
+def blade_idrac(name, idrac_password=None, idrac_ipmi=None,
                 idrac_ip=None, idrac_netmask=None, idrac_gateway=None,
                 idrac_dnsname=None,
-                drac_dhcp=None):
+                idrac_dhcp=None):
     '''
     Set parameters for iDRAC in a blade.
 
-    :param idrac_password: Password to establish for the iDRAC interface
+    :param idrac_password: Password to use to connect to the iDRACs directly
+        (idrac_ipmi and idrac_dnsname must be set directly on the iDRAC.  They
+        can't be set through the CMC.  If this password is present, use it
+        instead of the CMC password)
     :param idrac_ipmi: Enable/Disable IPMI over LAN
     :param idrac_ip: Set IP address for iDRAC
     :param idrac_netmask: Set netmask for iDRAC
     :param idrac_gateway: Set gateway for iDRAC
-    :param drac_dhcp: Turn on DHCP for iDRAC (True turns on, False does nothing
-      becaause setting a static IP will disable DHCP).
+    :param idrac_dhcp: Turn on DHCP for iDRAC (True turns on, False does
+        nothing becaause setting a static IP will disable DHCP).
+
     :return: A standard Salt changes dictionary
+
+    NOTE: If any of the IP address settings is configured, all of ip, netmask,
+    and gateway must be present
     '''
 
-    ret = {'result': True,
+    ret = {'name': name,
+           'result': True,
            'changes': {},
            'comment': ''}
 
     if not idrac_password:
-        password = __pillar__['proxy']['admin_password']
+        (username, password) = __salt__['chassis.chassis_credentials']()
     else:
         password = idrac_password
 
-    if idrac_ipmi:
-        if idrac_ipmi is True:
+    module_network = __salt__['chassis.cmd']('network_info', module=name)
+    current_idrac_ip = module_network['Network']['IP Address']
+
+    if idrac_ipmi is not None:
+        if idrac_ipmi is True or idrac_ipmi == 1:
             idrac_ipmi = '1'
-        if idrac_ipmi is False:
+        if idrac_ipmi is False or idrac_ipmi == 0:
             idrac_ipmi = '0'
         current_ipmi = __salt__['dracr.get_general']('cfgIpmiLan', 'cfgIpmiLanEnable',
-                                                     host=idrac_ip, admin_username='root',
+                                                     host=current_idrac_ip, admin_username='root',
                                                      admin_password=password)
 
         if current_ipmi != idrac_ipmi:
             ch = {'Old': current_ipmi, 'New': idrac_ipmi}
             ret['changes']['IPMI'] = ch
 
-    if idrac_dnsname:
-        dnsret = __salt__['dracr.get_dns_dracname'](host=idrac_ip, admin_username='root',
+    if idrac_dnsname is not None:
+        dnsret = __salt__['dracr.get_dns_dracname'](host=current_idrac_ip,
+                                                    admin_username='root',
                                                     admin_password=password)
-        current_dnsname = dnsret['Key=iDRAC.Embedded.1#NIC.1']['DNSRacName']
+        current_dnsname = dnsret['[Key=iDRAC.Embedded.1#NIC.1]']['DNSRacName']
         if current_dnsname != idrac_dnsname:
             ch = {'Old': current_dnsname,
                   'New': idrac_dnsname}
             ret['changes']['DNSRacName'] = ch
 
+    if idrac_dhcp is not None or idrac_ip or idrac_netmask or idrac_gateway:
+        if idrac_dhcp is True or idrac_dhcp == 1:
+            idrac_dhcp = 1
+        else:
+            idrac_dhcp = 0
+        if str(module_network['Network']['DHCP Enabled']) == '0' and idrac_dhcp == 1:
+            ch = {'Old': module_network['Network']['DHCP Enabled'],
+                  'New': idrac_dhcp}
+            ret['changes']['DRAC DHCP'] = ch
+
+        if idrac_dhcp == 0 and all([idrac_ip, idrac_netmask, idrac_netmask]):
+            current_network = __salt__['chassis.cmd']('network_info',
+                                                      module=name)
+            old_ipv4 = {}
+            new_ipv4 = {}
+            if current_network['Network']['IP Address'] != idrac_ip:
+                old_ipv4['ip'] = current_network['Network']['IP Address']
+                new_ipv4['ip'] = idrac_ip
+            if current_network['Network']['Subnet Mask'] != idrac_netmask:
+                old_ipv4['netmask'] = current_network['Network']['Subnet Mask']
+                new_ipv4['netmask'] = idrac_netmask
+            if current_network['Network']['Gateway'] != idrac_gateway:
+                old_ipv4['gateway'] = current_network['Network']['Gateway']
+                new_ipv4['gateway'] = idrac_gateway
+
+            if new_ipv4 != {}:
+                ret['changes']['Network'] = {}
+                ret['changes']['Network']['Old'] = old_ipv4
+                ret['changes']['Network']['New'] = new_ipv4
+
+    if ret['changes'] == {}:
+        ret['comment'] = 'iDRAC on blade is already in the desired state.'
+        return ret
+
+    if __opts__['test'] and ret['changes'] != {}:
+        ret['result'] = None
+        ret['comment'] = 'iDRAC on blade will change.'
+        return ret
+
+    if 'IPMI' in ret['changes']:
+        ipmi_result = __salt__['dracr.set_general']('cfgIpmiLan',
+                                                    'cfgIpmiLanEnable',
+                                                    idrac_ipmi,
+                                                    host=current_idrac_ip,
+                                                    admin_username='root',
+                                                    admin_password=password)
+        if not ipmi_result:
+            ret['result'] = False
+            ret['changes']['IPMI']['success'] = False
+
+    if 'DNSRacName' in ret['changes']:
+        dnsracname_result = __salt__['dracr.set_dns_dracname'](idrac_dnsname,
+            host=current_idrac_ip,
+            admin_username='root',
+            admin_password=password)
+        if dnsracname_result['retcode'] == 0:
+            ret['changes']['DNSRacName']['success'] = True
+        else:
+            ret['result'] = False
+            ret['changes']['DNSRacName']['success'] = False
+            ret['changes']['DNSRacName']['return'] = dnsracname_result
+
+    if 'DRAC DHCP' in ret['changes']:
+        dhcp_result = __salt__['chassis.cmd']('set_niccfg', dhcp=idrac_dhcp)
+        if dhcp_result['retcode']:
+            ret['changes']['DRAC DHCP']['success'] = True
+        else:
+            ret['result'] = False
+            ret['changes']['DRAC DHCP']['success'] = False
+            ret['changes']['DRAC DHCP']['return'] = dhcp_result
+
+    if 'Network' in ret['changes']:
+        network_result = __salt__['chassis.cmd']('set_niccfg', ip=idrac_ip,
+                                                 netmask=idrac_netmask,
+                                                 gateway=idrac_gateway,
+                                                 module=name)
+        if network_result['retcode'] == 0:
+            ret['changes']['Network']['success'] = True
+        else:
+            ret['result'] = False
+            ret['changes']['Network']['success'] = False
+            ret['changes']['Network']['return'] = network_result
+
     return ret
 
 
@@ -235,7 +333,11 @@ def chassis(name, chassis_name=None, password=None, datacenter=None,
         The location of the chassis.
 
     password
-        Password for the chassis
+        Password for the chassis. Note: If this password is set for the chassis,
+        the current implementation of this state will set this password both on
+        the chassis and the iDrac passwords on any configured blades. If the
+        password for the blades should be distinct, they should be set separately
+        with the blade_idrac function.
 
     mode
         The management mode of the chassis. Viable options are:
@@ -268,7 +370,7 @@ def chassis(name, chassis_name=None, password=None, datacenter=None,
 
         my-dell-chassis:
           dellchassis.chassis:
-            - name: my-dell-chassis
+            - chassis_name: my-dell-chassis
             - location: my-location
             - datacenter: london
             - mode: 2
@@ -281,7 +383,8 @@ def chassis(name, chassis_name=None, password=None, datacenter=None,
               - server-2: off
               - server-3: powercycle
     '''
-    ret = {'chassis_name': chassis_name,
+    ret = {'name': chassis_name,
+           'chassis_name': chassis_name,
            'result': True,
            'changes': {},
            'comment': ''}
@@ -369,7 +472,7 @@ def chassis(name, chassis_name=None, password=None, datacenter=None,
                     target_power_states[key] = 'powerup'
                 if current_power_states[key] != -1 and current_power_states[key]:
                     target_power_states[key] = 'powercycle'
-        for k, v in target_power_states.iteritems():
+        for k, v in six.iteritems(target_power_states):
             old = {k: current_power_states[k]}
             new = {k: v}
             if ret['changes'].get('Blade Power States') is None:
@@ -389,7 +492,7 @@ def chassis(name, chassis_name=None, password=None, datacenter=None,
         return ret
 
     # Finally, set the necessary configurations on the chassis.
-    name = __salt__[chassis_cmd]('set_chassis_name', name)
+    name = __salt__[chassis_cmd]('set_chassis_name', chassis_name)
     if location:
         location = __salt__[chassis_cmd]('set_chassis_location', location)
     pw_result = True
@@ -397,7 +500,7 @@ def chassis(name, chassis_name=None, password=None, datacenter=None,
         pw_single = True
         if __salt__[chassis_cmd]('change_password', username='root', uid=1,
                                    password=password):
-            for blade in inventory['server'].keys():
+            for blade in inventory['server']:
                 pw_single = __salt__[chassis_cmd]('deploy_password',
                                                   username='root',
                                                   password=password,
@@ -429,7 +532,7 @@ def chassis(name, chassis_name=None, password=None, datacenter=None,
             slot_names = True
 
     powerchange_all_ok = True
-    for k, v in target_power_states.iteritems():
+    for k, v in six.iteritems(target_power_states):
         powerchange_ok = __salt__[chassis_cmd]('server_power', v, module=k)
         if not powerchange_ok:
             powerchange_all_ok = False
@@ -476,7 +579,7 @@ def switch(name, ip=None, netmask=None, gateway=None, dhcp=None,
     .. code-block:: yaml
 
         my-dell-chassis:
-          dellchassis.dell_switch:
+          dellchassis.switch:
             - switch: switch-1
             - ip: 192.168.1.1
             - netmask: 255.255.255.0
@@ -492,43 +595,48 @@ def switch(name, ip=None, netmask=None, gateway=None, dhcp=None,
            'comment': ''}
 
     current_nic = __salt__['chassis.cmd']('network_info', module=name)
-    if current_nic.get('retcode', 0) != 0:
-        ret['result'] = False
-        ret['comment'] = current_nic['stdout']
-        return ret
+    try:
+        if current_nic.get('retcode', 0) != 0:
+            ret['result'] = False
+            ret['comment'] = current_nic['stdout']
+            return ret
 
-    if ip or netmask or gateway:
-        if not ip:
-            ip = current_nic['Network']['IP Address']
-        if not netmask:
-            ip = current_nic['Network']['Subnet Mask']
-        if not gateway:
-            ip = current_nic['Network']['Gateway']
+        if ip or netmask or gateway:
+            if not ip:
+                ip = current_nic['Network']['IP Address']
+            if not netmask:
+                ip = current_nic['Network']['Subnet Mask']
+            if not gateway:
+                ip = current_nic['Network']['Gateway']
 
-    if current_nic['Network']['DHCP Enabled'] == '0' and dhcp:
-        ret['changes'].update({'DHCP': {'Old': {'DHCP Enabled': current_nic['Network']['DHCP Enabled']},
-                                        'New': {'DHCP Enabled': dhcp}}})
+        if current_nic['Network']['DHCP Enabled'] == '0' and dhcp:
+            ret['changes'].update({'DHCP': {'Old': {'DHCP Enabled': current_nic['Network']['DHCP Enabled']},
+                                            'New': {'DHCP Enabled': dhcp}}})
 
-    if ((ip or netmask or gateway) and not dhcp and (ip != current_nic['Network']['IP Address'] or
-                                                             netmask != current_nic['Network']['Subnet Mask'] or
-                                                             gateway != current_nic['Network']['Gateway'])):
-        ret['changes'].update({'IP': {'Old': current_nic['Network'],
-                                      'New': {'IP Address': ip,
-                                              'Subnet Mask': netmask,
-                                              'Gateway': gateway}}})
+        if ((ip or netmask or gateway) and not dhcp and (ip != current_nic['Network']['IP Address'] or
+                                                                 netmask != current_nic['Network']['Subnet Mask'] or
+                                                                 gateway != current_nic['Network']['Gateway'])):
+            ret['changes'].update({'IP': {'Old': current_nic['Network'],
+                                          'New': {'IP Address': ip,
+                                                  'Subnet Mask': netmask,
+                                                  'Gateway': gateway}}})
 
-    if password:
-        if 'New' not in ret['changes']:
-            ret['changes']['New'] = {}
-        ret['changes']['New'].update({'Password': '*****'})
+        if password:
+            if 'New' not in ret['changes']:
+                ret['changes']['New'] = {}
+            ret['changes']['New'].update({'Password': '*****'})
 
-    if snmp:
-        if 'New' not in ret['changes']:
-            ret['changes']['New'] = {}
-        ret['changes']['New'].update({'SNMP': '*****'})
+        if snmp:
+            if 'New' not in ret['changes']:
+                ret['changes']['New'] = {}
+            ret['changes']['New'].update({'SNMP': '*****'})
 
-    if ret['changes'] == {}:
-        ret['comment'] = 'Switch ' + name + ' is already in desired state'
+        if ret['changes'] == {}:
+            ret['comment'] = 'Switch ' + name + ' is already in desired state'
+            return ret
+    except AttributeError:
+        ret['changes'] = {}
+        ret['comment'] = 'Something went wrong retrieving the switch details'
         return ret
 
     if __opts__['test']:
@@ -546,7 +654,7 @@ def switch(name, ip=None, netmask=None, gateway=None, dhcp=None,
         password_ret = __salt__['chassis.cmd']('deploy_password', 'root', password, module=name)
 
     if snmp:
-        snmp_ret = __salt__['chassis.cmd']('deploy_snmp', password, module=name)
+        snmp_ret = __salt__['chassis.cmd']('deploy_snmp', snmp, module=name)
 
     if any([password_ret, snmp_ret, net_ret, dhcp_ret]) is False:
         ret['result'] = False
@@ -554,3 +662,70 @@ def switch(name, ip=None, netmask=None, gateway=None, dhcp=None,
 
     ret['comment'] = 'Dell chassis switch {0} was updated.'.format(name)
     return ret
+
+
+def _firmware_update(firmwarefile='', host='',
+                     directory=''):
+    '''
+    Update firmware for a single host
+    '''
+    dest = os.path.join(directory, firmwarefile[7:])
+
+    __salt__['cp.get_file'](firmwarefile, dest)
+
+    username = __pillar__['proxy']['admin_user']
+    password = __pillar__['proxy']['admin_password']
+    __salt__['dracr.update_firmware'](dest,
+                                      host=host,
+                                      admin_username=username,
+                                      admin_password=password)
+
+
+def firmware_update(hosts=None, directory=''):
+    '''
+        State to update the firmware on host
+        using the ``racadm`` command
+
+        firmwarefile
+            filename (string) starting with ``salt://``
+        host
+            string representing the hostname
+            supplied to the ``racadm`` command
+        directory
+            Directory name where firmwarefile
+            will be downloaded
+
+    .. code-block:: yaml
+
+        dell-chassis-firmware-update:
+          dellchassis.firmware_update:
+            hosts:
+              cmc:
+                salt://firmware_cmc.exe
+              server-1:
+                salt://firmware.exe
+            directory: /opt/firmwares
+    '''
+    ret = {}
+    ret.changes = {}
+    success = True
+    for host, firmwarefile in hosts:
+        try:
+            _firmware_update(firmwarefile, host, directory)
+            ret['changes'].update({
+                'host': {
+                    'comment': 'Firmware update submitted for {0}'.format(host),
+                    'success': True,
+                }
+            })
+        except CommandExecutionError as err:
+            success = False
+            ret['changes'].update({
+                'host': {
+                    'comment': 'FAILED to update firmware for {0}'.format(host),
+                    'success': False,
+                    'reason': str(err),
+                }
+            })
+    ret['result'] = success
+    return ret
