@@ -1235,9 +1235,10 @@ def running(name,
         If ``True``, runs the exec process with extended privileges
 
         .. code-block:: yaml
+
             foo:
               docker_container.running:
-                - image: bar/baz:lates
+                - image: bar/baz:latest
                 - privileged: True
 
     publish_all_ports (or *publish_all*) : False
@@ -1479,7 +1480,7 @@ def running(name,
                 - image: bar/baz:latest
                 - userns_mode: host
 
-    volumes (or *volume)
+    volumes (or *volume*)
         List of directories to expose as volumes. Can be expressed as a
         comma-separated list or a YAML list. The below two examples are
         equivalent:
