@@ -5,6 +5,8 @@ Send a message to Hipchat
 
 This state is useful for sending messages to Hipchat during state runs.
 
+The property api_url is optional. By defaul will use the public HipChat API at https://api.hipchat.com
+
 .. versionadded:: 2015.5.0
 
 .. code-block:: yaml
@@ -14,10 +16,12 @@ This state is useful for sending messages to Hipchat during state runs.
         - room_id: 123456
         - from_name: SuperAdmin
         - message: 'This state was executed successfully.'
+        - api_url: https://hipchat.myteam.com
         - api_key: peWcBiMOS9HrZG15peWcBiMOS9HrZG15
         - api_version: v1
 
 The api key can be specified in the master or minion configuration like below:
+
 .. code-block:: yaml
 
     hipchat:
@@ -38,6 +42,7 @@ def send_message(name,
                  room_id,
                  from_name,
                  message,
+                 api_url=None,
                  api_key=None,
                  api_version=None,
                  message_color='yellow',
@@ -52,9 +57,10 @@ def send_message(name,
             - room_id: 123456
             - from_name: SuperAdmin
             - message: 'This state was executed successfully.'
+            - api_url: https://hipchat.myteam.com
             - api_key: peWcBiMOS9HrZG15peWcBiMOS9HrZG15
             - api_version: v1
-            - color: green
+            - message_color: green
             - notify: True
 
     The following parameters are required:
@@ -73,6 +79,12 @@ def send_message(name,
         The message that is to be sent to the Hipchat room.
 
     The following parameters are optional:
+
+    api_url
+        The API URl to be used.
+        If not specified here or in the configuration options of master or minion,
+        will use the public HipChat API: https://api.hipchat.com
+
     api_key
         The api key for Hipchat to use for authentication,
         if not specified in the configuration options of master or minion.
@@ -81,7 +93,7 @@ def send_message(name,
         The api version for Hipchat to use,
         if not specified in the configuration options of master or minion.
 
-    color
+    message_color
         The color the Hipchat message should be displayed in. One of the following, default: yellow
         "yellow", "red", "green", "purple", "gray", or "random".
 
@@ -114,6 +126,7 @@ def send_message(name,
         room_id=room_id,
         message=message,
         from_name=from_name,
+        api_url=api_url,
         api_key=api_key,
         api_version=api_version,
         color=message_color,
