@@ -5,7 +5,7 @@ Management of iptables
 
 This is an iptables-specific module designed to manage Linux firewalls. It is
 expected that this state module, and other system-specific firewall states, may
-at some point be deprecated in favor of a more generic `firewall` state.
+at some point be deprecated in favor of a more generic ``firewall`` state.
 
 .. code-block:: yaml
 
@@ -17,7 +17,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - match: state
         - connstate: NEW
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -32,7 +32,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - comment: "Allow HTTP"
         - connstate: NEW
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -48,7 +48,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - connstate: NEW
         - source: '127.0.0.1'
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -65,7 +65,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - connstate: NEW
         - source: '! 127.0.0.1'
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -81,7 +81,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - connstate: NEW
         - source: 'not 127.0.0.1'
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -94,7 +94,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - match: state
         - connstate: NEW
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -109,7 +109,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - dports:
             - 80
             - 443
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -122,7 +122,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - match: state
         - connstate: NEW
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -136,7 +136,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - match: state
         - connstate: NEW
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -148,7 +148,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - match: state
         - connstate: NEW
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -161,7 +161,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - match: state
         - connstate: NEW
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -174,7 +174,7 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - match: state
         - connstate: NEW
         - dport: 80
-        - proto: tcp
+        - protocol: tcp
         - sport: 1025:65535
         - save: True
 
@@ -183,6 +183,55 @@ at some point be deprecated in favor of a more generic `firewall` state.
         - chain: INPUT
         - policy: ACCEPT
 
+.. note::
+
+    Whereas iptables will accept ``-p``, ``--proto[c[o[l]]]`` as synonyms of
+    ``--protocol``, if ``--proto`` appears in an iptables command after the
+    appearance of ``-m policy``, it is interpreted as the ``--proto`` option of
+    the policy extension (see the iptables-extensions(8) man page).
+
+Example rules for IPSec policy:
+
+.. code-block:: yaml
+
+    accept_esp_in:
+      iptables.append:
+        - table: filter
+        - chain: INPUT
+        - jump: ACCEPT
+        - source: 10.20.0.0/24
+        - destination: 10.10.0.0/24
+        - in-interface: eth0
+        - match: policy
+        - dir: in
+        - pol: ipsec
+        - reqid: 1
+        - proto: esp
+    accept_esp_forward_in:
+      iptables.append:
+        - use:
+          - iptables: accept_esp_in
+        - chain: FORWARD
+
+    accept_esp_out:
+      iptables.append:
+        - table: filter
+        - chain: OUTPUT
+        - jump: ACCEPT
+        - source: 10.10.0.0/24
+        - destination: 10.20.0.0/24
+        - out-interface: eth0
+        - match: policy
+        - dir: out
+        - pol: ipsec
+        - reqid: 1
+        - proto: esp
+    accept_esp_forward_out:
+      iptables.append:
+        - use:
+          - iptables: accept_esp_out
+        - chain: FORWARD
+
 .. note::
 
     Various functions of the ``iptables`` module use the ``--check`` option. If
@@ -263,6 +312,9 @@ def chain_absent(name, table='filter', family='ipv4'):
 
     Verify the chain is absent.
 
+    table
+        The table to remove the chain from
+
     family
         Networking family, either ipv4 or ipv6
     '''
@@ -307,16 +359,19 @@ def chain_absent(name, table='filter', family='ipv4'):
     return ret
 
 
-def append(name, family='ipv4', **kwargs):
+def append(name, table='filter', family='ipv4', **kwargs):
     '''
     .. versionadded:: 0.17.0
 
-    Append a rule to a chain
+    Add a rule to the end of the specified chain.
 
     name
         A user-defined name to call this rule by in another part of a state or
         formula. This should not be an actual rule.
 
+    table
+        The table that owns the chain which should be modified
+
     family
         Network family, ipv4 or ipv6.
 
@@ -324,6 +379,9 @@ def append(name, family='ipv4', **kwargs):
     that would normally be used for iptables, with one exception: ``--state`` is
     specified as `connstate` instead of `state` (not to be confused with
     `ctstate`).
+
+    Jump options that doesn't take arguments should be passed in with an empty
+    string.
     '''
     ret = {'name': name,
            'changes': {},
@@ -340,6 +398,7 @@ def append(name, family='ipv4', **kwargs):
             if '__agg__' in rule:
                 del rule['__agg__']
             if 'save' in rule and rule['save']:
+                save = True
                 if rule['save'] is not True:
                     save_file = rule['save']
                 else:
@@ -363,9 +422,10 @@ def append(name, family='ipv4', **kwargs):
         if ignore in kwargs:
             del kwargs[ignore]
     kwargs['name'] = name
+    kwargs['table'] = table
     rule = __salt__['iptables.build_rule'](family=family, **kwargs)
     command = __salt__['iptables.build_rule'](full='True', family=family, command='A', **kwargs)
-    if __salt__['iptables.check'](kwargs['table'],
+    if __salt__['iptables.check'](table,
                                   kwargs['chain'],
                                   rule,
                                   family) is True:
@@ -401,7 +461,7 @@ def append(name, family='ipv4', **kwargs):
             command.strip(),
             family)
         return ret
-    if __salt__['iptables.append'](kwargs['table'], kwargs['chain'], rule, family):
+    if __salt__['iptables.append'](table, kwargs['chain'], rule, family):
         ret['changes'] = {'locale': name}
         ret['result'] = True
         ret['comment'] = 'Set iptables rule for {0} to: {1} for {2}'.format(
@@ -427,7 +487,7 @@ def append(name, family='ipv4', **kwargs):
         return ret
 
 
-def insert(name, family='ipv4', **kwargs):
+def insert(name, table='filter', family='ipv4', **kwargs):
     '''
     .. versionadded:: 2014.1.0
 
@@ -437,13 +497,23 @@ def insert(name, family='ipv4', **kwargs):
         A user-defined name to call this rule by in another part of a state or
         formula. This should not be an actual rule.
 
+    table
+        The table that owns the chain that should be modified
+
     family
         Networking family, either ipv4 or ipv6
 
+    position
+        The numerical representation of where the rule should be inserted into
+        the chain. Note that ``-1`` is not a supported position value.
+
     All other arguments are passed in with the same name as the long option
     that would normally be used for iptables, with one exception: ``--state`` is
     specified as `connstate` instead of `state` (not to be confused with
     `ctstate`).
+
+    Jump options that doesn't take arguments should be passed in with an empty
+    string.
     '''
     ret = {'name': name,
            'changes': {},
@@ -460,6 +530,7 @@ def insert(name, family='ipv4', **kwargs):
             if '__agg__' in rule:
                 del rule['__agg__']
             if 'save' in rule and rule['save']:
+                save = True
                 if rule['save'] is not True:
                     save_file = rule['save']
                 else:
@@ -483,9 +554,10 @@ def insert(name, family='ipv4', **kwargs):
         if ignore in kwargs:
             del kwargs[ignore]
     kwargs['name'] = name
+    kwargs['table'] = table
     rule = __salt__['iptables.build_rule'](family=family, **kwargs)
     command = __salt__['iptables.build_rule'](full=True, family=family, command='I', **kwargs)
-    if __salt__['iptables.check'](kwargs['table'],
+    if __salt__['iptables.check'](table,
                                   kwargs['chain'],
                                   rule,
                                   family) is True:
@@ -521,7 +593,7 @@ def insert(name, family='ipv4', **kwargs):
             family,
             command.strip())
         return ret
-    if not __salt__['iptables.insert'](kwargs['table'], kwargs['chain'], kwargs['position'], rule, family):
+    if not __salt__['iptables.insert'](table, kwargs['chain'], kwargs['position'], rule, family):
         ret['changes'] = {'locale': name}
         ret['result'] = True
         ret['comment'] = 'Set iptables rule for {0} to: {1} for {2}'.format(
@@ -543,7 +615,7 @@ def insert(name, family='ipv4', **kwargs):
         return ret
 
 
-def delete(name, family='ipv4', **kwargs):
+def delete(name, table='filter', family='ipv4', **kwargs):
     '''
     .. versionadded:: 2014.1.0
 
@@ -553,6 +625,9 @@ def delete(name, family='ipv4', **kwargs):
         A user-defined name to call this rule by in another part of a state or
         formula. This should not be an actual rule.
 
+    table
+        The table that owns the chain that should be modified
+
     family
         Networking family, either ipv4 or ipv6
 
@@ -560,6 +635,9 @@ def delete(name, family='ipv4', **kwargs):
     that would normally be used for iptables, with one exception: ``--state`` is
     specified as `connstate` instead of `state` (not to be confused with
     `ctstate`).
+
+    Jump options that doesn't take arguments should be passed in with an empty
+    string.
     '''
     ret = {'name': name,
            'changes': {},
@@ -599,10 +677,11 @@ def delete(name, family='ipv4', **kwargs):
         if ignore in kwargs:
             del kwargs[ignore]
     kwargs['name'] = name
+    kwargs['table'] = table
     rule = __salt__['iptables.build_rule'](family=family, **kwargs)
     command = __salt__['iptables.build_rule'](full=True, family=family, command='D', **kwargs)
 
-    if not __salt__['iptables.check'](kwargs['table'],
+    if not __salt__['iptables.check'](table,
                                       kwargs['chain'],
                                       rule,
                                       family) is True:
@@ -622,13 +701,13 @@ def delete(name, family='ipv4', **kwargs):
 
     if 'position' in kwargs:
         result = __salt__['iptables.delete'](
-                kwargs['table'],
+                table,
                 kwargs['chain'],
                 family=family,
                 position=kwargs['position'])
     else:
         result = __salt__['iptables.delete'](
-                kwargs['table'],
+                table,
                 kwargs['chain'],
                 family=family,
                 rule=rule)
@@ -654,12 +733,15 @@ def delete(name, family='ipv4', **kwargs):
         return ret
 
 
-def set_policy(name, family='ipv4', **kwargs):
+def set_policy(name, table='filter', family='ipv4', **kwargs):
     '''
     .. versionadded:: 2014.1.0
 
     Sets the default policy for iptables firewall tables
 
+    table
+        The table that owns the chain that should be modified
+
     family
         Networking family, either ipv4 or ipv6
 
@@ -677,23 +759,23 @@ def set_policy(name, family='ipv4', **kwargs):
             del kwargs[ignore]
 
     if __salt__['iptables.get_policy'](
-            kwargs['table'],
+            table,
             kwargs['chain'],
             family) == kwargs['policy']:
         ret['result'] = True
         ret['comment'] = ('iptables default policy for chain {0} on table {1} for {2} already set to {3}'
-                          .format(kwargs['chain'], kwargs['table'], family, kwargs['policy']))
+                          .format(kwargs['chain'], table, family, kwargs['policy']))
         return ret
     if __opts__['test']:
         ret['comment'] = 'iptables default policy for chain {0} on table {1} for {2} needs to be set to {3}'.format(
             kwargs['chain'],
-            kwargs['table'],
+            table,
             family,
             kwargs['policy']
         )
         return ret
     if not __salt__['iptables.set_policy'](
-            kwargs['table'],
+            table,
             kwargs['chain'],
             kwargs['policy'],
             family):
@@ -719,12 +801,15 @@ def set_policy(name, family='ipv4', **kwargs):
         return ret
 
 
-def flush(name, family='ipv4', **kwargs):
+def flush(name, table='filter', family='ipv4', **kwargs):
     '''
     .. versionadded:: 2014.1.0
 
     Flush current iptables state
 
+    table
+        The table that owns the chain that should be modified
+
     family
         Networking family, either ipv4 or ipv6
 
@@ -738,22 +823,19 @@ def flush(name, family='ipv4', **kwargs):
         if ignore in kwargs:
             del kwargs[ignore]
 
-    if 'table' not in kwargs:
-        kwargs['table'] = 'filter'
-
     if 'chain' not in kwargs:
         kwargs['chain'] = ''
     if __opts__['test']:
         ret['comment'] = 'iptables rules in {0} table {1} chain {2} family needs to be flushed'.format(
             name,
-            kwargs['table'],
+            table,
             family)
         return ret
-    if not __salt__['iptables.flush'](kwargs['table'], kwargs['chain'], family):
+    if not __salt__['iptables.flush'](table, kwargs['chain'], family):
         ret['changes'] = {'locale': name}
         ret['result'] = True
         ret['comment'] = 'Flush iptables rules in {0} table {1} chain {2} family'.format(
-            kwargs['table'],
+            table,
             kwargs['chain'],
             family
         )
