@@ -63,8 +63,14 @@ from __future__ import absolute_import
 import time
 
 # Import Salt libs
-from salt.exceptions import CommandExecutionError
 import salt.utils
+from salt.utils.args import get_function_argspec as _argspec
+from salt.exceptions import CommandExecutionError
+
+# Import 3rd-party libs
+import salt.ext.six as six
+
+SYSTEMD_ONLY = ('no_block', 'unmask', 'unmask_runtime')
 
 __virtualname__ = 'service'
 
@@ -83,6 +89,24 @@ def __virtual__():
                )
 
 
+# Double-asterisk deliberately not used here
+def _get_systemd_only(func, kwargs):
+    ret = {}
+    warnings = []
+    valid_args = _argspec(func).args
+    for systemd_arg in SYSTEMD_ONLY:
+        arg_val = kwargs.get(systemd_arg, False)
+        if arg_val:
+            if systemd_arg in valid_args:
+                ret[systemd_arg] = arg_val
+            else:
+                warnings.append(
+                    'The \'{0}\' argument is not supported by this '
+                    'platform/action'.format(systemd_arg)
+                )
+    return ret, warnings
+
+
 def _enabled_used_error(ret):
     '''
     Warn of potential typo.
@@ -150,25 +174,32 @@ def _enable(name, started, result=True, **kwargs):
         ret['comment'] = 'Service {0} set to be enabled'.format(name)
         return ret
 
-    if __salt__['service.enable'](name, **kwargs):
-        # Service has been enabled
-        ret['changes'] = {}
-        after_toggle_enable_status = __salt__['service.enabled'](name, **kwargs)
-        # on upstart, certain services like apparmor will always return
-        # False, even if correctly activated
-        # do not trigger a change
-        if before_toggle_enable_status != after_toggle_enable_status:
-            ret['changes'][name] = True
-        if started is True:
-            ret['comment'] = ('Service {0} has been enabled,'
-                              ' and is running').format(name)
-        elif started is None:
-            ret['comment'] = ('Service {0} has been enabled,'
-                              ' and is in the desired state').format(name)
-        else:
-            ret['comment'] = ('Service {0} has been enabled,'
-                              ' and is dead').format(name)
-        return ret
+    try:
+        if __salt__['service.enable'](name, **kwargs):
+            # Service has been enabled
+            ret['changes'] = {}
+            after_toggle_enable_status = __salt__['service.enabled'](
+                name,
+                **kwargs)
+            # on upstart, certain services like apparmor will always return
+            # False, even if correctly activated
+            # do not trigger a change
+            if before_toggle_enable_status != after_toggle_enable_status:
+                ret['changes'][name] = True
+            if started is True:
+                ret['comment'] = ('Service {0} has been enabled,'
+                                  ' and is running').format(name)
+            elif started is None:
+                ret['comment'] = ('Service {0} has been enabled,'
+                                  ' and is in the desired state').format(name)
+            else:
+                ret['comment'] = ('Service {0} has been enabled,'
+                                  ' and is dead').format(name)
+            return ret
+    except CommandExecutionError as exc:
+        enable_error = exc.strerror
+    else:
+        enable_error = False
 
     # Service failed to be enabled
     ret['result'] = False
@@ -181,6 +212,12 @@ def _enable(name, started, result=True, **kwargs):
     else:
         ret['comment'] = ('Failed when setting service {0} to start at boot,'
                           ' and the service is dead').format(name)
+
+    if enable_error:
+        ret['comment'] += '. Additional information follows:\n\n{0}'.format(
+            enable_error
+        )
+
     return ret
 
 
@@ -218,7 +255,12 @@ def _disable(name, started, result=True, **kwargs):
         return ret
 
     # Service can be disabled
-    before_toggle_disable_status = __salt__['service.disabled'](name)
+    if salt.utils.is_windows():
+        # service.disabled in Windows returns True for services that are set to
+        # Manual start, so we need to check specifically for Disabled
+        before_toggle_disable_status = __salt__['service.info'](name)['StartType'] in ['Disabled']
+    else:
+        before_toggle_disable_status = __salt__['service.disabled'](name)
     if before_toggle_disable_status:
         # Service is disabled
         if started is True:
@@ -290,7 +332,14 @@ def _available(name, ret):
     return avail
 
 
-def running(name, enable=None, sig=None, init_delay=None, **kwargs):
+def running(name,
+            enable=None,
+            sig=None,
+            init_delay=None,
+            no_block=False,
+            unmask=False,
+            unmask_runtime=False,
+            **kwargs):
     '''
     Ensure that the service is running
 
@@ -298,9 +347,9 @@ def running(name, enable=None, sig=None, init_delay=None, **kwargs):
         The name of the init or rc script used to manage the service
 
     enable
-        Set the service to be enabled at boot time, True sets the service to
-        be enabled, False sets the named service to be disabled. The default
-        is None, which does not enable or disable anything.
+        Set the service to be enabled at boot time, ``True`` sets the service
+        to be enabled, ``False`` sets the named service to be disabled. The
+        default is ``None``, which does not enable or disable anything.
 
     sig
         The string to search for when looking for the service process with ps
@@ -313,6 +362,27 @@ def running(name, enable=None, sig=None, init_delay=None, **kwargs):
         for requisite states wherein a dependent state might assume a service
         has started but is not yet fully initialized.
 
+    no_block : False
+        **For systemd minions only.** Starts the service using ``--no-block``.
+
+        .. versionadded:: 2017.7.0
+
+    unmask : False
+        **For systemd minions only.** Set to ``True`` to remove an indefinite
+        mask before attempting to start the service.
+
+        .. versionadded:: 2017.7.0
+            In previous releases, Salt would simply unmask a service before
+            making any changes. This behavior is no longer the default.
+
+    unmask_runtime : False
+        **For systemd minions only.** Set to ``True`` to remove a runtime mask
+        before attempting to start the service.
+
+        .. versionadded:: 2017.7.0
+            In previous releases, Salt would simply unmask a service before
+            making any changes. This behavior is no longer the default.
+
     .. note::
         ``watch`` can be used with service.running to restart a service when
          another state changes ( example: a file.managed state that creates the
@@ -328,6 +398,10 @@ def running(name, enable=None, sig=None, init_delay=None, **kwargs):
     if 'enabled' in kwargs:
         return _enabled_used_error(ret)
 
+    # Convert enable to boolean in case user passed a string value
+    if isinstance(enable, six.string_types):
+        enable = salt.utils.is_true(enable)
+
     # Check if the service is available
     try:
         if not _available(name, ret):
@@ -364,7 +438,18 @@ def running(name, enable=None, sig=None, init_delay=None, **kwargs):
         if enable is True:
             ret.update(_enable(name, False, result=False, **kwargs))
 
-    func_ret = __salt__['service.start'](name)
+    # Conditionally add systemd-specific args to call to service.start
+    start_kwargs, warnings = \
+        _get_systemd_only(__salt__['service.start'], locals())
+    if warnings:
+        ret.setdefault('warnings', []).extend(warnings)
+
+    try:
+        func_ret = __salt__['service.start'](name, **start_kwargs)
+    except CommandExecutionError as exc:
+        ret['result'] = False
+        ret['comment'] = exc.strerror
+        return ret
 
     if not func_ret:
         ret['result'] = False
@@ -410,7 +495,11 @@ def running(name, enable=None, sig=None, init_delay=None, **kwargs):
     return ret
 
 
-def dead(name, enable=None, sig=None, **kwargs):
+def dead(name,
+         enable=None,
+         sig=None,
+         init_delay=None,
+         **kwargs):
     '''
     Ensure that the named service is dead by stopping the service if it is running
 
@@ -424,6 +513,17 @@ def dead(name, enable=None, sig=None, **kwargs):
 
     sig
         The string to search for when looking for the service process with ps
+
+    init_delay
+        Add a sleep command (in seconds) before the check to make sure service
+        is killed.
+
+        .. versionadded:: 2017.7.0
+
+    no_block : False
+        **For systemd minions only.** Stops the service using ``--no-block``.
+
+        .. versionadded:: 2017.7.0
     '''
     ret = {'name': name,
            'changes': {},
@@ -434,6 +534,10 @@ def dead(name, enable=None, sig=None, **kwargs):
     if 'enabled' in kwargs:
         return _enabled_used_error(ret)
 
+    # Convert enable to boolean in case user passed a string value
+    if isinstance(enable, six.string_types):
+        enable = salt.utils.is_true(enable)
+
     # Check if the service is available
     try:
         if not _available(name, ret):
@@ -450,7 +554,12 @@ def dead(name, enable=None, sig=None, **kwargs):
     # command, so it is just an indicator but can not be fully trusted
     before_toggle_status = __salt__['service.status'](name, sig)
     if 'service.enabled' in __salt__:
-        before_toggle_enable_status = __salt__['service.enabled'](name)
+        if salt.utils.is_windows():
+            # service.enabled in Windows returns True for services that are set
+            # to Auto start, but services set to Manual can also be disabled
+            before_toggle_enable_status = __salt__['service.info'](name)['StartType'] in ['Auto', 'Manual']
+        else:
+            before_toggle_enable_status = __salt__['service.enabled'](name)
     else:
         before_toggle_enable_status = True
 
@@ -469,7 +578,12 @@ def dead(name, enable=None, sig=None, **kwargs):
         ret['comment'] = 'Service {0} is set to be killed'.format(name)
         return ret
 
-    func_ret = __salt__['service.stop'](name)
+    # Conditionally add systemd-specific args to call to service.start
+    stop_kwargs, warnings = _get_systemd_only(__salt__['service.stop'], kwargs)
+    if warnings:
+        ret.setdefault('warnings', []).extend(warnings)
+
+    func_ret = __salt__['service.stop'](name, **stop_kwargs)
     if not func_ret:
         ret['result'] = False
         ret['comment'] = 'Service {0} failed to die'.format(name)
@@ -479,6 +593,9 @@ def dead(name, enable=None, sig=None, **kwargs):
             ret.update(_disable(name, True, result=False, **kwargs))
         return ret
 
+    if init_delay:
+        time.sleep(init_delay)
+
     # only force a change state if we have explicitly detected them
     after_toggle_status = __salt__['service.status'](name)
     if 'service.enabled' in __salt__:
@@ -544,6 +661,158 @@ def disabled(name, **kwargs):
     return ret
 
 
+def masked(name, runtime=False):
+    '''
+    .. versionadded:: 2017.7.0
+
+    .. note::
+        This state is only available on minions which use systemd_.
+
+    Ensures that the named service is masked (i.e. prevented from being
+    started).
+
+    name
+        Name of the service to mask
+
+    runtime : False
+        By default, this state will manage an indefinite mask for the named
+        service. Set this argument to ``True`` to runtime mask the service.
+
+    .. note::
+        It is possible for a service to have both indefinite and runtime masks
+        set for it. Therefore, this state will manage a runtime or indefinite
+        mask independently of each other. This means that if the service is
+        already indefinitely masked, running this state with ``runtime`` set to
+        ``True`` will _not_ remove the indefinite mask before setting a runtime
+        mask. In these cases, if it is desirable to ensure that the service is
+        runtime masked and not indefinitely masked, pair this state with a
+        :py:func:`service.unmasked <salt.states.service.unmasked>` state, like
+        so:
+
+        .. code-block:: yaml
+
+            mask_runtime_foo:
+              service.masked:
+                - name: foo
+                - runtime: True
+
+            unmask_indefinite_foo:
+              service.unmasked:
+                - name: foo
+                - runtime: False
+
+    .. _systemd: https://freedesktop.org/wiki/Software/systemd/
+
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'result': True,
+           'comment': ''}
+
+    if 'service.masked' not in __salt__:
+        ret['comment'] = 'Service masking not available on this minion'
+        ret['result'] = False
+        return ret
+
+    mask_type = 'runtime masked' if runtime else 'masked'
+    expected_changes = {mask_type: {'old': False, 'new': True}}
+
+    try:
+        if __salt__['service.masked'](name, runtime):
+            ret['comment'] = 'Service {0} is already {1}'.format(
+                name,
+                mask_type,
+            )
+            return ret
+
+        if __opts__['test']:
+            ret['result'] = None
+            ret['changes'] = expected_changes
+            ret['comment'] = 'Service {0} would be {1}'.format(name, mask_type)
+            return ret
+
+        __salt__['service.mask'](name, runtime)
+
+        if __salt__['service.masked'](name, runtime):
+            ret['changes'] = expected_changes
+            ret['comment'] = 'Service {0} was {1}'.format(name, mask_type)
+        else:
+            ret['comment'] = 'Failed to mask service {0}'.format(name)
+        return ret
+
+    except CommandExecutionError as exc:
+        ret['result'] = False
+        ret['comment'] = exc.strerror
+        return ret
+
+
+def unmasked(name, runtime=False):
+    '''
+    .. versionadded:: 2017.7.0
+
+    .. note::
+        This state is only available on minions which use systemd_.
+
+    Ensures that the named service is unmasked
+
+    name
+        Name of the service to unmask
+
+    runtime : False
+        By default, this state will manage an indefinite mask for the named
+        service. Set this argument to ``True`` to ensure that the service is
+        runtime masked.
+
+    .. note::
+        It is possible for a service to have both indefinite and runtime masks
+        set for it. Therefore, this state will manage a runtime or indefinite
+        mask independently of each other. This means that if the service is
+        indefinitely masked, running this state with ``runtime`` set to
+        ``True`` will _not_ remove the indefinite mask.
+
+    .. _systemd: https://freedesktop.org/wiki/Software/systemd/
+
+    '''
+    ret = {'name': name,
+           'changes': {},
+           'result': True,
+           'comment': ''}
+
+    if 'service.masked' not in __salt__:
+        ret['comment'] = 'Service masking not available on this minion'
+        ret['result'] = False
+        return ret
+
+    mask_type = 'runtime masked' if runtime else 'masked'
+    action = 'runtime unmasked' if runtime else 'unmasked'
+    expected_changes = {mask_type: {'old': True, 'new': False}}
+
+    try:
+        if not __salt__['service.masked'](name, runtime):
+            ret['comment'] = 'Service {0} was already {1}'.format(name, action)
+            return ret
+
+        if __opts__['test']:
+            ret['result'] = None
+            ret['changes'] = expected_changes
+            ret['comment'] = 'Service {0} would be {1}'.format(name, action)
+            return ret
+
+        __salt__['service.unmask'](name, runtime)
+
+        if not __salt__['service.masked'](name, runtime):
+            ret['changes'] = expected_changes
+            ret['comment'] = 'Service {0} was {1}'.format(name, action)
+        else:
+            ret['comment'] = 'Failed to unmask service {0}'.format(name)
+        return ret
+
+    except CommandExecutionError as exc:
+        ret['result'] = False
+        ret['comment'] = exc.strerror
+        return ret
+
+
 def mod_watch(name,
               sfun=None,
               sig=None,
@@ -627,7 +896,17 @@ def mod_watch(name,
         # stop service before start
         __salt__['service.stop'](name)
 
-    result = func(name)
+    func_kwargs, warnings = _get_systemd_only(func, kwargs)
+    if warnings:
+        ret.setdefault('warnings', []).extend(warnings)
+
+    try:
+        result = func(name, **func_kwargs)
+    except CommandExecutionError as exc:
+        ret['result'] = False
+        ret['comment'] = exc.strerror
+        return ret
+
     if init_delay:
         time.sleep(init_delay)
 
