@@ -7,18 +7,9 @@ from __future__ import absolute_import
 
 import tornado.ioloop
 import tornado.concurrent
-LOOP_CLASS = tornado.ioloop.IOLoop
-# attempt to use zmq-- if we have it otherwise fallback to tornado loop
-try:
-    import zmq.eventloop.ioloop
-    # support pyzmq 13.0.x, TODO: remove once we force people to 14.0.x
-    if not hasattr(zmq.eventloop.ioloop, 'ZMQIOLoop'):
-        zmq.eventloop.ioloop.ZMQIOLoop = zmq.eventloop.ioloop.IOLoop
-    LOOP_CLASS = zmq.eventloop.ioloop.ZMQIOLoop
-except ImportError:
-    pass  # salt-ssh doesn't dep zmq
 
 import contextlib
+from salt.utils import zeromq
 
 
 @contextlib.contextmanager
@@ -52,7 +43,7 @@ class SyncWrapper(object):
         if kwargs is None:
             kwargs = {}
 
-        self.io_loop = zmq.eventloop.ioloop.ZMQIOLoop()
+        self.io_loop = zeromq.ZMQDefaultLoop()
         kwargs['io_loop'] = self.io_loop
 
         with current_ioloop(self.io_loop):
@@ -61,8 +52,9 @@ class SyncWrapper(object):
     def __getattribute__(self, key):
         try:
             return object.__getattribute__(self, key)
-        except AttributeError:
-            pass
+        except AttributeError as ex:
+            if key == 'async':
+                raise ex
         attr = getattr(self.async, key)
         if hasattr(attr, '__call__'):
             def wrap(*args, **kwargs):
@@ -86,6 +78,15 @@ class SyncWrapper(object):
         '''
         On deletion of the async wrapper, make sure to clean up the async stuff
         '''
-        self.io_loop.close()
         if hasattr(self, 'async'):
+            if hasattr(self.async, 'close'):
+                # Certain things such as streams should be closed before
+                # their associated io_loop is closed to allow for proper
+                # cleanup.
+                self.async.close()
             del self.async
+            self.io_loop.close()
+            del self.io_loop
+        elif hasattr(self, 'io_loop'):
+            self.io_loop.close()
+            del self.io_loop
