@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
+    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
 
 
     salt.utils.nb_popen
@@ -25,11 +25,13 @@ import logging
 import tempfile
 import subprocess
 
-if subprocess.mswindows:
+mswindows = (sys.platform == "win32")
+
+try:
     from win32file import ReadFile, WriteFile
     from win32pipe import PeekNamedPipe
     import msvcrt
-else:
+except ImportError:
     import fcntl
 
 log = logging.getLogger(__name__)
@@ -88,20 +90,11 @@ class NonBlockingPopen(subprocess.Popen):
             self._stderr_logger_name_.format(pid=self.pid)
         )
 
-        if logging_command is None:
-            log.info(
-                'Running command under pid {0}: {1!r}'.format(
-                    self.pid,
-                    *args
-                )
-            )
-        else:
-            log.info(
-                'Running command under pid {0}: {1!r}'.format(
-                    self.pid,
-                    logging_command
-                )
-            )
+        log.info(
+            'Running command under pid %s: \'%s\'',
+            self.pid,
+            args if logging_command is None else logging_command
+        )
 
     def recv(self, maxsize=None):
         return self._recv('stdout', maxsize)
@@ -123,7 +116,7 @@ class NonBlockingPopen(subprocess.Popen):
         getattr(self, which).close()
         setattr(self, which, None)
 
-    if subprocess.mswindows:
+    if mswindows:
         def send(self, input):
             if not self.stdin:
                 return None
