@@ -5,7 +5,6 @@ Nova class
 
 # Import Python libs
 from __future__ import absolute_import, with_statement
-from distutils.version import LooseVersion
 import inspect
 import logging
 import time
@@ -39,12 +38,14 @@ except ImportError:
 # Import salt libs
 import salt.utils
 from salt.exceptions import SaltCloudSystemExit
+from salt.utils.versions import LooseVersion as _LooseVersion
 
 # Get logging started
 log = logging.getLogger(__name__)
 
 # Version added to novaclient.client.Client function
 NOVACLIENT_MINVER = '2.6.1'
+NOVACLIENT_MAXVER = '6.0.1'
 
 # dict for block_device_mapping_v2
 CLIENT_BDM2_KEYS = {
@@ -63,10 +64,14 @@ CLIENT_BDM2_KEYS = {
 
 def check_nova():
     if HAS_NOVA:
-        novaclient_ver = LooseVersion(novaclient.__version__)
-        min_ver = LooseVersion(NOVACLIENT_MINVER)
-        if novaclient_ver >= min_ver:
+        novaclient_ver = _LooseVersion(novaclient.__version__)
+        min_ver = _LooseVersion(NOVACLIENT_MINVER)
+        max_ver = _LooseVersion(NOVACLIENT_MAXVER)
+        if novaclient_ver >= min_ver and novaclient_ver <= max_ver:
             return HAS_NOVA
+        elif novaclient_ver > max_ver:
+            log.debug('Older novaclient version required. Maximum: {0}'.format(NOVACLIENT_MAXVER))
+            return False
         log.debug('Newer novaclient version required.  Minimum: {0}'.format(NOVACLIENT_MINVER))
     return False
 
@@ -186,6 +191,16 @@ def get_entry_multi(dict_, pairs, raise_error=True):
     return {}
 
 
+def get_endpoint_url_v3(catalog, service_type, region_name):
+    for service_entry in catalog:
+        if service_entry['type'] == service_type:
+            for endpoint_entry in service_entry['endpoints']:
+                if (endpoint_entry['region'] == region_name and
+                            endpoint_entry['interface'] == 'public'):
+                    return endpoint_entry['url']
+    return None
+
+
 def sanatize_novaclient(kwargs):
     variables = (
         'username', 'api_key', 'project_id', 'auth_url', 'insecure',
@@ -348,21 +363,16 @@ class SaltNova(object):
 
     def _v3_setup(self, region_name):
         if region_name is not None:
-            servers_endpoints = get_entry(self.catalog, 'type', 'compute')['endpoints']
-            self.kwargs['bypass_url'] = get_entry_multi(
-                servers_endpoints,
-                [('region', region_name), ('interface', 'public')]
-            )['url']
+            self.client_kwargs['bypass_url'] = get_endpoint_url_v3(self.catalog, 'compute', region_name)
+            log.debug('Using Nova bypass_url: %s', self.client_kwargs['bypass_url'])
 
         self.compute_conn = client.Client(version=self.version, session=self.session, **self.client_kwargs)
 
         volume_endpoints = get_entry(self.catalog, 'type', 'volume', raise_error=False).get('endpoints', {})
         if volume_endpoints:
             if region_name is not None:
-                self.kwargs['bypass_url'] = get_entry_multi(
-                    volume_endpoints,
-                    [('region', region_name), ('interface', 'public')]
-                )['url']
+                self.client_kwargs['bypass_url'] = get_endpoint_url_v3(self.catalog, 'volume', region_name)
+                log.debug('Using Cinder bypass_url: %s', self.client_kwargs['bypass_url'])
 
             self.volume_conn = client.Client(version=self.version, session=self.session, **self.client_kwargs)
             if hasattr(self, 'extensions'):
@@ -898,7 +908,7 @@ class SaltNova(object):
             try:
                 ret[item.name] = {
                     'id': item.id,
-                    'status': 'Running'
+                    'state': 'Running'
                 }
             except TypeError:
                 pass
@@ -1145,7 +1155,14 @@ class SaltNova(object):
         floating_ips = nt_ks.floating_ips.list()
         for floating_ip in floating_ips:
             if floating_ip.ip == ip:
-                return floating_ip
+                response = {
+                    'ip': floating_ip.ip,
+                    'fixed_ip': floating_ip.fixed_ip,
+                    'id': floating_ip.id,
+                    'instance_id': floating_ip.instance_id,
+                    'pool': floating_ip.pool
+                }
+                return response
         return {}
 
     def floating_ip_create(self, pool=None):
