@@ -13,6 +13,7 @@ import os
 import salt.utils
 
 # 3rd-party libs
+import salt.ext.six as six
 from ctypes import cdll, c_char_p, c_int, c_void_p, pointer, create_string_buffer
 from ctypes.util import find_library
 
@@ -34,11 +35,14 @@ def _load_libcrypto():
             'libcrypto.so*'))[0])
     else:
         lib = find_library('crypto')
-        if not lib and salt.utils.is_smartos():
-            # smartos does not have libraries in std location
-            lib = glob.glob(os.path.join(
-                '/opt/local/lib',
-                'libcrypto.so*'))
+        if not lib and salt.utils.is_sunos():
+            # Solaris-like distribution that use pkgsrc have
+            # libraries in a non standard location.
+            # (SmartOS, OmniOS, OpenIndiana, ...)
+            # This could be /opt/tools/lib (Global Zone)
+            # or /opt/local/lib (non-Global Zone), thus the
+            # two checks below
+            lib = glob.glob('/opt/local/lib/libcrypto.so*') + glob.glob('/opt/tools/lib/libcrypto.so*')
             lib = lib[0] if len(lib) > 0 else None
         if lib:
             return cdll.LoadLibrary(lib)
@@ -66,10 +70,9 @@ def _init_libcrypto():
     libcrypto.RSA_public_decrypt.argtypes = (c_int, c_char_p, c_char_p, c_void_p, c_int)
 
     try:
-        if libcrypto.OPENSSL_init_crypto(OPENSSL_INIT_NO_LOAD_CONFIG |
-                                         OPENSSL_INIT_ADD_ALL_CIPHERS |
-                                         OPENSSL_INIT_ADD_ALL_DIGESTS, None) != 1:
-            raise OSError("Failed to initialize OpenSSL library (OPENSSL_init_crypto failed)")
+        libcrypto.OPENSSL_init_crypto(OPENSSL_INIT_NO_LOAD_CONFIG |
+                                      OPENSSL_INIT_ADD_ALL_CIPHERS |
+                                      OPENSSL_INIT_ADD_ALL_DIGESTS, None)
     except AttributeError:
         # Support for OpenSSL < 1.1 (OPENSSL_API_COMPAT < 0x10100000L)
         libcrypto.OPENSSL_no_config()
@@ -132,6 +135,7 @@ class RSAX931Verifier(object):
         :param str pubdata: The RSA public key in PEM format
         '''
         pubdata = salt.utils.to_bytes(pubdata, 'ascii')
+        pubdata = pubdata.replace(six.b('RSA '), six.b(''))
         self._bio = libcrypto.BIO_new_mem_buf(pubdata, len(pubdata))
         self._rsa = c_void_p(libcrypto.RSA_new())
         if not libcrypto.PEM_read_bio_RSA_PUBKEY(self._bio, pointer(self._rsa), None, None):
