@@ -14,6 +14,7 @@ import tarfile
 import zipfile
 import tempfile
 import subprocess
+import concurrent
 
 # Import third party libs
 import jinja2
@@ -43,9 +44,8 @@ except ImportError:
 
 try:
     import backports_abc
-    HAS_BACKPORTS_ABC = True
 except ImportError:
-    HAS_BACKPORTS_ABC = False
+    import salt.ext.backports_abc as backports_abc
 
 try:
     import markupsafe
@@ -107,7 +107,10 @@ def get_tops(extra_mods='', so_mods=''):
             os.path.dirname(msgpack.__file__),
             ]
 
+    if _six.PY2:
+        tops.append(os.path.dirname(concurrent.__file__))
     tops.append(_six.__file__.replace('.pyc', '.py'))
+    tops.append(backports_abc.__file__.replace('.pyc', '.py'))
 
     if HAS_CERTIFI:
         tops.append(os.path.dirname(certifi.__file__))
@@ -118,9 +121,6 @@ def get_tops(extra_mods='', so_mods=''):
     if HAS_SINGLEDISPATCH_HELPERS:
         tops.append(singledispatch_helpers.__file__.replace('.pyc', '.py'))
 
-    if HAS_BACKPORTS_ABC:
-        tops.append(backports_abc.__file__.replace('.pyc', '.py'))
-
     if HAS_SSL_MATCH_HOSTNAME:
         tops.append(os.path.dirname(os.path.dirname(ssl_match_hostname.__file__)))
 
@@ -154,7 +154,8 @@ def get_tops(extra_mods='', so_mods=''):
 
 
 def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
-             python2_bin='python2', python3_bin='python3'):
+             python2_bin='python2', python3_bin='python3', absonly=True,
+             compress='gzip'):
     '''
     Generate the salt-thin tarball and print the location of the tarball
     Optional additional mods to include (e.g. mako) can be supplied as a comma
@@ -172,7 +173,11 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
     thindir = os.path.join(cachedir, 'thin')
     if not os.path.isdir(thindir):
         os.makedirs(thindir)
-    thintar = os.path.join(thindir, 'thin.tgz')
+    if compress == 'gzip':
+        thin_ext = 'tgz'
+    elif compress == 'zip':
+        thin_ext = 'zip'
+    thintar = os.path.join(thindir, 'thin.' + thin_ext)
     thinver = os.path.join(thindir, 'version')
     pythinver = os.path.join(thindir, '.thin-gen-py-version')
     salt_call = os.path.join(thindir, 'salt-call')
@@ -261,7 +266,10 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
             except ValueError:
                 pass
 
-    tfp = tarfile.open(thintar, 'w:gz', dereference=True)
+    if compress == 'gzip':
+        tfp = tarfile.open(thintar, 'w:gz', dereference=True)
+    elif compress == 'zip':
+        tfp = zipfile.ZipFile(thintar, 'w')
     try:  # cwd may not exist if it was removed but salt was run from it
         start_dir = os.getcwd()
     except OSError:
@@ -269,6 +277,8 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
     tempdir = None
     for py_ver, tops in _six.iteritems(tops_py_version_mapping):
         for top in tops:
+            if absonly and not os.path.isabs(top):
+                continue
             base = os.path.basename(top)
             top_dirname = os.path.dirname(top)
             if os.path.isdir(top_dirname):
@@ -283,25 +293,42 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
             if not os.path.isdir(top):
                 # top is a single file module
                 if os.path.exists(os.path.join(top_dirname, base)):
-                    tfp.add(base, arcname=os.path.join('py{0}'.format(py_ver), base))
+                    if compress == 'gzip':
+                        tfp.add(base, arcname=os.path.join('py{0}'.format(py_ver), base))
+                    elif compress == 'zip':
+                        tfp.write(base, arcname=os.path.join('py{0}'.format(py_ver), base))
                 continue
             for root, dirs, files in os.walk(base, followlinks=True):
                 for name in files:
                     if not name.endswith(('.pyc', '.pyo')):
-                        tfp.add(os.path.join(root, name),
-                                arcname=os.path.join('py{0}'.format(py_ver), root, name))
+                        if compress == 'gzip':
+                            tfp.add(os.path.join(root, name),
+                                    arcname=os.path.join('py{0}'.format(py_ver), root, name))
+                        elif compress == 'zip':
+                            try:
+                                # This is a little slow but there's no clear way to detect duplicates
+                                tfp.getinfo(os.path.join('py{0}'.format(py_ver), root, name))
+                            except KeyError:
+                                tfp.write(os.path.join(root, name), arcname=os.path.join('py{0}'.format(py_ver), root, name))
             if tempdir is not None:
                 shutil.rmtree(tempdir)
                 tempdir = None
     os.chdir(thindir)
-    tfp.add('salt-call')
+    if compress == 'gzip':
+        tfp.add('salt-call')
+    elif compress == 'zip':
+        tfp.write('salt-call')
     with salt.utils.fopen(thinver, 'w+') as fp_:
         fp_.write(salt.version.__version__)
     with salt.utils.fopen(pythinver, 'w+') as fp_:
         fp_.write(str(sys.version_info[0]))
     os.chdir(os.path.dirname(thinver))
-    tfp.add('version')
-    tfp.add('.thin-gen-py-version')
+    if compress == 'gzip':
+        tfp.add('version')
+        tfp.add('.thin-gen-py-version')
+    elif compress == 'zip':
+        tfp.write('version')
+        tfp.write('.thin-gen-py-version')
     if start_dir:
         os.chdir(start_dir)
     tfp.close()
@@ -510,6 +537,7 @@ def gen_min(cachedir, extra_mods='', overwrite=False, so_mods='',
         'salt/modules/test.py',
         'salt/modules/selinux.py',
         'salt/modules/cmdmod.py',
+        'salt/modules/saltutil.py',
         'salt/minion.py',
         'salt/pillar',
         'salt/pillar/__init__.py',
