@@ -4,15 +4,19 @@
 from __future__ import absolute_import
 import os
 import sys
+import tempfile
 import textwrap
 
 # Import Salt Testing libs
 from tests.support.case import ModuleCase
+from tests.support.unit import skipIf
 from tests.support.helpers import (
     destructiveTest,
     skip_if_binaries_missing,
-    skip_if_not_root
+    skip_if_not_root,
+    this_user,
 )
+from tests.support.paths import TMP
 
 # Import salt libs
 import salt.utils
@@ -20,7 +24,6 @@ import salt.utils
 # Import 3rd-party libs
 import salt.ext.six as six
 
-
 AVAILABLE_PYTHON_EXECUTABLE = salt.utils.which_bin([
     'python',
     'python2',
@@ -69,7 +72,7 @@ class CMDModuleTest(ModuleCase):
         '''
         self.assertEqual(self.run_function('cmd.run_stdout',
                                            ['echo "cheese"']).rstrip(),
-                         'cheese')
+                         'cheese' if not salt.utils.is_windows() else '"cheese"')
 
     def test_stderr(self):
         '''
@@ -84,7 +87,7 @@ class CMDModuleTest(ModuleCase):
                                            ['echo "cheese" 1>&2',
                                             'shell={0}'.format(shell)], python_shell=True
                                            ).rstrip(),
-                         'cheese')
+                         'cheese' if not salt.utils.is_windows() else '"cheese"')
 
     def test_run_all(self):
         '''
@@ -105,7 +108,7 @@ class CMDModuleTest(ModuleCase):
         self.assertTrue(isinstance(ret.get('retcode'), int))
         self.assertTrue(isinstance(ret.get('stdout'), six.string_types))
         self.assertTrue(isinstance(ret.get('stderr'), six.string_types))
-        self.assertEqual(ret.get('stderr').rstrip(), 'cheese')
+        self.assertEqual(ret.get('stderr').rstrip(), 'cheese' if not salt.utils.is_windows() else '"cheese"')
 
     def test_retcode(self):
         '''
@@ -139,6 +142,28 @@ class CMDModuleTest(ModuleCase):
         ret = self.run_function('cmd.script_retcode', [script])
         self.assertEqual(ret, 0)
 
+    def test_script_cwd(self):
+        '''
+        cmd.script with cwd
+        '''
+        tmp_cwd = tempfile.mkdtemp(dir=TMP)
+        args = 'saltines crackers biscuits=yes'
+        script = 'salt://script.py'
+        ret = self.run_function('cmd.script', [script, args], cwd=tmp_cwd)
+        self.assertEqual(ret['stdout'], args)
+
+    def test_script_cwd_with_space(self):
+        '''
+        cmd.script with cwd
+        '''
+        tmp_cwd = "{0}{1}test 2".format(tempfile.mkdtemp(dir=TMP), os.path.sep)
+        os.mkdir(tmp_cwd)
+
+        args = 'saltines crackers biscuits=yes'
+        script = 'salt://script.py'
+        ret = self.run_function('cmd.script', [script, args], cwd=tmp_cwd)
+        self.assertEqual(ret['stdout'], args)
+
     @destructiveTest
     def test_tty(self):
         '''
@@ -193,10 +218,13 @@ class CMDModuleTest(ModuleCase):
         '''
         cmd = '''echo 'SELECT * FROM foo WHERE bar="baz"' '''
         expected_result = 'SELECT * FROM foo WHERE bar="baz"'
+        if salt.utils.is_windows():
+            expected_result = '\'SELECT * FROM foo WHERE bar="baz"\''
         result = self.run_function('cmd.run_stdout', [cmd]).strip()
         self.assertEqual(result, expected_result)
 
     @skip_if_not_root
+    @skipIf(salt.utils.is_windows, 'skip windows, requires password')
     def test_quotes_runas(self):
         '''
         cmd.run with quoted command
@@ -204,17 +232,13 @@ class CMDModuleTest(ModuleCase):
         cmd = '''echo 'SELECT * FROM foo WHERE bar="baz"' '''
         expected_result = 'SELECT * FROM foo WHERE bar="baz"'
 
-        try:
-            runas = os.getlogin()
-        except:  # pylint: disable=W0702
-            # On some distros (notably Gentoo) os.getlogin() fails
-            import pwd
-            runas = pwd.getpwuid(os.getuid())[0]
+        runas = this_user()
 
         result = self.run_function('cmd.run_stdout', [cmd],
                                    runas=runas).strip()
         self.assertEqual(result, expected_result)
 
+    @skipIf(not salt.utils.which_bin('sleep'), 'sleep cmd not installed')
     def test_timeout(self):
         '''
         cmd.run trigger timeout
@@ -225,6 +249,7 @@ class CMDModuleTest(ModuleCase):
                                 python_shell=True)
         self.assertTrue('Timed out' in out)
 
+    @skipIf(not salt.utils.which_bin('sleep'), 'sleep cmd not installed')
     def test_timeout_success(self):
         '''
         cmd.run sufficient timeout to succeed
@@ -234,3 +259,13 @@ class CMDModuleTest(ModuleCase):
                                 f_timeout=2,
                                 python_shell=True)
         self.assertEqual(out, 'hello')
+
+    def test_cmd_run_whoami(self):
+        '''
+        test return of whoami
+        '''
+        cmd = self.run_function('cmd.run', ['whoami'])
+        if salt.utils.is_windows():
+            self.assertIn('administrator', cmd)
+        else:
+            self.assertEqual('root', cmd)
