@@ -5,11 +5,13 @@ from __future__ import absolute_import
 import random
 import string
 from copy import deepcopy
+import os.path
 
 # Import Salt Testing Libs
 from tests.support.mixins import LoaderModuleMockMixin
 from tests.support.unit import skipIf, TestCase
 from tests.support.mock import NO_MOCK, NO_MOCK_REASON
+from tests.support.paths import TESTS_DIR
 
 # Import Salt libs
 import salt.config
@@ -23,6 +25,7 @@ import salt.modules.boto_secgroup as boto_secgroup
 from salt.ext.six.moves import range  # pylint: disable=redefined-builtin
 try:
     import boto
+    boto.ENDPOINTS_PATH = os.path.join(TESTS_DIR, 'unit/files/endpoints.json')
     import boto.ec2  # pylint: enable=unused-import
     HAS_BOTO = True
 except ImportError:
@@ -207,7 +210,7 @@ class BotoSecgroupTestCase(TestCase, LoaderModuleMockMixin):
         from_port = 22
         to_port = 22
         cidr_ip = u'0.0.0.0/0'
-        rules_egress = [{'to_port': -1, 'from_port': -1, 'ip_protocol': u'-1', 'cidr_ip': u'0.0.0.0/0'}]
+        rules_egress = [{'to_port': None, 'from_port': None, 'ip_protocol': u'-1', 'cidr_ip': u'0.0.0.0/0'}]
 
         conn = boto.ec2.connect_to_region(region, **boto_conn_parameters)
         group = conn.create_security_group(name=group_name, description=group_name)
