@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    :codeauthor: :email:`Rajvi Dhimar <rajvidhimar95@gmail.com>`
+    :codeauthor: Rajvi Dhimar <rajvidhimar95@gmail.com>
 '''
 # Import python libs
 from __future__ import absolute_import, print_function
@@ -20,6 +20,8 @@ try:
     from jnpr.junos.utils.config import Config
     from jnpr.junos.utils.sw import SW
     from jnpr.junos.device import Device
+    from jnpr.junos.device import Device
+    import jxmlease  # pylint: disable=unused-import
     HAS_JUNOS = True
 except ImportError:
     HAS_JUNOS = False
@@ -28,7 +30,7 @@ except ImportError:
 import salt.modules.junos as junos
 
 
-@skipIf(not HAS_JUNOS, 'Install junos-eznc to be able to run this test.')
+@skipIf(not HAS_JUNOS, 'The junos-eznc and jxmlease modules are required')
 class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def setup_loader_modules(self):
@@ -52,6 +54,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 host='1.1.1.1',
                 user='test',
                 password='test123',
+                fact_style='old',
                 gather_facts=False)
             self.dev.open()
             self.dev.timeout = 30
@@ -500,7 +503,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                     'confirm': 2, '__pub_fun': 'junos.rollback',
                     '__pub_jid': '20170221184518526067', '__pub_tgt': 'mac_min',
                     '__pub_tgt_type': 'glob', '__pub_ret': ''}
-            junos.rollback(2, **args)
+            junos.rollback(id=2, **args)
             mock_rollback.assert_called_with(2)
             mock_commit.assert_called_with(confirm=2)
 
@@ -570,7 +573,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                 patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback, \
-                patch('salt.modules.junos.fopen') as mock_fopen, \
+                patch('salt.utils.fopen') as mock_fopen, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
             mock_commit_check.return_value = True
             mock_diff.return_value = 'diff'
@@ -591,7 +594,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.commit') as mock_commit, \
                 patch('jnpr.junos.utils.config.Config.rollback') as mock_rollback, \
-                patch('salt.modules.junos.fopen') as mock_fopen, \
+                patch('salt.utils.fopen') as mock_fopen, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
             mock_commit_check.return_value = True
             mock_diff.return_value = None
@@ -645,7 +648,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def test_diff_with_arg(self):
         with patch('jnpr.junos.utils.config.Config.diff') as mock_diff:
-            junos.diff(2)
+            junos.diff(id=2)
             mock_diff.assert_called_with(rb_id=2)
 
     def test_diff_exception(self):
@@ -701,7 +704,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def test_cli_with_format_as_empty_string(self):
         with patch('jnpr.junos.device.Device.cli') as mock_cli:
-            junos.cli('show version', '')
+            junos.cli('show version', format='')
             mock_cli.assert_called_with('show version', 'text', warning=False)
 
     def test_cli(self):
@@ -744,7 +747,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
             self.assertEqual(junos.cli('show version'), ret)
 
     def test_cli_write_output(self):
-        with patch('salt.modules.junos.fopen') as mock_fopen, \
+        with patch('salt.utils.fopen') as mock_fopen, \
                 patch('jnpr.junos.device.Device.cli') as mock_cli:
             mock_cli.return_vale = 'cli text output'
             args = {'__pub_user': 'root',
@@ -874,8 +877,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -895,8 +898,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -916,8 +919,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -937,8 +940,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -970,8 +973,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1003,8 +1006,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1032,8 +1035,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
     def test_install_config_load_causes_exception(self):
         with patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1051,8 +1054,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
     def test_install_config_no_diff(self):
         with patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1069,10 +1072,10 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
-                patch('salt.modules.junos.fopen') as mock_fopen, \
+                patch('salt.utils.fopen') as mock_fopen, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
             mock_getsize.return_value = 10
@@ -1104,10 +1107,10 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
-                patch('salt.modules.junos.fopen') as mock_fopen, \
+                patch('salt.utils.fopen') as mock_fopen, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
             mock_getsize.return_value = 10
@@ -1140,8 +1143,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1173,8 +1176,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1192,8 +1195,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
         with patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1212,8 +1215,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.utils.config.Config.commit_check') as mock_commit_check, \
                 patch('jnpr.junos.utils.config.Config.diff') as mock_diff, \
                 patch('jnpr.junos.utils.config.Config.load') as mock_load, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_isfile.return_value = True
@@ -1274,8 +1277,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def test_install_os(self):
         with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_getsize.return_value = 10
@@ -1288,8 +1291,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
     def test_install_os_with_reboot_arg(self):
         with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
                 patch('jnpr.junos.utils.sw.SW.reboot') as mock_reboot, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_getsize.return_value = 10
@@ -1305,8 +1308,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
 
     def test_install_os_pyez_install_throws_exception(self):
         with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_getsize.return_value = 10
@@ -1320,8 +1323,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
     def test_install_os_with_reboot_raises_exception(self):
         with patch('jnpr.junos.utils.sw.SW.install') as mock_install, \
                 patch('jnpr.junos.utils.sw.SW.reboot') as mock_reboot, \
-                patch('salt.modules.junos.safe_rm') as mock_safe_rm, \
-                patch('salt.modules.junos.files.mkstemp') as mock_mkstemp, \
+                patch('salt.utils.safe_rm') as mock_safe_rm, \
+                patch('salt.utils.files.mkstemp') as mock_mkstemp, \
                 patch('os.path.isfile') as mock_isfile, \
                 patch('os.path.getsize') as mock_getsize:
             mock_getsize.return_value = 10
@@ -1489,8 +1492,8 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
             mock_execute.return_value = etree.XML(
                 '<rpc-reply>text rpc reply</rpc-reply>')
             m = mock_open()
-            with patch('salt.modules.junos.fopen', m, create=True):
-                junos.rpc('get-chassis-inventory', '/path/to/file', 'text')
+            with patch('salt.utils.fopen', m, create=True):
+                junos.rpc('get-chassis-inventory', '/path/to/file', format='text')
                 handle = m()
                 handle.write.assert_called_with('text rpc reply')
 
@@ -1499,7 +1502,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('salt.modules.junos.json.dumps') as mock_dumps:
             mock_dumps.return_value = 'json rpc reply'
             m = mock_open()
-            with patch('salt.modules.junos.fopen', m, create=True):
+            with patch('salt.utils.fopen', m, create=True):
                 junos.rpc('get-chassis-inventory', '/path/to/file', format='json')
                 handle = m()
                 handle.write.assert_called_with('json rpc reply')
@@ -1510,7 +1513,7 @@ class Test_Junos_Module(TestCase, LoaderModuleMockMixin, XMLEqualityMixin):
                 patch('jnpr.junos.device.Device.execute') as mock_execute:
             mock_tostring.return_value = 'xml rpc reply'
             m = mock_open()
-            with patch('salt.modules.junos.fopen', m, create=True):
+            with patch('salt.utils.fopen', m, create=True):
                 junos.rpc('get-chassis-inventory', '/path/to/file')
                 handle = m()
                 handle.write.assert_called_with('xml rpc reply')
